SystemOrganization addCategory: #'Zinc-HTTP-Client-Server'!
SystemOrganization addCategory: #'Zinc-HTTP-Core'!
SystemOrganization addCategory: #'Zinc-HTTP-Deprecated'!
SystemOrganization addCategory: #'Zinc-HTTP-Exceptions'!
SystemOrganization addCategory: #'Zinc-HTTP-Logging'!
SystemOrganization addCategory: #'Zinc-HTTP-Streaming'!
SystemOrganization addCategory: #'Zinc-HTTP-Support'!

!HierarchicalUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
asZnUrl
	^ ZnUrl new
		scheme: self scheme;
		host: self authority;
		port: self port;
		segments: (self path collect: [ :each | each isEmpty ifTrue: [ #/ ] ifFalse: [ each ] ]);
		query: (self query isNil ifTrue: [ nil ] ifFalse: [ ZnUtils parseQueryFrom: self query readStream ]);
		fragment: self fragment;
		yourself! !

DynamicVariable subclass: #ZnConnectionTimeout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnConnectionTimeout commentStamp: 'SvenVanCaekenberghe 5/9/2012 20:54' prior: 0!
I am ZnConnectionTimeout.
I am a DynamicVariable and a ProcessSpecificVariable.

I can be used to modify the global default socket IO timeout on a per process basis, for example:

ZnConnectionTimeout 
	value: 5 
	during: [ ^ ZnClient new get: 'http://zn.stfx.eu/zn/small.html' ]!

Error subclass: #ZnEntityTooLarge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnEntityTooLarge commentStamp: 'SvenVanCaekenberghe 1/31/2012 12:35' prior: 0!
ZnEntityTooLarge is signalled when too big an entity is read from a stream.

This is a resumable exception!

!ZnEntityTooLarge methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/31/2012 12:35'!
isResumable
	"Determine whether an exception is resumable."

	^ true! !

Error subclass: #ZnHttpUnsuccessful
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnHttpUnsuccessful commentStamp: 'SvenVanCaekenberghe 12/20/2011 12:52' prior: 0!
ZnHttpUnsuccessful is signalled when an HTTP client receives a non success response.

See ZnResponse>>#isSuccess

I hold the response that was unsuccessful.

Part of Zinc HTTP Components. !

!ZnHttpUnsuccessful class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/20/2011 12:51'!
response: response
	^ self new
		response: response;
		yourself! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:53'!
messageText
	"Overwritten to initialiaze the message text to a standard text if it has not yet been set"
	
	^ messageText ifNil: [ messageText := self standardMessageText ]! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:50'!
response
	^ response! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:50'!
response: anObject
	response := anObject! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:56'!
standardMessageText
	"Generate a standard textual description"
	
	^ String streamContents: [ :stream |
		response ifNotNil: [
			response statusLine printCodeAndReasonOn: stream ] ]! !

Error subclass: #ZnMissingHost
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnMissingHost commentStamp: 'SvenVanCaekenberghe 8/11/2011 19:37' prior: 0!
ZnMissingHost is signalled when a host should be specified but was not.

Part of Zinc HTTP Components. !

Error subclass: #ZnParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnParseError commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:13' prior: 0!
ZnParseError is the superclass of all Zinc HTTP parsing related errors.

Part of Zinc HTTP Components.!

ZnParseError subclass: #ZnLineTooLong
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnLineTooLong commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:15' prior: 0!
ZnLineTooLong is signalled when an HTTP CRLF terminated input line exceeds a predefined limit.

Part of Zinc HTTP Components. !

!ZnLineTooLong class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
limit: limit
	^ self new 
		limit: limit; 
		yourself! !

!ZnLineTooLong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:32'!
limit
	^ limit! !

!ZnLineTooLong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:32'!
limit: anObject
	limit := anObject! !

ZnParseError subclass: #ZnPercentEncodingWrong
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnPercentEncodingWrong commentStamp: 'SvenVanCaekenberghe 2/28/2011 15:50' prior: 0!
ZnPercentEncodingWrong is signalled when a wrong percent encoding is encountered.

Part of Zinc HTTP Components.!

!ZnPercentEncodingWrong class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 15:50'!
string: string
	^ self new
		string: string;
		yourself ! !

!ZnPercentEncodingWrong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:49'!
string
	^ string! !

!ZnPercentEncodingWrong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:49'!
string: anObject
	string := anObject! !

ZnParseError subclass: #ZnPortNotANumber
	instanceVariableNames: 'port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnPortNotANumber commentStamp: 'SvenVanCaekenberghe 2/28/2011 15:23' prior: 0!
ZnPortNotANumber is signalled when an URL is parsed containing an illegal port specification.

Part of Zinc HTTP Components.!

!ZnPortNotANumber class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: port
	^ self new
		port: port;
		yourself! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port
	^ port! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: anObject
	port := anObject! !

ZnParseError subclass: #ZnUnknownHttpMethod
	instanceVariableNames: 'method'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownHttpMethod commentStamp: 'SvenVanCaekenberghe 12/11/2011 18:09' prior: 0!
ZnUnknownHttpMethod is signalled when an unknown/unsupported HTTP method is found while parsing a request.

I am resumable to allow non-standard methods.

Part of Zinc HTTP Components. !

!ZnUnknownHttpMethod class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
method: method
	^ self new
		method: method;
		yourself! !

!ZnUnknownHttpMethod methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2011 18:09'!
isResumable
	^ true! !

!ZnUnknownHttpMethod methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
method
	^ method! !

!ZnUnknownHttpMethod methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
method: anObject
	method := anObject! !

ZnParseError subclass: #ZnUnknownHttpStatusCode
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownHttpStatusCode commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownHttpStatusCode is signalled when an unknown/unsupported HTTP status code is found.

Part of Zinc HTTP Components. !

!ZnUnknownHttpStatusCode class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
code: code
	^ self new
		code: code;
		yourself! !

!ZnUnknownHttpStatusCode methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
code
	^ code! !

!ZnUnknownHttpStatusCode methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
code: anObject
	code := anObject! !

ZnParseError subclass: #ZnUnknownHttpVersion
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownHttpVersion commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownHttpVersion is signalled when an unknown/unsupported HTTP version is found.

Part of Zinc HTTP Components. !

!ZnUnknownHttpVersion class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:35'!
version: version
	^ self new
		version: version;
		yourself! !

!ZnUnknownHttpVersion methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
version
	^ version! !

!ZnUnknownHttpVersion methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
version: anObject
	version := anObject! !

ZnParseError subclass: #ZnUnknownScheme
	instanceVariableNames: 'scheme'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownScheme commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownScheme is signalled when an unknown/unsupported URL scheme is found.

Part of Zinc HTTP Components. !

!ZnUnknownScheme class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/18/2011 13:17'!
scheme: scheme
	^ self new
		scheme: scheme;
		yourself! !

!ZnUnknownScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme
	^ scheme! !

!ZnUnknownScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme: anObject
	scheme := anObject! !

Error subclass: #ZnTooManyDictionaryEntries
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnTooManyDictionaryEntries commentStamp: 'SvenVanCaekenberghe 1/2/2012 19:22' prior: 0!
ZnTooManyDictionaryEntries is signalled when too many entries are added to a limited ZnMultiValueDictionary.

Part of Zinc HTTP Components. !

!ZnTooManyDictionaryEntries class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit: limit
	^ self new 
		limit: limit; 
		yourself! !

!ZnTooManyDictionaryEntries methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit
	^ limit! !

!ZnTooManyDictionaryEntries methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit: anObject
	limit := anObject! !

Error subclass: #ZnTooManyRedirects
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnTooManyRedirects commentStamp: 'SvenVanCaekenberghe 9/19/2011 09:59' prior: 0!
ZnTooManyRedirects is signalled when an HTTP client has been following more redirects than allowed.

Part of Zinc HTTP Components. !

!ZnTooManyRedirects methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2012 15:06'!
isResumable
	"Determine whether an exception is resumable."

	^ true! !

Error subclass: #ZnUnexpectedContentType
	instanceVariableNames: 'expectedContentType actualContentType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnexpectedContentType commentStamp: 'SvenVanCaekenberghe 12/20/2011 12:52' prior: 0!
ZnUnexpectedContentType is signalled when a different content type (mime type) was expected.

I hold the expected and actual content types.

Part of Zinc HTTP Components. !

!ZnUnexpectedContentType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/20/2011 12:48'!
expected: expectedMimeType actual: actualMimeType
	^ self new
		expectedContentType: expectedMimeType;
		actualContentType: actualMimeType;
		yourself! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
actualContentType
	^ actualContentType! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
actualContentType: anObject
	actualContentType := anObject! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
expectedContentType
	^ expectedContentType! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
expectedContentType: anObject
	expectedContentType := anObject! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:45'!
messageText
	"Overwritten to initialiaze the message text to a standard text if it has not yet been set"
	
	^ messageText ifNil: [ messageText := self standardMessageText ]! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:43'!
standardMessageText
	"Generate a standard textual description"
	
	^ String streamContents: [ :stream |
		self expectedContentType ifNotNil: [ :contentType |
			stream << 'expected '; print: contentType ].
		self actualContentType ifNotNil: [ :contentType |
			self expectedContentType ifNotNil: [ stream space ].
			stream << 'actual '; print: contentType ] ]! !

!String methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ ZnMimeType fromString: self! !

!String methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 13:03'!
asZnUrl
	^ ZnUrl fromString: self! !

Announcement subclass: #ZnLogEvent
	instanceVariableNames: 'timeStamp processId category message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogEvent commentStamp: 'SvenVanCaekenberghe 3/29/2011 13:27' prior: 0!
I am ZnLogEvent, an Announcement sent by an HTTP server or client containing logging information.

A log event has a TimeStamp, a Process ID, a category and a message.
The following categories are used: #info (I), #debug (D) and #transaction (T).  

Part of Zinc HTTP Components.!

!ZnLogEvent class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/28/2011 22:07'!
category: symbol message: string
	^ self new
		category: symbol;
		message: string;
		yourself    ! !

!ZnLogEvent class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/28/2011 22:08'!
debug: message
	^ self category: #debug message: message    ! !

!ZnLogEvent class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/28/2011 22:08'!
info: message
	^ self category: #info message: message    ! !

!ZnLogEvent class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/29/2011 09:26'!
transaction: message
	^ self category: #transaction message: message    ! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 22:05'!
category
	^ category! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:51'!
category: symbol
	category := symbol! !

!ZnLogEvent methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 12:51'!
initialize 
	self timeStamp: TimeStamp now.
	self category: #info.  
	self processId: ZnUtils currentProcessID     ! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 22:05'!
message
	^ message! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
message: string
	message := string! !

!ZnLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:51'!
printOn: stream
	self timeStamp printYMDOn: stream.
	stream space.
	self timeStamp printHMSOn: stream.
	stream space.
	self processId printOn: stream base: 10 length: 6 padded: true.
	stream space; nextPut: self category first asUppercase.
	self message isNil ifFalse: [ stream space; nextPutAll: self message ]! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:48'!
processId 
	^ processId ! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:48'!
processId: integer
	processId := integer ! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 22:05'!
timeStamp
	^ timeStamp! !

!ZnLogEvent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 09:49'!
timeStamp: aTimeStamp
	timeStamp := aTimeStamp! !

!MIMEType methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 1/4/2011 19:54'!
asZnMimeType
	^ ZnMimeType main: self main sub: self
! !

Stream subclass: #ZnBivalentWriteStream
	instanceVariableNames: 'stream binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnBivalentWriteStream commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:50' prior: 0!
I am ZnBivalentWriteStream, wrapping another write stream.

I allow both Characters and Strings as well as Integers (0 to: 255) and ByteArrays to be writtten to me.

The write stream that I wrap can be both binary as well as character based.

Part of Zinc HTTP Components.!

!ZnBivalentWriteStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:44'!
on: writeStream
	"Wrap writeStream as a ZnBivalentWriteStream, unless it is already one"
	
	(writeStream isKindOf: self) ifTrue: [ ^ writeStream ].
	
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
close
	stream close! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/4/2012 16:06'!
flush
	stream flush! !

!ZnBivalentWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 20:54'!
isBinary
	binary isNil
		ifTrue: [ [ binary := stream isBinary ] on: MessageNotUnderstood do: [ :err | binary := false ] ].
	^ binary! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 20:46'!
next: count putAll: collection startingAt: offset
	self isBinary
		ifTrue: [ stream next: count putAll: collection asByteArray startingAt: offset ]
		ifFalse: [ stream next: count putAll: collection asString startingAt: offset ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPut: object
	self isBinary
		ifTrue: [ stream nextPut: object asInteger ]
		ifFalse: [ stream nextPut: object asCharacter ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPutAll: collection
	self isBinary
		ifTrue: [ stream nextPutAll: collection asByteArray ]
		ifFalse: [ stream nextPutAll: collection asString ]! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
on: writeStream
	stream := writeStream! !

!ZnBivalentWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:08'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:12'!
space
	self nextPut: Character space! !

Stream subclass: #ZnBufferedWriteStream
	instanceVariableNames: 'stream buffer position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnBufferedWriteStream commentStamp: 'SvenVanCaekenberghe 12/8/2010 10:10' prior: 0!
I am ZnBufferedWriteStream.
I wrap a write stream and add buffering.

Make sure to always send me #flush or #close when you're done,
otherwise the last buffer might not yet have been written.

Part of Zinc HTTP Components.!

!ZnBufferedWriteStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/7/2010 21:20'!
on: writeStream
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBufferedWriteStream class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/9/2010 10:19'!
on: writeStream do: block
	"Execute block with as argument a ZnBufferedWriteStream on writeStream,
	making sure #flush is called at the end."
	
	| bufferedWriteStream |
	bufferedWriteStream := self on: writeStream.
	block value: bufferedWriteStream.
	bufferedWriteStream flush! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/8/2010 10:15'!
buffer
	buffer isNil 
		ifTrue: [ self sizeBuffer: self defaultBufferSize ].
	^ buffer! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 23:42'!
bufferFreeSize
	^ self bufferSize - position! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2010 10:08'!
bufferSize
	^ buffer isNil 
		ifTrue: [ self defaultBufferSize ]
		ifFalse: [ buffer size ]! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 21:22'!
close
	self flushBuffer.
	stream close! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:24'!
defaultBufferSize
	^ 4096! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:23'!
flush
	self flushBuffer.
	stream flush! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/7/2010 23:48'!
flushBuffer
	position = 0 ifTrue: [ ^ self ].
	position = self bufferSize
		ifTrue: [
			stream nextPutAll: buffer ]
		ifFalse: [
			(stream respondsTo: #next:putAll:startingAt:)
				ifTrue: [ stream next: position putAll: buffer startingAt: 1 ]
				ifFalse: [ stream nextPutAll: (buffer copyFrom: 1 to: position) ] ].
	position := 0! !

!ZnBufferedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/7/2010 23:39'!
flushBufferIfFull
	position = self bufferSize 
		ifTrue: [ self flushBuffer ]
! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/8/2010 10:16'!
nextPut: object
	self flushBufferIfFull.
	position := position + 1.
	self buffer at: position put: object! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 20:44'!
nextPutAll: collection
	self flushBufferIfFull.
	collection size <= self bufferFreeSize
		ifTrue: [
			self buffer replaceFrom: position + 1 to: position + collection size with: collection.
			position := position + collection size ]
		ifFalse: [
			self flushBuffer.
			collection size > (self bufferSize / 2)
				ifTrue: [ stream nextPutAll: collection ]
				ifFalse: [ self nextPutAll: collection ] ]
! !

!ZnBufferedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 21:30'!
on: writeStream
	stream := writeStream.
	position := 0! !

!ZnBufferedWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:19'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:28'!
sizeBuffer: size
	buffer := (stream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: size! !

!ZnBufferedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 21:20'!
space
	self nextPut: Character space! !

Stream subclass: #ZnChunkedReadStream
	instanceVariableNames: 'stream chunk position limit atEnd totalSize extraHeaders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnChunkedReadStream commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:01' prior: 0!
I am ZnChunkedReadStream, implementing HTTP 1.1 chunked transfer encoding on a wrapped streams.

Clients should read me until I am atEnd.

After I am completely read, I can tell you my totalSize and optional extraHeaders.

Part of Zinc HTTP Components.!

!ZnChunkedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 14:10'!
new
	^ super new initialize! !

!ZnChunkedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 19:43'!
on: stream
	^ self basicNew
		initialize;
		on: stream;
		yourself! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
atEnd
	self ensureChunkOrAtEnd.
	^ atEnd! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 08:11'!
binary
	stream binary! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
collectionSpecies
	^ self isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]
	! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 16:02'!
contents
	"This is technically not correct, but it is better than nothing"
	
	^ self upToEnd! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/19/2011 12:39'!
ensureChunkBufferOfSize: size
	(chunk notNil and: [ size <= chunk size ]) ifTrue: [ ^ self ].  
	chunk := self collectionSpecies new: size.
! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/19/2011 12:45'!
ensureChunkOrAtEnd
	atEnd 
		ifTrue: [ ^ self ].
	(chunk isNil or: [ position >= limit ]) 
		ifTrue: [ self getNextChunk ]
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:21'!
extraHeaders
	^ extraHeaders! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 17:08'!
getExtraHeaders
	extraHeaders := ZnHeaders readFrom: stream! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/19/2011 12:45'!
getNextChunk
	| size |
	(size := self getNextChunkSize) = 0
		ifTrue: [
			atEnd := true.
			chunk := position := limit := nil.
			self getExtraHeaders ]
		ifFalse: [
			totalSize := totalSize + size.
			self ensureChunkBufferOfSize: size.  
			stream next: size into: chunk.
			stream next: 2. "consume CRLF"
			limit := size.
			position := 0 ]! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:26'!
getNextChunkSize
	| line |
	(line := (ZnLineReader on: stream) nextLine) isEmpty ifTrue: [ ^ 0 ].
	^ Integer readFrom: (line copyUpTo: $;) base: 16! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:11'!
initialize
	atEnd := false.
	totalSize := 0! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
isBinary
	^ stream isBinary
! !

!ZnChunkedReadStream methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 5/2/2012 16:06'!
match: subCollection
	"Try to read and match the elements of subCollection. 
	If successful return true and leave the me positioned after the match.
	If unsuccesful return false and leave me at end."
	
	| buffer pattern bufferPosition bufferLimit |
	pattern := subCollection readStream.
	"we have to use an internal buffer because we are not positionable"
	buffer := subCollection species new: subCollection size.
	bufferPosition := bufferLimit := 0.
	[ pattern atEnd ] whileFalse: [ | nextElement |
		self atEnd ifTrue: [ ^ false ].
		"get the next char to match either from the buffer or from the stream while buffering it"
		(bufferPosition < bufferLimit) 
			ifTrue: [ 
				nextElement := buffer at: bufferPosition + 1 ]
			ifFalse: [
				nextElement := buffer at: bufferPosition + 1 put: self next.
				bufferLimit := bufferLimit + 1 ].
		bufferPosition := bufferPosition + 1.
		pattern next = nextElement
			ifFalse: [
				"shift the buffer down one element and restart"
				2 to: bufferLimit do: [ :each |
					buffer at: each - 1 put: (buffer at: each) ].
				bufferLimit := bufferLimit - 1.
				bufferPosition := 0.
				pattern position: 0 ] ].
	^ true! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
next
	self ensureChunkOrAtEnd.
	self atEnd ifTrue: [ ^ nil ].
	^ chunk at: (position := position + 1)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:11'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:11'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:11'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:13'!
on: readStream
	stream := readStream! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/19/2011 12:36'!
peek
	self ensureChunkOrAtEnd.
	self atEnd ifTrue: [ ^ nil ].
	^ chunk at: (position + 1)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 15:30'!
position
	^ self atEnd 
		ifTrue: [ totalSize ]
		ifFalse: [ totalSize - chunk size + position ]! !

!ZnChunkedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 14:00'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset.
	Return the actual number of elements read."

	| read |
	read := 0.
	[ read < requestedCount and: [ self atEnd not] ] whileTrue: [
		self ensureChunkOrAtEnd.
		collection replaceFrom: read + 1 to: read + limit with: chunk startingAt: 1.
		read := read + limit.
		position := position + limit ].
	^ read! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
reset
	"We don't allow it"! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:10'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:20'!
totalSize
	^ totalSize! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 13:15'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	^ self collectionSpecies streamContents: [ :out | | element |
		[ self atEnd or: [ (element := self next) = anObject ] ]
			whileFalse: [ out nextPut: element ] ]! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 12:33'!
upToEnd
	"We use out own collectionSpecies."

	^ self collectionSpecies streamContents: [ :out |
		[ self atEnd ] whileFalse: [ 
			out nextPut: self next ] ]! !

Stream subclass: #ZnLimitedReadStream
	instanceVariableNames: 'stream position limit binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnLimitedReadStream commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:02' prior: 0!
I am ZnLimitedReadStream, wrapping another read stream delegating to it 
but limiting reading to a fixed number of elements.

I can be atEnd while my wrapped stream is not.

My contentSpecies can be forced to be ByteArray (binary) or ByteString (ascii).

I do byte/char conversions on the fly (support bivalent access).

This implementation is really a kludge.

Part of Zinc HTTP Components.!

!ZnLimitedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 11:15'!
on: stream limit: limit
	^ self basicNew
		on: stream limit: limit;
		yourself! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:38'!
ascii
	binary := false! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:03'!
atEnd
	^ position >= limit or: [ stream atEnd ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:36'!
binary
	binary := true! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:34'!
collectionSpecies
	"I can be set to binary or ascii explicitely.
	If not set, delegate to our wrapped stream."
	
	^ binary notNil
		ifTrue: [ 
			binary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ]
		ifFalse: [
			(stream respondsTo: #collectionSpecies)
				ifTrue: [ stream collectionSpecies ]  
				ifFalse: [ stream isBinary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 16:02'!
contents
	"This is technically not correct, but it is better than nothing"
	
	^ self upToEnd! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:17'!
contentsOfEntireFile
	^ self upToEnd! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:06'!
isBinary
	^ binary isNil
		ifTrue: [ stream isBinary ]
		ifFalse: [ binary ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 15:55'!
match: subCollection
	"Try to read and match the elements of subCollection. 
	If successful return true and leave the me positioned after the match.
	If unsuccesful return false and leave me at end."
	
	| buffer pattern bufferPosition bufferLimit |
	pattern := subCollection readStream.
	"we have to use an internal buffer because we are not positionable"
	buffer := subCollection species new: subCollection size.
	bufferPosition := bufferLimit := 0.
	[ pattern atEnd ] whileFalse: [ | nextElement |
		self atEnd ifTrue: [ ^ false ].
		"get the next char to match either from the buffer or from the stream while buffering it"
		(bufferPosition < bufferLimit) 
			ifTrue: [ 
				nextElement := buffer at: bufferPosition + 1 ]
			ifFalse: [
				nextElement := buffer at: bufferPosition + 1 put: self next.
				bufferLimit := bufferLimit + 1 ].
		bufferPosition := bufferPosition + 1.
		pattern next = nextElement
			ifFalse: [
				"shift the buffer down one element and restart"
				2 to: bufferLimit do: [ :each |
					buffer at: each - 1 put: (buffer at: each) ].
				bufferLimit := bufferLimit - 1.
				bufferPosition := 0.
				pattern position: 0 ] ].
	^ true! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/21/2011 23:19'!
next
	"We do byte/char conversions on the fly."
	
	^ self atEnd 
		ifTrue: [ nil ] 
		ifFalse: [ 
			position := position + 1. 
			binary isNil
				ifTrue: [ stream next ] 
				ifFalse: [ 
					binary
					 	ifTrue: [ stream next asInteger ]
						ifFalse: [ stream next asCharacter ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:14'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:11'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:14'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!ZnLimitedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 13:06'!
on: readStream limit: integer
	stream := readStream.
	limit := integer.
	position := 0! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/4/2011 18:43'!
peek
	"We do byte/char conversions on the fly."
	
	^ self atEnd 
		ifTrue: [ nil ] 
		ifFalse: [ 
			binary isNil
				ifTrue: [ stream peek ] 
				ifFalse: [ 
					binary
					 	ifTrue: [ stream peek asInteger ]
						ifFalse: [ stream peek asCharacter ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:09'!
position
	^ position! !

!ZnLimitedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/15/2010 12:50'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:06'!
readInto: collection startingAt: offset count: count 
	"Read count elements and place them in collection starting at offset.
	Return the number of elements actually read."
	
	| target actual |
	target := count min: (limit - position).
	actual := stream 
		readInto: collection 
		startingAt: offset 
		count: target.
	position := position + actual.
	^ actual! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:20'!
reset
	"We don't allow it"! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:03'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:02'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	^ self collectionSpecies streamContents: [ :out | | element |
		[ self atEnd or: [ (element := self next) = anObject ] ] whileFalse: [ 
			out nextPut: element ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:07'!
upToEnd
	^ ZnUtils readUpToEnd: self limit: nil! !

Dictionary subclass: #ZnMultiValueDictionary
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnMultiValueDictionary commentStamp: 'SvenVanCaekenberghe 1/2/2012 19:41' prior: 0!
I am ZnMultiValueDictionary. 
I am a Dictionary.

I offer an #at:add: method to transparently create Array valued multi entries when needed.
My #keysAndValuesDo: is overwritten to transparently deal with multi entries.
I normally limit the number of entries to protect me from resource abuse.

Note that most other methods will show the actual value.

Part of Zinc HTTP Components.!

!ZnMultiValueDictionary methodsFor: 'adding' stamp: 'SvenVanCaekenberghe 1/2/2012 19:44'!
add: anAssociation
	"Overridden to signal an exception when the limit, if present, is exceeded."
	
	| index element |
	index := self findElementOrNil: anAssociation key.
	element := array at: index.
	element == nil
		ifTrue: [
			(self limit notNil and: [ self size >= self limit ])
				ifTrue: [ (ZnTooManyDictionaryEntries limit: self limit) signal ].
			self atNewIndex: index put: anAssociation ]
		ifFalse: [
			element value: anAssociation value ].
	^ anAssociation! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:19'!
at: key add: value
	"Store value under key. If key already exists, store multiple values as Array"
	
	| existingValue newValue |
	existingValue := self at: key ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [
			existingValue isArray 
				ifTrue: [ existingValue copyWith: value ]
				ifFalse: [ Array with: existingValue with: value ] ].
	^ self at: key put: newValue 

! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:40'!
at: key put: anObject
	"Set the value at key to be anObject.  If key is not found, create a
	new entry for key and set is value to anObject. Answer anObject.
	Overridden to signal an exception when the limit, if present, is exceeded."

	| index assoc |
	index := self findElementOrNil: key.
	assoc := array at: index.
	assoc
		ifNil: [
			(self limit notNil and: [ self size >= self limit ])
				ifTrue: [ (ZnTooManyDictionaryEntries limit: self limit) signal ].
			self atNewIndex: index put: (Association key: key value: anObject) ]
		ifNotNil: [
			assoc value: anObject ].
	^ anObject! !

!ZnMultiValueDictionary methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/12/2011 11:10'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	super keysAndValuesDo: [ :key :value |
		value isArray
			ifTrue: [ 
				value do: [ :each | 
					block value: key value: each ] ]
			ifFalse: [
				block value: key value: value ] ]! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:35'!
limit
	^ limit ifNil: [ limit := ZnConstants maximumNumberOfDictionaryEntries ]! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 19:35'!
limit: numberOfEntries
	limit := numberOfEntries ! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 19:36'!
unlimited
	self limit: nil! !

Object subclass: #ZnBasicAuthenticator
	instanceVariableNames: 'credentials realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicAuthenticator commentStamp: 'SvenVanCaekenberghe 9/30/2010 15:37' prior: 0!
I am ZnBasicAuthenticator.
I help servers handle HTTP Basic Authentication.
I implement the required #authenticateRequest:do:

I have a credentials dictionary with username->password associations.

Part of Zinc HTTP Components.!

!ZnBasicAuthenticator class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
defaultRealm
	^ 'ZincHTTPComponents'! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
new
	^ super new 
		initialize;
		yourself! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
username: username password: password
	^ self new
		atUsername: username putPassword: password;
		yourself! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:55'!
username: username password: password realm: realm
	^ self new
		atUsername: username putPassword: password;
		realm: realm;
		yourself! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 17:19'!
atUsername: username putPassword: password
	credentials at: username put: password! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
authHeader
	^ 'Basic realm=', self realm! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 15:56'!
authenticateRequest: request do: block
	^ (self isRequestAuthenticated: request)
		ifTrue: [ block value ]
		ifFalse: [ ZnResponse unauthorized: self authHeader ]! !

!ZnBasicAuthenticator methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 15:37'!
initialize
	credentials := Dictionary new! !

!ZnBasicAuthenticator methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:13'!
isRequestAuthenticated: request
	| authorization username password |
	authorization := [ request basicAuthentication ] on: Error do: [ ^ false ].
	username := authorization first.
	password := authorization second.
	^ (credentials at: username ifAbsent: [ nil ]) = password
	! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:02'!
realm
	^ realm ifNil: [ realm := self class defaultRealm ]! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:51'!
realm: anObject
	realm := anObject! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeAll
	credentials removeAll! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeUsername: username
	credentials removeKey: username! !

ZnBasicAuthenticator subclass: #ZnDigestAuthenticator
	instanceVariableNames: 'nonces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestAuthenticator commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:47' prior: 0!
I am ZnDigestAuthenticator.
I help servers handle HTTP Digest Authentication.

I have a nonces dictionary with nonce->opaque associations. 
I generate a new nonce value for every 401 response I trigger.

Part of Zinc HTTP Components.!

!ZnDigestAuthenticator class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
hasMD5Support
	^ (self md5Hash: 'test') isNil not! !

!ZnDigestAuthenticator class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:27'!
md5Hash: aString
	| hash |
	"Answer hash of aString as lowercase 32 digit hex String."
	Smalltalk at: #MD5 ifPresent: [ :cls |
		^ ((cls new initialize) hashStream: (aString asByteArray readStream)) hex ].
	hash := Smalltalk at: #CMD5Hasher ifPresent: [ :cls |
		cls hashMessage: aString ].
	hash ifNil: [
		hash := Smalltalk at: #TCryptoRandom ifPresent: [ :cls |
			(cls basicNew perform: #md5HashMessage: with: aString) asInteger ] ].
	hash ifNotNil: [
		hash := hash hex asLowercase.
		(hash beginsWith: '16r') ifTrue: [ hash := hash allButFirst: 3 ].
		hash := hash padded: #left to: 32 with: $0 ].
	^ hash
! !

!ZnDigestAuthenticator class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/22/2012 10:21'!
parseAuthRequest: headerValue
	| data dict |
	dict := Dictionary new.
	"Chops off the ' Digest ' scheme name, a bit too brittle."
	data := headerValue copyFrom: 8 to: headerValue size.
	(data subStrings: ',') do: [ :fragment | | tokens i key value |
		tokens := fragment trimBoth.
		i := tokens indexOf: $=.
		(i > 0) ifTrue: [
			key := tokens copyFrom: 1 to: i - 1.
			value := tokens copyFrom: i + 1 to: tokens size.  
			dict at: key put: (value trimBoth: [ :char | char = $" ]) ] ].
	^ dict! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:03'!
a1for: username
	| password |
	password := credentials at: username ifAbsent: [ ^ nil ].
	^ self class md5Hash: username, ':', self realm, ':', password
	! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
a2forUrl: uri method: method
	^ self class md5Hash: method, ':', uri
	! !

!ZnDigestAuthenticator methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
authHeader
	| nonce opaque |
	nonce := self createNonce.
	opaque := self createOpaque.
	self nonces at: nonce put: opaque.
	^ 'Digest realm="', self realm, '", nonce="', nonce, '", opaque="', opaque, '"'! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
createNonce
	| pt |
	pt := TimeStamp current asString, ':', Random new next asString.
	^ self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
createOpaque
	| pt |
	pt := Random new next asString, ':', TimeStamp current asString.
	^ self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
isRequestAuthenticated: request
	| authorization response cresponse a1 a2 nonce opaque |
	authorization := self class parseAuthRequest: (request headers at: 'Authorization' ifAbsent: [ ^ false ]).
	nonce := authorization at: 'nonce' ifAbsent: [ ^ false ].
	opaque := self nonces at: nonce ifAbsent: [ ^ false ].
	a1 := self a1for: (authorization at: 'username' ifAbsent: [ ^ false ]).
	a1 ifNil: [ ^ false ].
	a2 := self a2forUrl: (authorization at: 'uri' ifAbsent: [ '/' ]) method: request method.
	response := self class md5Hash: (a1, ':', nonce, ':', a2).
	"(opaque = (authorization at: 'opaque' ifAbsent: [^false])) ifFalse: [^false]."
	cresponse := authorization at: 'response' ifAbsent: [ ^ false ].
	^ (response = cresponse)! !

!ZnDigestAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
nonces 
	^ nonces ifNil: [ nonces := Dictionary new ]! !

Object subclass: #ZnCharacterEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnCharacterEncoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:52' prior: 0!
I am ZnCharacterEncoder, I encode and decode Character objects to and from a binary stream.
I am an abstract class with following protocol:

#nextFromStream:
#nextPut:toStream:
#encodedByteCountFor:

The first two are compatible with TextConverter and subclasses.

Part of Zinc HTTP Components.!

ZnCharacterEncoder subclass: #ZnByteEncoder
	instanceVariableNames: 'byteToUnicode unicodeToByte'
	classVariableNames: 'ByteTextConverters'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnByteEncoder commentStamp: 'SvenVanCaekenberghe 1/25/2011 11:02' prior: 0!
I am ZnByteEncoder, a concrete subclass of ZnCharacterEncoder.
I handle single byte encodings where byte values 0 to 127 map to ASCII
and 128 to 255 are a permutation to Unicode characters.

I am reusing the specifications from the subclasses of ByteTextEncoder.

Part of Zinc HTTP Components.!

!ZnByteEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 13:02'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ ByteTextConverters includesKey: string! !

!ZnByteEncoder class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 1/25/2011 12:26'!
initialize
	self initializeByteTextConverters! !

!ZnByteEncoder class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/3/2012 11:34'!
initializeByteTextConverters
	(ByteTextConverters := Dictionary new)
		at: 'cp-1250' put: CP1250TextConverter;
		at: 'cp-1253' put: CP1253TextConverter;
		at: 'iso-8859-2' put: ISO88592TextConverter; 
		at: 'latin-2' put: ISO88592TextConverter; 
		at: 'latin2' put: ISO88592TextConverter; 
		at: 'iso-8859-7' put: ISO88597TextConverter;
		at: 'iso-8859-15' put: ISO885915TextConverter
		! !

!ZnByteEncoder class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/25/2011 12:47'!
initializeEncoder: encoder using: byteTextConverterClass
	"We are reusing the tables specified, computed and cached in subclasses of ByteTextConverter"
	
	encoder byteToUnicode: byteTextConverterClass byteToUnicodeTable.
	encoder unicodeToByte: byteTextConverterClass unicodeToByteTable! !

!ZnByteEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 12:44'!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	Search for a subclass that handles it and delegate"
	
	| encoder byteTextConverterClass |
	encoder := self new.
	byteTextConverterClass := ByteTextConverters at: string.
	self initializeEncoder: encoder using: byteTextConverterClass.
	^ encoder! !

!ZnByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/25/2011 12:31'!
byteToUnicode: map
	byteToUnicode := map! !

!ZnByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 12:19'!
encodedByteCountFor: character
	^ 1! !

!ZnByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 13:22'!
nextFromStream: stream
	| byteValue |
	^ (byteValue := stream next) < 128
		ifTrue: [ Character codePoint: byteValue ]
		ifFalse: [ | ch |
			"In the standard Squeak table some strange leading char was added"
			ch := byteToUnicode at: byteValue - 127.
			ch asUnicodeChar ]! !

!ZnByteEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 13:13'!
nextPut: character toStream: stream
	| code |
	(code := character charCode) < 128
		ifTrue: [ stream nextPut: code ]
		ifFalse: [ | ch |
			"In the standard Squeak table the byte values were converted to characters"
			ch := unicodeToByte at: code ifAbsent: [ self error: 'Character code outside encoder range' ].
			stream nextPut: ch charCode ]! !

!ZnByteEncoder methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/25/2011 12:31'!
unicodeToByte: map
	unicodeToByte := map! !

!ZnCharacterEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 11:07'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 13:27'!
newForEncoding: string
	"Return a new character encoder object for an encoding described by string.
	Search for a subclass that handles it and delegate (subclassResponsibility).
	We default to ZnNullEncoder."
	
	| concreteSubclass encoding |
	encoding := string asLowercase.
	concreteSubclass := self subclasses 
		detect: [ :each | each handlesEncoding: encoding ] 
		ifNone: [ ZnNullEncoder ].
	^ concreteSubclass newForEncoding: encoding! !

!ZnCharacterEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:56'!
= anObject
	^ self class == anObject class! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/2/2012 16:12'!
decodeBytes: bytes
	| byteStream |
	byteStream := bytes readStream.
	^ String streamContents: [ :stream |
		[ byteStream atEnd ] whileFalse: [
			stream nextPut: (self nextFromStream: byteStream) ] ] ! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/2/2012 16:10'!
encodeString: string
	^ ByteArray streamContents: [ :stream |
		string do: [ :each | 
			self nextPut: each toStream: stream ] ] ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:17'!
encodedByteCountFor: character
	"Return how many bytes are needed to encode character"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/2/2012 16:36'!
encodedByteCountForString: string
	^ string 
		inject: 0 
		into: [ :sum :each |
			sum + (self encodedByteCountFor: each) ]! !

!ZnCharacterEncoder methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:19'!
hash
	^ self class name hash! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:16'!
nextFromStream: stream
	"Read and return the next character from stream"
	
	self subclassResponsibility ! !

!ZnCharacterEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:16'!
nextPut: character toStream: stream
	"Write the encoding for character to stream"
	
	self subclassResponsibility ! !

ZnCharacterEncoder subclass: #ZnNullEncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnNullEncoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:53' prior: 0!
I am ZnNullEncoder, a concrete subclass of ZnCharacterEncoder.
I perform no encoding or decoding at all for all characters with a code value below 256.
I can only be used for ASCII and Latin1 (ISO-8859-1).

Part of Zinc HTTP Components.!

!ZnNullEncoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 11:11'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ #( 'ascii' 'latin1' 'latin-1' 'iso-8859-1' ) includes: string! !

!ZnNullEncoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 11:19'!
newForEncoding: string
	"No further parametrization needed"
	
	^ self new! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/3/2012 11:32'!
decodeBytes: bytes
	^ bytes asString! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/3/2012 11:32'!
encodeString: string
	^ string asByteArray! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:17'!
encodedByteCountFor: character
	^ 1! !

!ZnNullEncoder methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/3/2012 11:32'!
encodedByteCountForString: string
	^ string size! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 20:52'!
nextFromStream: stream
	^ Character codePoint: stream next! !

!ZnNullEncoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/25/2011 12:36'!
nextPut: character toStream: stream
	| code |
	(code := character charCode) < 256
		ifTrue: [ stream nextPut: code ]
		ifFalse: [ self error: 'Character code outside encoder range' ]! !

ZnCharacterEncoder subclass: #ZnUTF8Encoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnUTF8Encoder commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:55' prior: 0!
I am ZnUTF8Encoder, a concrete subclass of ZnCharacterEncoder.
I implement the variable length UTF-8 encoding and decoding of Unicode according to RFC 3629.

Part of Zinc HTTP Components.!

!ZnUTF8Encoder class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2011 11:11'!
handlesEncoding: string
	"Return true when my instances handle the encoding described by string"
	
	^ #( 'utf-8' 'utf8' ) includes: string! !

!ZnUTF8Encoder class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2011 11:19'!
newForEncoding: string
	"No further parametrization needed"
	
	^ self new! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/29/2010 21:23'!
encodedByteCountFor: character
	| code |
	code := character charCode.
	code < 128 ifTrue: [ ^ 1 ].
	code < 2048 ifTrue: [ ^ 2 ].
	code < 65535 ifTrue: [ ^ 3 ].
	code <= self maximumUTF8Code ifTrue: [ ^ 4 ].
	self error: 'Unicode character code point outside range'
! !

!ZnUTF8Encoder methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/29/2010 21:19'!
maximumUTF8Code
	^ 16r10FFFF! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 20:52'!
nextFromStream: stream
	| code byte next |
	next := [ | nextByte |
		((nextByte := stream next) bitAnd: 2r11000000) = 2r10000000
			ifTrue: [ code := (code bitShift: 6) + (nextByte bitAnd: 2r00111111) ]
			ifFalse: [ self error: 'illegal UTF-8 encoding' ] ].
	(byte := stream next) < 128 ifTrue: [
		^ Character codePoint: byte ].
	(byte bitAnd: 2r11100000) = 2r11000000 ifTrue: [
		code := byte bitAnd: 2r00011111.
		next value.
		^ Character codePoint: code ].
	(byte bitAnd: 2r11110000) = 2r11100000 ifTrue: [
		code := byte bitAnd: 2r00001111.
		next value; value.
		^ Character codePoint: code ].
	(byte bitAnd: 2r11111000) = 2r11110000 ifTrue: [
		code := byte bitAnd: 2r00000111.
		next value; value; value.
		^ Character codePoint: code ].
	self error: 'illegal UTF-8 encoding'! !

!ZnUTF8Encoder methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/30/2010 13:19'!
nextPut: character toStream: stream
	| code next |
	code := character charCode.
	next := [ :shift |
		stream nextPut: (2r10000000 + ((code bitShift: shift) bitAnd: 2r111111)) ].
	code < 128 ifTrue: [ 
		stream nextPut: code.
		^ character ].
	code < 2048 ifTrue: [ 
		stream nextPut: (2r11000000 + (code bitShift: -6)).
		next value: 0.
		^ character ].
	code < 65535 ifTrue: [ 
		stream nextPut: (2r11100000 + (code bitShift: -12)).
		next value: -6; value: 0.
		^ character ].
	code <= self maximumUTF8Code ifTrue: [ 
		stream nextPut: (2r11110000 + (code bitShift: -18)).
		next value: -12; value: -6; value: 0.
		^ character ].
	self error: 'Unicode character code point outside range'
! !

Object subclass: #ZnCharacterReadStream
	instanceVariableNames: 'stream encoder peekedCharacter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnCharacterReadStream commentStamp: 'SvenVanCaekenberghe 5/3/2012 21:31' prior: 0!
I am ZnCharacterReadStream.
I wrap another binary ReadStream and use a ZnCharacerEncoder so allow Characters to be read.

I am not positionable, but I do allow a one character peek.

Part of Zinc HTTP Components.
!

!ZnCharacterReadStream class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 14:43'!
defaultEncoder
	^ ZnUTF8Encoder new! !

!ZnCharacterReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/3/2012 14:40'!
on: readStream
	^ self new
		on: readStream;
		yourself! !

!ZnCharacterReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/3/2012 15:13'!
on: readStream encoding: encoding
	^ self new
		on: readStream;
		encoding: encoding;
		yourself! !

!ZnCharacterReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/3/2012 21:29'!
atEnd
	^ peekedCharacter isNil and: [ stream atEnd ]! !

!ZnCharacterReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 14:39'!
close
	stream close! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:09'!
collectionSpecies
	^ String! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:12'!
contents
	"This is technically not correct, but it is better than nothing"
	
	^ self upToEnd! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 14:42'!
encoder
	^ encoder ifNil: [ encoder := self class defaultEncoder ]! !

!ZnCharacterReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 14:41'!
encoder: characterEncoder
	encoder := characterEncoder! !

!ZnCharacterReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 14:40'!
encoding: encoding
	encoder := ZnCharacterEncoder forEncoding: encoding! !

!ZnCharacterReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:07'!
isBinary
	^ false! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:46'!
next
	^ peekedCharacter
		ifNil: [ 
			stream atEnd
				ifFalse: [ self encoder nextFromStream: stream ] ]
		ifNotNil: [ | character |
			character := peekedCharacter.
			peekedCharacter := nil.
			character ]! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:18'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:18'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:19'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!ZnCharacterReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 14:40'!
on: readStream
	stream := readStream! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:35'!
peek
	^ peekedCharacter
		ifNil: [
			stream atEnd
				ifFalse: [ 
					peekedCharacter := self encoder nextFromStream: stream ] ]
! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:57'!
readInto: collection startingAt: offset count: requestedCount 
	"Read count elements and place them in collection starting at offset.
	Return the number of elements actually read.
	This is an inefficient implementation, reading characters one by one."
	
	0 to: requestedCount - 1 do: [ :count | | object |
		(object := self next) ifNil: [ ^ count ].  
		collection at: offset + count put: object ].
	^ requestedCount! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:11'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:17'!
upTo: anObject 	
	^ self collectionSpecies 
		streamContents: [ :out | | element |
			[ self atEnd or: [ (element := self next) = anObject ] ] whileFalse: [ 
				out nextPut: element ] ]! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:11'!
upToEnd
	^ ZnUtils readUpToEnd: self limit: nil! !

!ZnCharacterReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 14:41'!
wrappedStream
	^ stream! !

Object subclass: #ZnCharacterWriteStream
	instanceVariableNames: 'stream encoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnCharacterWriteStream commentStamp: 'SvenVanCaekenberghe 5/3/2012 15:50' prior: 0!
I am ZnCharacterWriteStream.
I wrap another binary WriteStream and use a ZnCharacerEncoder so allow Characters to be written.

Part of Zinc HTTP Components.!

!ZnCharacterWriteStream class methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 5/3/2012 15:50'!
defaultEncoder
	^ ZnUTF8Encoder new! !

!ZnCharacterWriteStream class methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 5/3/2012 15:51'!
on: writeStream
	^ self new
		on: writeStream;
		yourself! !

!ZnCharacterWriteStream class methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 5/3/2012 15:51'!
on: writeStream encoding: encoding
	^ self new
		on: writeStream;
		encoding: encoding;
		yourself! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:59'!
<< collection
	^ self nextPutAll: collection! !

!ZnCharacterWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 15:51'!
close
	stream close! !

!ZnCharacterWriteStream methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 5/3/2012 18:47'!
encoder
	^ encoder ifNil: [ encoder := self class defaultEncoder ]! !

!ZnCharacterWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 15:51'!
encoder: characterEncoder
	encoder := characterEncoder! !

!ZnCharacterWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 15:51'!
encoding: encoding
	encoder := ZnCharacterEncoder forEncoding: encoding! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:52'!
flush
	stream flush! !

!ZnCharacterWriteStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:52'!
isBinary
	^ false! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:54'!
next: count putAll: collection	
	self 
		next: count 
		putAll: collection 
		startingAt: 1! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:57'!
next: count putAll: collection startingAt: offset
	"Write count characters from collection starting at offset.
	This is an inefficient implementation, writing characters one by one."
	
	0 to: count - 1 do: [ :each | 
		self nextPut: (collection at: offset + each) ]! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:56'!
nextPut: object
	self encoder 
		nextPut: object 
		toStream: stream.
	^ object! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:56'!
nextPutAll: collection
	self 
		next: collection size 
		putAll: collection 
		startingAt: 1! !

!ZnCharacterWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/3/2012 15:51'!
on: writeStream
	stream := writeStream! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 15:52'!
space
	self nextPut: Character space! !

!ZnCharacterWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/3/2012 18:47'!
wrappedStream
	^ stream! !

Object subclass: #ZnClient
	instanceVariableNames: 'request response connection lastUsed state options session log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnClient commentStamp: 'SvenVanCaekenberghe 11/8/2011 22:31' prior: 0!
I am ZnClient, an object to build, execute and process HTTP client requests.

I have a rich protocol to construct requests and to access responses.
I have various error handling options.
I can reuse an existing connection to a specific host:port.
I handle sessions, cookies, redirects and authentication.
I have many options (settings) with sensible defaults.

Simplest possible invocation:

	ZnClient new
		get: 'http://zn.stfx.eu/zn/numbers.txt'.
	
Using some features to make a better HTTP request:

	ZnClient new
		systemPolicy;
		accept: ZnMimeType textPlain;
		http;
		host: 'zn.stfx.eu';
		path: 'zn/numbers.txt';
		contentReader: [ :entity | entity contents lines collect: [ :each | each asNumber ] ];
		ifFail: [ :exception | self inform: 'I am sorry: ', exception printString ];
		get.

Part of Zinc HTTP Components.!

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 16:55'!
accept: mimeTypeObject
	"Set the Accept HTTP header to mimeTypeObject for the current request."

	| mimeType |
	mimeType := mimeTypeObject asZnMimeType.
	request setAccept: mimeType! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:18'!
addPart: mimePart
	"Add mimePart to the multipart/form-data entity of the current request"

	self multiPartFormDataEntity 
		addPart: mimePart ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:03'!
addPath: object
	"Extend the path of the current request with object.
	Either a string or a collection of strings is allowed as argument."
	
	object isString
		ifTrue: [ self addPathSegment: object ] 
		ifFalse: [ object do: [ :each | self addPathSegment: each ] ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:03'!
addPathSegment: pathSegment
	"Extend the path of the current request with a single pathSegment."
	
	request url 
		ifNil: [ request url: (ZnUrl new addPathSegment: pathSegment; yourself) ]
		ifNotNil: [ request url addPathSegment: pathSegment ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:30'!
applicationFormUrlEncodedEntity
	"Initialize the current request to have an application/x-www-form-urlencoded entity.
	Return the existing entity if there is one. 
	Fail if another type was already set."
	
	request hasEntity
		ifTrue: [
			request entity contentType = ZnMimeType applicationFormUrlEncoded 
				ifFalse: [ self error: 'An incompatible entity was already set for the request' ] ]
		ifFalse: [
			request entity: ZnApplicationFormUrlEncodedEntity new ].
	^ request entity! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:34'!
beOneShot
	"Switch me to use connections only for a single request/response cycle.
	By default, I will keep HTTP connections open according to the HTTP 1.1 spec.
	When oneShot is true, I will add a Connection: close header to requests."
	
	self optionAt: #oneShot put: true   ! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2011 11:49'!
canReuseConnection
	self isConnected ifFalse: [ ^ false ].
	lastUsed ifNil: [ ^ false ].
	^ TimeStamp now - lastUsed < self connectionReuseTimeout seconds
	! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:10'!
cleanupConnection
	(self isOneShot or: [ request notNil and: [ request wantsConnectionClose ] ])
		ifTrue: [
			self close ]! !

!ZnClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 2/29/2012 16:40'!
close
	"Close any HTTP network connection that I might have kept open."
	
	connection ifNotNil: [ 
		connection close.
		connection := nil.
		state := #closed ].
	lastUsed := nil! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:36'!
connectionReuseTimeout
	"Return the timeout in seconds currently used to determine
	if I should try to reuse an open connection. Defaults to 10 seconds."

	^ self optionAt: #connectionReuseTimeout ifAbsent: [ 10 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:30'!
connectionReuseTimeout: seconds
	"Set the timeout used to determine if I should try 
	to reuse an open connection to seconds."

	self optionAt: #connectionReuseTimeout put: seconds! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:41'!
contentReader
	"Return the current content reader, an object conforming 
	to the #value: protocol to convert a response entity
	as used by #contents. Defaults to nil.
	Can be used to parse incoming data."
	
	^ self optionAt: #contentReader ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:30'!
contentReader: object
	"Set the content reader, an object conforming 
	to the #value: protocol to convert a response entity
	as used by #contents to object.
	Can be used to parse incoming data."

	self optionAt: #contentReader put: object! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 5/9/2012 09:21'!
contentType: mimeType
	"Set the content-type of the request entity, provided we have one"
	
	request hasEntity
		ifTrue: [ 
			request entity contentType: mimeType asZnMimeType ]
		ifFalse: [ 
			self error: 'There is no request entity yet, cannot set its content-type' ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:42'!
contentWriter
	"Return the current content writer, an object conforming 
	to the #value: protocol to convert an object to a request content entity
	as used by #contents:. Defaults to nil.
	Can be used to format data to be uploaded."

	^ self optionAt: #contentWriter ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
contentWriter: object
	"Set current content writer, an object conforming 
	to the #value: protocol to convert an object to a request content entity
	as used by #contents:. Defaults to nil.
	Can be used to format data to be uploaded."

	self optionAt: #contentWriter put: object! !

!ZnClient methodsFor: 'accessing response' stamp: 'SvenVanCaekenberghe 11/8/2011 20:56'!
contents 
	"Answer the contents or HTTP payload of the last response, 
	assuming the current request was executed.
	If I have a contentReader, it will be used to compute the contents.
	Otherwise I delegate to ZnResponse>>#contents, unless I am #streaming,
	in which case I return the stream of the streaming entity."

	^ response ifNotNil: [ 
		self contentReader 
			ifNil: [ 
				self streaming
					ifTrue: [  response entity stream ]
					ifFalse: [ response contents ] ]
			ifNotNil: [ :reader | reader value: response entity ] ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:13'!
contents: object
	"Set the entity contents of the current request to object.
	I will pass this through my contentWriter if I have one.
	Otherwise I delegate to ZnEntity to convert object to a generic entity."
	
	self contentWriter 
		ifNil: [ self entity: (ZnEntity with: object) ]
		ifNotNil: [ :writer | self entity: (writer value: object) ]! !

!ZnClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 15:07'!
debugPolicy
	self 
		beOneShot! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
delete
	"Execute a HTTP DELETE ont the request set up and return the response #contents."
	
	^ self 
		method: #DELETE; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:04'!
delete: url
	"Execute an HTTP DELETE request on url and return the response #contents."
	
	^ self 
		url: url; 
		delete! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 16:59'!
dontFollowRedirects
	"Disable the default automatic following of HTTP redirects.
	Redirects will be returned as normal responses."

	^ self followRedirects: false! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/2/2012 16:30'!
downloadEntityTo: path
	"If I hold an entity, download it directly to a file.
	If path describes a file, use that. 
	If path describes a directory, use the last path segment of the URI to construct a file in that directory.."

	| target |
	self entity ifNil: [ ^ self ].
	target := ZnFileSystemUtils 
		downloadPathTo: path 
		for: self request uri lastPathSegment.
	ZnFileSystemUtils 
		fileNamed: target 
		do:  [ :fileStream |
			fileStream binary.
			self entity writeOn: fileStream ]! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/10/2012 22:21'!
downloadTo: path
	"Setup streaming, do a GET request and download the entity directly to a file.
	If path describes a file, use that. 
	If path describes a directory, use the last path segment of the URI to construct a file in that directory.
	Return true when successful, false otherwise."

	self 
		streaming: true; 
		get.
	^ self isSuccess
		ifTrue: [
			self downloadEntityTo: path.
			true ]
		ifFalse: [ 
			false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:27'!
enforceAcceptContentType
	"Return whether or not I enforce that the content type of the
	response matches the content type specified in the accept request header."
	
	^ self optionAt: #enforceAcceptContentType ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:27'!
enforceAcceptContentType: boolean
	"Set whether or not I enforce that the content type of the
	response matches the content type specified in the accept request
	header to boolean."

	self optionAt: #enforceAcceptContentType put: boolean! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:27'!
enforceHttpSuccess
	"Return whether or not I enforce that the response 
	to the request is an HTTP success code."
	
	^ self optionAt: #enforceHttpSuccess ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:28'!
enforceHttpSuccess: boolean
	"Set whether or not I enforce that the response 
	to the request is an HTTP success code to boolean."

	self optionAt: #enforceHttpSuccess put: boolean! !

!ZnClient methodsFor: 'accessing response' stamp: 'SvenVanCaekenberghe 11/8/2011 18:58'!
entity
	"Answer the entity or HTTP payload of the last response, 
	assuming the current request was executed."
	
	^ response ifNotNil: [ response entity ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:14'!
entity: entity
	"Set the entity of the current request to enity.
	A ZnEntity subclass or compatible object is expected."
	
	request entity: entity! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/20/2011 14:11'!
exceptionSetForIfFail
	"Return an explicit exception set for which the ifFail block, if any, will be invoked.
	This could later be cached, instance or class side, if necessary."
	
	^ NetworkError, ZnParseError, ZnHttpUnsuccessful, ZnUnexpectedContentType 
	! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:09'!
execute
	"Execute the currently set up request to generate a response.
	Return the #contents of the response, if any."
	
	| initialMilliseconds result |
	initialMilliseconds := Time millisecondClockValue.
	self prepareRequest.
	result := self executeWithTimeout.
	self logRequestResponseStarted: initialMilliseconds.
	^ result! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 10/4/2011 13:33'!
executeRequestResponse
	self logRequest.
	self signalProgress ifTrue: [ HTTPProgress signal: 'Writing request' ].
	request writeOn: connection.
	connection flush.
	self signalProgress ifTrue: [ HTTPProgress signal: 'Reading response' ].
	response := request method = #HEAD
		ifTrue: [ ZnResponse readHeaderFrom: 	connection ]
		ifFalse: [ 
			self streaming
				ifTrue: [ ZnResponse readStreamingFrom: connection ]
				ifFalse: [ ZnResponse readFrom: connection ] ].
	self logResponse! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 11/23/2011 17:01'!
executeWithRedirectsRemaining: redirectCount
	self getConnectionAndExecute.
	response isRedirect 
		ifTrue: [
			(redirectCount > 0 and: [ self followRedirects ])
				ifTrue: [
					self 
						prepareRedirect; 
						executeWithRedirectsRemaining: redirectCount - 1 ]
				ifFalse: [
					self followRedirects 
						ifTrue: [ ZnTooManyRedirects signal ] ] ].
	^ self handleResponse
	! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/19/2011 10:52'!
executeWithRetriesRemaining: retryCount
	^ [ self executeWithRedirectsRemaining: self maxNumberOfRedirects ]
		on: Exception
		do: [ :exception |
			retryCount > 0
				ifTrue: [
					self 
						handleRetry;
						executeWithRetriesRemaining: retryCount - 1 ]
				ifFalse: [
					exception pass ] ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 12/20/2011 14:07'!
executeWithTimeout
	^ self withTimeoutDo: [ 
		self ifFailBlock 
			ifNil: [
				self executeWithRetriesRemaining: self numberOfRetries ]
			ifNotNil: [
				[ self executeWithRetriesRemaining: self numberOfRetries ]
					on: self exceptionSetForIfFail 
					do: self ifFailBlock ] ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 16:58'!
followRedirects
	"Return whether I will automatically following of HTTP redirects.
	This is still subject to #maxNumberOfRedirects. Defaults to true.
	When set to false, redirects will be returned as normal responses."

	^ self optionAt: #followRedirects ifAbsent: [ true ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
followRedirects: boolean
	"Set whether or not I will automatically following of HTTP redirects.
	This is still subject to #maxNumberOfRedirects. Defaults to true.
	When set to false, redirects will be returned as normal responses."

	self optionAt: #followRedirects put: boolean ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:19'!
formAdd: association
	"Add the key->value association to the application/x-www-form-urlencoded 
	entity of the current request."
	
	self applicationFormUrlEncodedEntity 
		add: association ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:20'!
formAddAll: keyedCollection
	"Add all key->value associations of keyedCollection to 
	the application/x-www-form-urlencoded entity of the current request."

	self applicationFormUrlEncodedEntity 
		addAll: keyedCollection ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:26'!
formAt: key add: value
	"Add key equals value to the application/x-www-form-urlencoded 
	entity of the current request. This is for multi-values form fields."

	self applicationFormUrlEncodedEntity 
		at: key add: value! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:21'!
formAt: key put: value
	"Set key equal to value in the application/x-www-form-urlencoded 
	entity of the current request."

	self applicationFormUrlEncodedEntity 
		at: key put: value! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
get
	"Execute an HTTP GET ont the request set up and return the response #contents."
	
	^ self 
		method: #GET; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:04'!
get: url
	"Execute an HTTP GET request on url and return the response #contents."

	^ self 
		url: url; 
		get! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 10/4/2011 13:33'!
getConnectionAndExecute
	self canReuseConnection 
		ifTrue: [ 
			[ ^ self executeRequestResponse; cleanupConnection ] 
				on: Exception 
				do: [ self noteIgnoringExceptionOnReusedConnection; close ] ].
	self signalProgress 
		ifTrue: [ HTTPProgress signal: 'Connecting to ', request url authority ].
	connection := ZnNetworkingUtils socketStreamToUrl: request url.
	[ self executeRequestResponse ]
		ensure: [ self cleanupConnection ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 12/20/2011 12:52'!
handleResponse
	self receiveCookies.
	(self enforceHttpSuccess and: [ response isSuccess not ])
		ifTrue: [ 
			(self followRedirects not and: [ response isRedirect ])
				ifFalse: [ (ZnHttpUnsuccessful response: response) signal ] ].
	(self enforceAcceptContentType and: [ self isContentTypeAcceptable not ])
		ifTrue: [ 
			(ZnUnexpectedContentType 
				expected: request accept asZnMimeType 
				actual: response contentType) signal ].
	lastUsed := TimeStamp now.
	state := #done.
	^ self contents! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 8/18/2011 11:38'!
handleRetry
	self noteRetrying.
	self close.
	(Delay forSeconds: self retryDelay) wait.
! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
head
	"Execute a HTTP HEAD ont the request set up and return nil."

	^ self 
		method: #HEAD; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:05'!
head: url
	"Execute an HTTP HEAD request on url and return nil."

	^ self 
		url: url; 
		head! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:23'!
headerAddAll: keyedCollection
	self resetRequestIfNeeded.
	request headers 
		addAll: keyedCollection ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:25'!
headerAt: key add: value
	"Add value to key in the HTTP header of the current request.
	This is for multi-values headers."
	
	self resetRequestIfNeeded.
	request headers 
		at: key add: value! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:24'!
headerAt: key put: value
	"Set key equals value in the HTTP header of the current request"
	
	self resetRequestIfNeeded.
	request headers 
		at: key put: value   ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:26'!
host: hostName
	"Set the host of the current request to hostName."
	
	self url: (ZnUrl new host: hostName; yourself)! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:27'!
http
	"Set the scheme of the current request to HTTP."
	
	self url: (ZnUrl new scheme: #http; yourself)! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:27'!
https
	"Set the scheme of the current request to HTTPS."
	
	self url: (ZnUrl new scheme: #https; yourself)! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:29'!
ifFail: block
	"Set the block that I will be execute when the request fails.
	The exception of the failure will be passed as argument."
	
	self optionAt: #ifFailBlock put: block! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:30'!
ifFailBlock
	"Return the block that I should execute if the request fails.
	Defaults to nil. Accepts an exception as argument."
	
	^ self optionAt: #ifFailBlock ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/27/2011 14:58'!
initialize
	request := ZnRequest empty! !

!ZnClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/19/2011 16:57'!
interactivePolicy
	self 
		beOneShot;
		ifFail: [ :exception | 
			self inform: 'HTTP call failed with ', exception printString.
			nil ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/8/2011 19:14'!
isConnected
	"Return true if I hold an open HTTP connection."

	^ connection notNil and: [ connection isConnected ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/8/2011 19:17'!
isContentTypeAcceptable
	"Return true if the contentType of the response of the last executed 
	HTTP request matches the accept header of the request."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ 
			response hasEntity not 
				or: [ response contentType matches: request accept asZnMimeType ] ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:30'!
isOneShot
	"Return if I use connections only for a single request/response cycle.
	By default, I will keep HTTP connections open according to the HTTP 1.1 spec.
	When oneShot is true, I will add a Connection: close header to requests."

	^ self optionAt: #oneShot ifAbsent: [ false ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/23/2011 17:17'!
isRedirect
	"Return true it the last executed HTTP request returned an HTTP redirect response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isRedirect ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/8/2011 19:17'!
isSuccess 
	"Return true it the last executed HTTP request returned an HTTP success response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isSuccess ]! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2011 12:35'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 2/29/2012 19:10'!
logRequest
	"Log a request that was just written"
	
	self log info: [ 
		'Wrote ', request printString ].
	self log debug: [
		String streamContents: [ :out | 
			out << 'Sent headers'; crlf.
			request headers writeOn: out ] ]! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 11/8/2011 19:08'!
logRequestResponseStarted: initialMilliseconds
	"Log a request/response cycle with timing and bytes received info,
	similar to standard HTTP logging."
	
	self log transaction: [
		String streamContents: [ :stream |
			request requestLine printMethodAndUriOn: stream.  
			stream space; print: response code.
			response hasEntity ifTrue: [
				stream space; print: response entity contentLength; nextPut: $B ].  
			stream space; print: (Time millisecondsSince: initialMilliseconds); nextPutAll:  'ms' ] ]! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 2/29/2012 19:11'!
logResponse
	"Log a response that was just read"
	
	self log info: [ 
		'Read ', response printString ].
	self log debug: [
		String streamContents: [ :out | 
			out << 'Received headers'; crlf.
			response headers writeOn: out ] ]! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 10/4/2011 12:35'!
logToTranscript
	"Setup logging to the Transcript"

	self log addListener: ZnTranscriptLogger new  ! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 17:23'!
maxNumberOfRedirects
	"Return the maximum number of HTTP redirect that will be followed.
	Note that when the #followsRedirect option is false, 
	no redirects will ever be followed regardless of the value of this option.
	Defaults to 3. A ZnTooManyRedirects will be signalled when the count reaches zero."

	^ self optionAt: #maxNumberOfRedirects ifAbsent: [ 3 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 17:23'!
maxNumberOfRedirects: count
	"Set the maximum number of HTTP redirect that will be followed to count.
	Note that when the #followsRedirect option is false, 
	no redirects will ever be followed regardless of the value of this option.
	A ZnTooManyRedirects will be signalled when the count reaches zero."

	self optionAt: #maxNumberOfRedirects put: count! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:28'!
method: httpMethod
	"Set the HTTP method of the current request to httpMethod.
	An uppercase symbol such as #GET is expected."
	
	self resetRequestIfNeeded.
	request method: httpMethod! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:30'!
multiPartFormDataEntity
	"Initialize the current request to have an multipart/form-data entity.
	Return the existing entity if there is one. 
	Fail if another type was already set."

	self resetRequestIfNeeded.
	request hasEntity
		ifTrue: [
			request entity contentType = ZnMimeType multiPartFormData
				ifFalse: [ self error: 'An incompatible entity was already set for the request' ] ]
		ifFalse: [
			request entity: ZnMultiPartFormDataEntity new ].
	^ request entity! !

!ZnClient methodsFor: 'notifications' stamp: 'SvenVanCaekenberghe 10/4/2011 12:43'!
noteIgnoringExceptionOnReusedConnection
	self log debug: 'Ignoring exception on reused connection'! !

!ZnClient methodsFor: 'notifications' stamp: 'SvenVanCaekenberghe 10/4/2011 12:43'!
noteRedirect
	self log debug: 'Redirecting'! !

!ZnClient methodsFor: 'notifications' stamp: 'SvenVanCaekenberghe 10/4/2011 12:43'!
noteRetrying
	self log debug: 'Retrying'! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:31'!
numberOfRetries
	"Return how many times I will retry a failed request.
	Defaults to 0."
	
	^ self optionAt: #numberOfRetries ifAbsent: [ 0 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:31'!
numberOfRetries: count
	"Set the number of times I will retry a failed request to count."
	
	self optionAt: #numberOfRetries put: count! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/8/2011 16:53'!
optionAt: key ifAbsent: block
	"Return my option/settings stored under key.
	Execute block if I have no such option/setting.
	This is a generic interface, see my options protocol for specific usages."

	options ifNil: [ ^ block value ].
	^ options at: key ifAbsent: block! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/30/2012 21:30'!
optionAt: key put: value
	"Set my option/setting identified by key to be value.
	This is a generic interface, see my options protocol for specific usages."
	
	options ifNil: [ options := Dictionary new ].
	options at: key put: value! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:43'!
path: pathString
	"Set the path of the current request to pathString.
	The argument will be merged with of the current/previous request."
	
	self url: pathString! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:32'!
port: portNumber
	"Set the port of the current request to portNumber."
	
	self url: (ZnUrl new port: portNumber; yourself)! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
post
	"Execute a HTTP POST ont the request set up and return the response #contents."

	^ self 
		method: #POST; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:12'!
post: url contents: object
	"Execute an HTTP POST request on url with object as request #contents:
	and return the response #contents."

	^ self 
		url: url; 
		contents: object; 
		post! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 2/29/2012 16:43'!
prepareRedirect
	"We implement the common behavior that POST/PUT redirects turn into GET request.
	Note also that the body data is dropped, as well as the cookie handling."

	self noteRedirect.
	self receiveCookies.
	(#(POST PUT) includes: request method)
		ifTrue: [ self method: #GET ].
	request hasEntity
		ifTrue: [ request resetEntity: nil ].
	self url: self redirectUrl.
	state := #inProgress.
	self sendCookies! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/23/2011 14:30'!
prepareRequest
	state := #inProgress.
	self isOneShot 
		ifTrue: [ request setConnectionClose ].
	request hasEntity
		ifTrue: [ 
			"This is necessary because entities are often modified after first being added empty,
			and a wrong content length is cached at the request level"
			request resetEntity: request entity ].
	self sendCookies ! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:14'!
put
	"Execute a HTTP PUT ont the request set up and return the response #contents."

	^ self 
		method: #PUT; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:12'!
put: url contents: object
	"Execute an HTTP PUT request on url with object as request #contents:
	and return the response #contents."

	^ self 
		url: url; 
		contents: object; 
		put! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:36'!
queryAddAll: keyedCollection
	"Add all key -> value elements of keyedCollection 
	as an HTTP query parameters to the current request."

	keyedCollection keysAndValuesDo: [ :key :value |
		self queryAt: key put: value ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:36'!
queryAt: key put: value
	"Add key equals value as an HTTP query parameter to the current request."
	
	self resetRequestIfNeeded.
	request url 
		ifNil: [ request url: (ZnUrl new queryAt: key put: value; yourself) ]
		ifNotNil: [ request url queryAt: key put: value ]  ! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/29/2012 19:07'!
receiveCookies
	response cookies do: [ :each |
		each defaultDomain: request url host.
		self session cookieJar add: each.
		self log debug: [ 'Received cookie: ', each printString ]  ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/19/2011 09:46'!
redirectUrl
	"We support relative locations although they are not the standard (http://en.wikipedia.org/wiki/HTTP_location)"
	
	| redirectUrl |
	redirectUrl := response location asZnUrl.
	redirectUrl isAbsolute
		ifFalse: [ | baseUrl |
			baseUrl := request url.
			redirectUrl 
				scheme: baseUrl scheme;
				host: baseUrl host;
				port: baseUrl port ].
	^ redirectUrl! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:33'!
request
	"Return the current request that I am building,
	or the previous request that I executed."
	
	^ request ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:34'!
request: aRequest
	"Replace my current request with aRequest.
	This is a low level operation only needed in special cases."
	
	self validateConnectionForHost: aRequest url host port: aRequest url port.
	request := aRequest! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/29/2012 17:40'!
resetCookies
	session ifNotNil: [ session resetCookies ]! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/23/2011 14:53'!
resetRequestIfNeeded
	request ifNil: [ ^ self ].
	state ifNotNil: [
		request 
			resetEntity: nil;
			headers: ZnHeaders defaultRequestHeaders.
		state := nil ]! !

!ZnClient methodsFor: 'accessing response' stamp: 'SvenVanCaekenberghe 11/8/2011 18:58'!
response 
	"Answer the last response, assuming the current request was executed."

	^ response ! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:33'!
retryDelay
	"Return the number of seconds I will delay retrying a failed request,
	provided I do retry requests. Defaults to 1 seconds."
	
	^ self optionAt: #retryDelay ifAbsent: [ 1 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
retryDelay: seconds
	"Set the delay before I will delay retrying a failed request to seconds,
	provided I do retry requests."

	self optionAt: #retryDelay put: seconds! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/29/2012 19:51'!
sendCookies
	| cookies cookiesString |
	session ifNil: [ ^ self ].
	cookies := (session cookieJar cookiesForUrl: request url).
	cookies isEmpty ifTrue: [ ^ self ]. 
	cookiesString := String streamContents: [ :stream |
		cookies
			do: [ :each | each writeNameAndValueOn: stream ]
			separatedBy: [ stream nextPutAll: '; ' ] ].
	request setCookie: cookiesString.
	self log debug: [ 'Cookie set to: ', cookiesString ]! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/8/2011 16:54'!
session
	"Return the current HTTP session object that I am using.
	This session object holds cookies among other things."

	^ session ifNil: [ session := ZnUserAgentSession new ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:35'!
setBasicAuthenticationUsername: username password: password
	"Set the username/password for basic authenticationfor the current request."
	
	self resetRequestIfNeeded.
	request setBasicAuthenticationUsername: username password: password
! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 5/10/2012 22:39'!
setIfModifiedSince: reference
	self request setIfModifiedSince: reference! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:44'!
signalProgress
	"Return true if I signal HTTPProgress notifications during execution."
	
	^ self optionAt: #signalProgress ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:45'!
signalProgress: boolean
	"Set whether I will signal HTTPProgress notifications during execution to boolean."
	
	self optionAt: #signalProgress put: boolean! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 10/4/2011 12:35'!
stopLogging
	"Stop doing any logging by disabling our log support"

	self log disable! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:58'!
streaming
	"Return whether I will return streaming response objects (see ZnStreamingEntity).
	If true, my users are responsible for actually reading the incoming response entity data stream."
	
	^ self optionAt: #streaming ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
streaming: boolean
	"Set whether I will return streaming response objects (see ZnStreamingEntity) to boolean.
	If true, my users are responsible for actually reading the incoming response entity data stream."

	self optionAt: #streaming put: boolean! !

!ZnClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 20:27'!
systemPolicy
	self 
		enforceHttpSuccess: true;
		enforceAcceptContentType: true;
		numberOfRetries: 2! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 12/14/2011 14:03'!
timeout
	"Return the timeout in seconds for network socket stream operations
	like connecting, reading and writing. Defaults to a global setting."
	
	^ self optionAt: #timeout ifAbsent: [ ZnNetworkingUtils defaultSocketStreamTimeout ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
timeout: seconds
	"Set the timeout for network socket stream operations
	like connecting, reading and writing to seconds."

	self optionAt: #timeout put: seconds! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/2/2012 16:19'!
uploadEntityFrom: path
	"Set up a ZnStreamingEntity from the binary contents of the file at path.
	The path extension is used to guess the content-type,
	can be overridden using #contentType: if necessary.
	Use the file's length to set the content-length.
	You can then use #put or #post to actually do an upload."
	
	| stream type length |
	stream := ZnFileSystemUtils fileStreamFor: path.
	stream binary.
	type := ZnMimeType forFilenameExtension: (ZnFileSystemUtils extensionFor: path).
	length := ZnFileSystemUtils fileSizeFor: path.
	self entity: (ZnStreamingEntity readFrom: stream usingType: type andLength: length)! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:42'!
url: urlObject
	"Modify the URL of the current request object to urlObject.
	This is done using ZnUrl>>#inContextOf: so that either scheme, 
	host, port or path are set and merged.
	Default the URL using ZnUrl>>#asZnUrlWithDefaults to fall back
	to http or the default http/https ports."
	
	| url |
	url := urlObject asZnUrl.
	request url
		ifNotNil: [ url := url inContextOf: request url ].  
	url := url asZnUrlWithDefaults.  
	self validateConnectionForHost: url host port: url port.   
	self resetRequestIfNeeded.
	request url: url! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:35'!
username: username password: password
	"Set the username/password for basic authenticationfor the current request."
	
	self setBasicAuthenticationUsername: username password: password
! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/12/2011 13:23'!
validateConnectionForHost: host port: port
	(connection isNil or: [ request url isNil ]) 
		ifTrue: [ ^ self ].
	(request url host = host and: [ request url port = port ])
		ifFalse: [ self close ]  ! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/19/2011 16:55'!
withTimeoutDo: block
	^ ZnConnectionTimeout value
		ifNil: [
			ZnConnectionTimeout 
				value: self timeout 
				during: [ ^ block value ] ]
		ifNotNil: [ 
			block value ]! !

Object subclass: #ZnClientOld
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Deprecated'!

!ZnClientOld commentStamp: 'SvenVanCaekenberghe 11/8/2011 22:12' prior: 0!
I am ZnClientOld, previously known as ZnClient.

I am deprecated, please use ZnEasy.

Part of Zinc HTTP Components.!

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:20'!
delete: urlObject
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy delete: urlObject! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
delete: urlObject username: username password: password
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy delete: urlObject username: username password: password! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
get: urlObject
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy get: urlObject! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
get: urlObject username: username password: password
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy get: urlObject username: username password: password! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
getGif: urlObject
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy getGif: urlObject ! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
getJpeg: urlObject
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy getJpeg: urlObject! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
getPng: urlObject
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy getPng: urlObject! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
head: urlObject
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy head: urlObject ! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
head: urlObject username: username password: password
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy head: urlObject username: username password: password! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
post: urlObject data: entity
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy post: urlObject data: entity! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
post: urlObject data: entity username: username password: password
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy post: urlObject data: entity username: username password: password! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
put: urlObject data: entity
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy put: urlObject data: entity! !

!ZnClientOld class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 3/4/2012 10:21'!
put: urlObject data: entity username: username password: password
	self deprecated: 'Please use ZnEasy' on: '3 Oct 2011' in: 'Zinc HTTP Components'.
	^ ZnEasy put: urlObject data: entity username: username password: password! !

Object subclass: #ZnConstants
	instanceVariableNames: ''
	classVariableNames: 'HTTPStatusCodes MaximumEntitySize'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnConstants commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:01' prior: 0!
I am ZnConstants, holding various constants that we use.

Part of Zinc HTTP Components.!

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
defaultHTTPVersion
	^ 'HTTP/1.1'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/31/2011 08:54'!
defaultServerString 
	^ self frameworkNameAndVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:49'!
defaultUserAgent
	^ self frameworkNameAndVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/10/2011 11:54'!
faviconBytes
	^ #(0 0 1 0 1 0 16 16 16 0 0 0 0 0 40 1 0 0 22 0 0 0 40 0 0 0 16 0 0 0 32 0 0 0 1 0 4 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 255 0 0 189 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 17 17 17 1 16 1 16 1 17 17 17 1 16 1 16 1 16 0 0 1 16 1 16 0 17 0 0 1 16 1 16 0 1 16 0 1 16 1 16 0 0 17 0 1 17 17 16 0 0 1 16 1 1 17 0 0 0 0 17 0 0 0 0 1 17 17 17 0 0 0 0 1 17 17 17 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0) asByteArray! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:36'!
frameworkLicense
	"http://en.wikipedia.org/wiki/MIT_License"
	
	^ 'MIT License'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 17:12'!
frameworkMCVersion
	MCWorkingCopy managersForClass: self class do: [ :each | ^ each ancestors first name ]! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:35'!
frameworkName
	^ 'Zinc HTTP Components'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:47'!
frameworkNameAndVersion
	^ self frameworkName, ' ', self frameworkVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/31/2011 08:58'!
frameworkVersion
	^ '1.0'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:53'!
httpStatusCodes
	^ HTTPStatusCodes ! !

!ZnConstants class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/6/2010 15:51'!
initialize 
	self initializeHTTPStatusCodes! !

!ZnConstants class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/6/2010 16:22'!
initializeHTTPStatusCodes 
	"This is the official list from http://www.iana.org/assignments/http-status-codes"

	HTTPStatusCodes := Dictionary newFromPairs: #(
		100 'Continue'
		101 'Switching Protocols'
		102 'Processing'
		200 'OK'
		201 'Created'
		202 'Accepted'
		203 'Non-Authoritative Information'
		204 'No Content'
		205 'Reset Content'
		206 'Partial Content'
		207 'Multi-Status'
		208 'Already Reported'
		226 'IM Used'
		300 'Multiple Choices'
		301 'Moved Permanently'
		302 'Found'
		303 'See Other'
		304 'Not Modified'
		305 'Use Proxy'
		306 'Reserved'
		307 'Temporary Redirect'
		400 'Bad Request'
		401 'Unauthorized'
		402 'Payment Required'
		403 'Forbidden'
		404 'Not Found'
		405 'Method Not Allowed'
		406 'Not Acceptable'
		407 'Proxy Authentication Required'
		408 'Request Timeout'
		409 'Conflict'
		410 'Gone'
		411 'Length Required'
		412 'Precondition Failed'
		413 'Request Entity Too Large'
		414 'Request-URI Too Long'
		415 'Unsupported Media Type'
		416 'Requested Range Not Satisfiable'
		417 'Expectation Failed'
		422 'Unprocessable Entity'
		423 'Locked'
		424 'Failed Dependency'
		425 'Reserved for WebDAV'
		426 'Upgrade Required'
		500 'Internal Server Error'
		501 'Not Implemented'
		502 'Bad Gateway'
		503 'Service Unavailable'
		504 'Gateway Timeout'
		505 'HTTP Version Not Supported'
		506 'Variant Also Negotiates (Experimental)'
		507 'Insufficient Storage'
		508 'Loop Detected'
		510 'Not Extended')! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2011 18:07'!
knownHTTPMethods
	^ #(
		"Standard HTTP 1.1 Verbs"
		GET PUT POST DELETE HEAD 
		TRACE OPTIONS CONNECT PATCH
		"WebDAV verbs"
		PROPFIND PROPPATCH MKCOL COPY MOVE LOCK UNLOCK)! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:22'!
knownHTTPVersions
	^ #('HTTP/1.1' 'HTTP/1.0')! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 1/31/2012 13:42'!
maximumEntitySize
	"Return the maximum entity size to accept from a stream.
	Used by ZnEntity and subclasses. Can be set using #maximumEntitySize:
	Default value is 16Mb. This affects both client and server code.
	This helps to protect us from malicious content."
	
	^ MaximumEntitySize ifNil: [ MaximumEntitySize := 16 * 1024 * 1024 ] ! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 1/31/2012 12:17'!
maximumEntitySize: integer
	"Set the maximum entity size to accept from a stream to integer bytes."
	
	^ MaximumEntitySize := integer! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 1/2/2012 17:45'!
maximumLineLength
	"Return the maximum line length to accept.
	Used by ZnLineReader and thus for reading request/status lines as well as headers.
	This helps to protect us from malicious content."
	
	^ 4096! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 1/2/2012 17:54'!
maximumNumberOfDictionaryEntries
	"Return the maximum number dictionary entries to accept.
	Used by ZnMultiValueDictionary and thus for reading headers, 
	url query and application form url encoded entity fields.
	This helps to protect us from malicious content."
	
	^ 256! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:19'!
remoteAddressHeader
	^ 'X-Zinc-Remote-Address'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
standardHtmlDocType
	^ '<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/11/2012 14:43'!
welcomePageHtml
	^ '<?xml version="1.0" encoding="UTF-8"?>
<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
<title>Zinc HTTP Components</title>
<style type="text/css">
body {
  color: black;
  background: white;
  width: 900px;
  font-family: Verdana, Arial, Helvetica, sans-serif;
  font-size: 13px;
}
p {
  width: 600px;
  padding: 0 20px 10px 20px;
} 
ul,ol {
  width: 600px;
  padding: 0 5px 5px 30px;
} 
#logo {
  color: orange;
  font-family: Helvetica, sans-serif;
  font-weight: bold;
  font-size: 100px;
}
</style>
</head>
<body>
<div id="logo">Zn</div>
<h1>Zinc HTTP Components</h1>
<p>
Welcome to Zinc HTTP Components, 
a modern, open-source Smalltalk framework to deal with the HTTP networking protocol.
</p>
<ul>
<li>Project homepage <a href="http://zn.stfx.eu/">http://zn.stfx.eu</a></li> 
<li>Locally available pages <a href="/help">/help</a></li>
</ul>
<h4>May the Source be with you!!</h4>
</body>
</html>
'! !

Object subclass: #ZnCookie
	instanceVariableNames: 'attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookie commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent an HTTP Cookie token.

Part of Zinc HTTP Components.!

!ZnCookie class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/30/2011 19:46'!
fromString: aString
	^ self new 
		fromString: aString; 
		yourself! !

!ZnCookie class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/30/2011 19:46'!
fromString: aString for: urlObject
	| cookie |
	cookie := self fromString: aString.
	"Ensures a domain value if none provided by the cookie"
	cookie defaultDomain: urlObject host.
	^ cookie! !

!ZnCookie class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/30/2011 22:23'!
name:  nameString value: value
	^ self new
		name: nameString;
		value: value;
		yourself! !

!ZnCookie methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
= aCookie
	(aCookie isKindOf: self class) ifFalse: [ ^ false ].
	^ (aCookie name = self name) & (aCookie path = self path) & (aCookie domain = self domain)
	! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/30/2011 19:45'!
defaultDomain: aDomain
	self domain ifNil: [ self domain: aDomain ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:04'!
domain
	^ attributes at: 'domain' ifAbsent: [ nil ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:25'!
domain: aDomain
	attributes at: 'domain' put: aDomain! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:04'!
expires
	^ attributes at: 'expires' ifAbsent: [ nil ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
expires: anExpiration
	attributes at: 'expires' put: anExpiration! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2011 11:29'!
expiresTimeStamp
	self expires ifNil: [ ^ TimeStamp current plusDays: 1 ].
	^ ZnUtils parseHttpDate: self expires! !

!ZnCookie methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:17'!
extraAttributes
	"Answer all attributes except name and value"
	^ attributes associationsSelect: [ :ea | (ea key = 'name' or: [ ea key = 'value' ]) not ]! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/8/2012 14:16'!
fromString: aString
	| tokens val data i |
	tokens := aString subStrings: ';'.
	i := tokens first indexOf: $=.
	val := tokens first copyFrom: 1 to: i - 1.
	data := tokens first copyFrom: i + 1 to: tokens first size. 
	self name: val trimBoth; value: data trimBoth.
	2 to: tokens size do: [ :index | 
		self readAttribute: (tokens at: index) ]! !

!ZnCookie methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/30/2011 21:43'!
fullString
	^ String streamContents: [ :stream | 
		self writeOn: stream ]! !

!ZnCookie methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/30/2011 20:02'!
hash
	^ (self name hash bitXor: self path hash) bitXor: self domain hash! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:03'!
httponly
	"This is not an RFC cookie attribute, but a supplement found in MS Internet Explorer and supported in PHP.
	 See http://msdn.microsoft.com/en-us/library/ms533046.aspx"
	^ attributes at: 'httponly' ifAbsent: [ false ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
httponly: aBoolean
	attributes at: 'httponly' put: aBoolean! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'MattKennedy 9/27/2010 20:21'!
initialize 
	attributes := Dictionary new.! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
isExpired
	^ (self expiresTimeStamp asUTC < TimeStamp current asUTC) or: [ self expires = 0 ]! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
isInDomain: urlObject
	| domain |
	domain := (self domain beginsWith: '.')
		ifTrue: [ self domain copyFrom: 2 to: self domain size ]
		ifFalse: [ self domain ].
	^ urlObject host includesSubString: domain
	! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
isInPath: urlObject
	^ urlObject pathPrintString includesSubString: self path
	! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
maxage
	^ self expires! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
name
	^ attributes at: 'name'! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
name: aName
	attributes at: 'name' put: aName! !

!ZnCookie methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/30/2011 21:42'!
nameValueString
	^ String streamContents: [ :stream | 
		self writeNameAndValueOn: stream ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:04'!
path
	^ attributes at: 'path' ifAbsent: [ '/' ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:08'!
path: aPath
	attributes at: 'path' put: aPath! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:43'!
printOn: stream
	self writeOn: stream! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/8/2012 14:16'!
readAttribute: aToken
	| i key data |
	i := aToken indexOf: $=.
	i > 0
		ifTrue: [
			key := (aToken copyFrom: 1 to: i - 1) trimBoth asLowercase.
			data := (aToken copyFrom: i + 1 to: aToken size) trimBoth ]
		ifFalse: [ 
			key := (aToken copyFrom: 1 to: aToken size) trimBoth asLowercase.
			data := true ].
	attributes at: key put: data! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
secure
	^ attributes at: 'secure' ifAbsent: [ false ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
secure: aBoolean
	attributes at: 'secure' put: aBoolean! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
value
	^ attributes at: 'value' ifAbsent: [ nil ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
value: aValue
	attributes at: 'value' put: aValue! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/30/2011 20:03'!
writeExtraAttributesOn: aStream
	self extraAttributes
		associationsDo: [ :ea | 
			aStream
				nextPutAll: '; ';
				nextPutAll: ea key.
			(ea value isKindOf: Boolean)
				ifFalse: [ 
					aStream
						nextPutAll: '=';
						nextPutAll: ea value ] ]! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:37'!
writeNameAndValueOn: aStream
	aStream
		nextPutAll: self name;
		nextPutAll: '=';
		nextPutAll: self value! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:32'!
writeOn: aStream
	self
		writeNameAndValueOn: aStream;
		writeExtraAttributesOn: aStream ! !

Object subclass: #ZnCookieJar
	instanceVariableNames: 'cookies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookieJar commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent a container for storing HTTP Cookies for a client session.

Part of Zinc HTTP Components.!

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
add: aCookie
	(self removeIfExpired: aCookie) ifTrue: [ ^ nil ].
	self cookies do: [ :cookie |
		cookie = aCookie ifTrue: [ self cookies remove: cookie ] ].
	self cookies add: aCookie
	! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
cookieAt: aName forUrl: urlObject
	(self cookiesForUrl: urlObject) do: [ :cookie | cookie name = aName ifTrue: [ ^ cookie ] ].
	^ nil! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
cookies 
	^ cookies ifNil: [ cookies := Bag new ]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:06'!
cookiesForUrl: urlObject
	^ self cookies select: [ :cookie | 
		(cookie isExpired not) & (cookie isInDomain: urlObject) & (cookie isInPath: urlObject) ]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:06'!
remove: aCookie
	self cookies remove: aCookie ifAbsent: [ nil ]! !

!ZnCookieJar methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 2/29/2012 17:39'!
removeAll
	cookies ifNotNil: [ cookies removeAll ]! !

!ZnCookieJar methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
removeIfExpired: aCookie
	^ aCookie isExpired 
		ifTrue: [ 
			self remove: aCookie. 
			true ]
		ifFalse: [
			false ]! !

Object subclass: #ZnCredential
	instanceVariableNames: 'domain realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:56' prior: 0!
I am ZnCredential, an abstract class representing credentials in a domain and realm.

Part of Zinc HTTP Components.!

ZnCredential subclass: #ZnBasicCredential
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:58' prior: 0!
I am ZnBasicCredential, a concrete subclass of ZnCredential.
I implement basic HTTP authentication.
I hold a username and password.

Part of Zinc HTTP Components.!

!ZnBasicCredential class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
authName
	^ 'Basic'! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
authTokenForUrl: urlObject method: method
	^ 'Basic ', (ZnUtils encodeBase64: (username, ':', password)) ! !

!ZnBasicCredential methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
isComplete 
	^ username notNil & password notNil ! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password
	^ password! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password: anObject
	password := anObject! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username
	^ username! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username: anObject
	username := anObject! !

ZnBasicCredential subclass: #ZnDigestCredential
	instanceVariableNames: 'nonce opaque'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:59' prior: 0!
I am ZnDigestCredential, a concrete subclass of ZnCredential and subclass of ZnBasicCredential.
I implement digest HTTP authentication.
I hold a nounce and opaque.

Part of Zinc HTTP Components.!

!ZnDigestCredential class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
authName
	^ 'Digest'! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
a1
	^ ZnDigestAuthenticator md5Hash: self username, ':', self realm, ':', self password! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
a2ForUrl: urlObject method: method
	"This doesn't support all qop options yet."
	^ ZnDigestAuthenticator md5Hash: method, ':', (self uriPathFrom: urlObject)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:37'!
authTokenForUrl: urlObject method: method
	| result |
	result := WriteStream on: String new.
	result
		nextPutAll: 'Digest username="';
		nextPutAll: self username;
		nextPutAll: '", ';
		nextPutAll: 'nonce="';
		nextPutAll: self nonce;
		nextPutAll: '", ';
		nextPutAll: 'realm="';
		nextPutAll: self realm;
		nextPutAll: '", ';
		nextPutAll: 'uri="';
		nextPutAll: (self uriPathFrom: urlObject);
		nextPutAll: '", ';
		nextPutAll: 'response="';
		nextPutAll: (self responseForUrl: urlObject method: method);
		nextPutAll: '", '.
	self opaque isNil
		ifFalse: [ 
			result
				nextPutAll: 'opaque="';
				nextPutAll: self opaque;
				nextPutAll: '"' ].
	^ result contents! !

!ZnDigestCredential methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
isComplete 
	^ super isComplete & nonce notNil ! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:08'!
linkToAuthRequest: aHeader
	| data |
	data := self parseAuthRequest: aHeader.
	self nonce: (data at: 'nonce').
	self realm: (data at: 'realm' ifAbsent: [ '' ])! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce
	^ nonce! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce: anObject
	nonce := anObject! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque
	^ opaque! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque: anObject
	opaque := anObject! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
parseAuthRequest: aHeader
	^ ZnDigestAuthenticator parseAuthRequest: aHeader! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
responseForUrl: urlObject method: method
	^ ZnDigestAuthenticator md5Hash: self a1, ':', self nonce, ':', (self a2ForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:28'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: (self authTokenForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:08'!
uriPathFrom: urlObject
	^ urlObject pathPrintString ifNil: [ '/' ]! !

!ZnCredential class methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:28'!
authName
	self subclassResponsibility ! !

!ZnCredential class methodsFor: 'accessing class hierarchy' stamp: 'SvenVanCaekenberghe 8/18/2011 14:09'!
findHandlerFor: anAuthHeader
	| type i |
	i := anAuthHeader indexOf: Character space.
	type := anAuthHeader copyFrom: 1 to: i - 1.
	self allSubclassesDo: [ :subclass |
		subclass authName = type ifTrue: [ ^ subclass ] ].
	^ nil! !

!ZnCredential class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
findRealmFor: anAuthHeader
	(anAuthHeader subStrings: ',') do: [ :token |
		"String>>trimBoth: is a grease method, so not sure if we should be using it."
		(token includesSubString: 'realm=') 
			ifTrue: [ ^ ((token subStrings: '=') second) ] ].
	^ nil! !

!ZnCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:20'!
authToken
	^ self authTokenForUrl: nil method: nil ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:48'!
authTokenForUrl: urlObject method: method
	self subclassResponsibility ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain
	^ domain! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain: anObject
	domain := anObject! !

!ZnCredential methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:20'!
isComplete
	^ self subclassResponsibility ! !

!ZnCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:10'!
linkToAuthRequest: aHeader
	"Link this credential to the WWW-Authenticate request in the provided header. 
	Used for authentication methods which require parameters provided by the server
	(e.g. Digest authentication)."
	realm := self class findRealmFor: aHeader! !

!ZnCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
realm
	^ realm ifNil: [ realm := '' ]! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
realm: anObject
	realm := anObject! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:42'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: self authToken! !

Object subclass: #ZnDefaultServerDelegate
	instanceVariableNames: 'prefixMap byteArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnDefaultServerDelegate commentStamp: 'SvenVanCaekenberghe 8/18/2011 14:46' prior: 0!
I am ZnDefaultServerDelegate.
I function as a delegate for ZnServer, implementing #handleRequest:

I implement responses to the following prefixes:

/echo - an echo text of request information for debugging purposes
/dw-bench - a dynamic html page for benchmarking purposes
/unicode - a unicode test page
/random - a random string (/random/32 for a specific size)
/bytes - bytes according to a pattern (/bytes/32 for a specific size)
/favicon.ico - a Zn favicon
/status - a server status page
/help - lists all page prefixes
/ - an html welcome page

Without any matches, I respond with a page not found.

Part of Zinc HTTP Components.!

!ZnDefaultServerDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/10/2010 15:17'!
new
	^ super new initialize! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 8/18/2011 14:09'!
bytes: request
	"Answer a number of bytes according to a pattern as an application/octet-stream.
	Optionally take the size from the URI, as in /bytes/32 for 32 bytes, default to 64 bytes.
	The main goal is to be efficient, esp. in benchmarks, that's why we cache (non-thread-safe)"
	
	| count |
	count := Integer readFrom: request uri lastPathSegment ifFail: [ 64 ].
	(byteArray isNil or: [ byteArray size ~= count ]) 
		ifTrue: [
			byteArray := ByteArray new: count.
			1 to: count - 1 do: [ :each | 
				byteArray at: each put: (#(65 66 67 68) at: (each \\ 4 + 1)) ].
			byteArray at: count put: 10 ].
	^ ZnResponse ok: (ZnEntity bytes: byteArray)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:30'!
dwbench: request
	"Reply with the dynamic DW-Bench HTML page"

	^ ZnResponse ok: (ZnEntity html: self generateDWBench)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 7/14/2011 09:36'!
echoRequest: request	
	"Echo request by generating a plain text response useful for debugging purposes"
	
	| delay info entity |
	delay := Integer readFrom: (request uri queryAt: 'delay' ifAbsent: [ '' ]) ifFail: [ 0 ].
	delay > 0 ifTrue: [ (Delay forSeconds: delay) wait ].  
	info := self generateEchoRequestString: request.
	entity := ZnEntity text: info.
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:39'!
favicon: request
	"Answer a nice favicon for browsers to display"

	| entity |
	entity := ZnEntity with: ZnConstants faviconBytes type: 'image/vnd.microsoft.icon'. 
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/18/2012 19:08'!
generateDWBench
	"Generate the dynamic DW-Bench HTML page.
	Although the date/time is variable, the page size is constant."
	
	^ String streamContents: [ :stream |
		stream << ZnConstants standardHtmlDocType.
		stream << '<html><head><title>DW-Bench Dynamic</title></head>'.
		stream << '<body><h1>DW-Bench Dynamic</h1><table border=1>'.
		1 to: 25 do: [ :row |
			stream << '<tr>'.
			1 to: 25 do: [ :col |
				stream << '<td>'; print: row * col; << '</td>' ].
			stream << '</tr>' ].
		stream << '</table><p>'; << Date today yyyymmdd; space; << Time now print24; << '</p></body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/19/2011 10:30'!
generateEchoRequestString: request
	"Generate a string echoing the request"
	
	^ String streamContents: [ :stream |
		stream nextPutAll: 'This is Zinc HTTP Components echoing your request !!'; crlf.
		stream nextPutAll: request method; nextPutAll: ' request for '.
		request uri printPathQueryFragmentOn: stream.
		stream crlf; nextPutAll: 'with headers'; crlf.
		request headersDo: [ :key :value |
			stream space; nextPutAll: key; nextPutAll: ': '; nextPutAll: value; crlf ].
		request entity isNil ifFalse: [
			stream nextPutAll: ' containing '; print: request entity ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/11/2011 10:11'!
generateHelp
	"Generate an HTML page with links to all pages I support"
	
	| title |
	title := self class name, ' Help'.
	^ String streamContents: [ :stream |
		stream << ZnConstants standardHtmlDocType.
		stream << '<html><head><title>'; << title; << '</title></head>'.
		stream << '<body><h1>'; << title; << '</h1><h3>Available Pages</h3><ul>'.
		prefixMap keys do: [ :each | 
			stream << '<li><a href="'; << each; << '">'; << each; << '</a></li>' ].
		stream << '<ul></body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/1/2012 13:55'!
generateStatus
	"Generate an HTML page with the state of the server"
	
	^ String streamContents: [ :stream |
		stream << ZnConstants standardHtmlDocType.
		stream nextPutAll: '<html><head><title>ZnServer Status</title></head>'.
		stream nextPutAll: '<body><h1>ZnServer Status</h1>'.
		stream nextPutAll: '<p>'; nextPutAll: self systemVersionInfo; nextPutAll: '</p>'.
		stream nextPutAll: '<pre>'; nextPutAll: self vmStats; nextPutAll: '</pre>'.  
		stream nextPutAll: '<h1>Processes</h1><ol>'.
		self processes do: [ :each |
			stream 
				nextPutAll: '<li>'; 
				nextPutAll: each name; space; nextPut: $[; print: each priority; nextPut: $]; nextPutAll: '<br/>'; nextPutAll: each printString;
				nextPutAll: '</li>' ].
		stream nextPutAll: '</ol>'.  
		stream nextPutAll: '</body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/11/2011 10:12'!
generateUnicodeTest
	"Generate an HTML page containing all Unicode characters with code points between 0 and 16r024F in a nice table"
	
	^ String streamContents: [ :stream |
		stream << ZnConstants standardHtmlDocType.
		stream << '<html><head><title>Unicode Test Page</title></head>'.
		stream << '<body><h1>Unicode Test Page</h1><table>'.
		0 to: 16r024F by: 8 do: [ :row |
			stream nextPutAll: '<tr>'.
			row to: row + 7 do: [ :each |
				stream nextPutAll: '<td>U+'.
				stream nextPutAll: (each printPaddedWith: $0 to: 4 base: 16). 
				stream nextPut: $=; nextPut: each asCharacter.
				stream nextPutAll: '</td>' ].
			stream nextPutAll: '</tr>' ].
		stream << '</table></body></html>' ]! !

!ZnDefaultServerDelegate methodsFor: 'public' stamp: 'PaulDeBruicker 4/19/2012 16:02'!
handleRequest: request
	| prefix prefixHandler |
	prefix := request uri isSlash
		ifTrue: [ prefixMap at: '/' ifAbsent: [ nil ] ]
		ifFalse: [ request uri firstPathSegment ].
	prefixHandler := prefixMap at: prefix ifAbsent: [ nil ].
	prefixHandler isNil
		ifFalse: [ 
			^ prefixHandler isSymbol
				ifTrue: [ self perform: prefixHandler with: request ]
				ifFalse: [ prefixHandler value: request ] ].
	^ ZnResponse notFound: request uri! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 3/31/2011 09:17'!
help: request
	"Reply with a dynamic HTML page containing links to all pages I support"

	^ ZnResponse ok: (ZnEntity html: self generateHelp)
	! !

!ZnDefaultServerDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/28/2011 22:09'!
initialize
	(prefixMap := Dictionary new)
		at: 'welcome' put: #welcome:;
		at: 'help' put: #help:;
		at: 'echo' put: #echoRequest:;
		at: 'dw-bench' put: #dwbench:;
		at: 'favicon.ico' put: #favicon:;
		at: 'random' put: #random:;
		at: 'bytes' put: #bytes:;
		at: 'status' put: #status:;
		at: 'unicode' put: #unicode:;
		at: '/' put: 'welcome'! !

!ZnDefaultServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/26/2011 13:28'!
prefixMap
	^ prefixMap ! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/31/2011 09:46'!
processes
	^ (Process allSubInstances
		reject: [ :each | each isTerminated or: [ each = Processor activeProcess ] ])
		sorted: [ :a :b | a priority >= b priority ] ! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 1/4/2011 14:52'!
random: request
	"Answer a text plain UTF-8 encoded string of random hex characters.
	Optionally take the size from the URI, as in /random/32 for 32 characters.
	Default to 64 characters with a newline at the end"
	
	| count hexString |
	count := Integer readFrom: request uri lastPathSegment ifFail: [ 64 ].
	hexString := String new: count streamContents: [ :stream |
		count - 1 timesRepeat: [ stream nextPut: '0123456789ABCDEF' atRandom ].
		stream lf ].
	^ ZnResponse ok: (ZnEntity text: hexString)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 3/31/2011 09:18'!
status: request
	"Reply with a dynamic HTML page describing the state of the server"

	^ ZnResponse ok: (ZnEntity html: self generateStatus)
	! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/1/2012 13:55'!
systemVersionInfo
	^ String streamContents: [ :stream |
			stream 
				print: SystemVersion current; 
				nextPutAll: ' - '; 
				nextPutAll: ZnConstants defaultServerString ]
	! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:40'!
unicode: request
	"Answer a Unicode Test HTML page"
	
	^ ZnResponse ok: (ZnEntity html: self generateUnicodeTest)! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/1/2012 13:54'!
vmStats
	^  [ SmalltalkImage current vm statisticsReport ]
		on: MessageNotUnderstood 
		do: [ SmalltalkImage current vmStatisticsReportString ]! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:39'!
welcome: request
	"Answer a nice HTML welcome page"
	
	| html entity |
	html := ZnConstants welcomePageHtml copy 
		replaceAll: Character cr with: Character lf.
	entity := ZnEntity html: html. 
	^ ZnResponse ok: entity! !

Object subclass: #ZnDispatcherDelegate
	instanceVariableNames: 'dispatcherDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnDispatcherDelegate commentStamp: 'SvenVanCaekenberghe 5/15/2012 16:03' prior: 0!
I am ZnDispatcherDelegate, doing a straight-forward dispatching to mapped URLs.

ZnDispatcherDelegate API:

(server := ZnServer startDefaultOn: 9090)
	delegate: (ZnDispatcherDelegate new 
		map: '/hello' to: [ :request :response | response entity: (ZnEntity html: '<h1>hello server</h1>') ];
		map: '/counter' to: [ :request :response | counterApplication handleRequest: request response: response ]).!

!ZnDispatcherDelegate methodsFor: 'accessing' stamp: 'NickAger 5/9/2011 16:25'!
dispatcherDictionary
	^ dispatcherDictionary ifNil: [ dispatcherDictionary := Dictionary new ]! !

!ZnDispatcherDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/18/2011 14:16'!
handleRequest: request
	| handler |
	handler := self dispatcherDictionary 
		at: request uri pathPrintString 
		ifAbsent: [ ^ ZnResponse notFound: request uri ].	
	^ handler 
		value: request 
		value: (ZnResponse new
			statusLine: (ZnStatusLine ok);
			headers: ZnHeaders defaultResponseHeaders;
			yourself).

! !

!ZnDispatcherDelegate methodsFor: 'public' stamp: 'NickAger 5/9/2011 16:19'!
map: relativeUrlString to: handler
	self dispatcherDictionary at: relativeUrlString put: handler! !

Object subclass: #ZnEasy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnEasy commentStamp: 'SvenVanCaekenberghe 4/21/2012 11:13' prior: 0!
I am ZnEasy, a simplified HTTP client for occasional use.

My class side implements a number of convenience methods to do common HTTP client operations.

	ZnEasy get: 'http://zn.stfx.eu/zn/small.html'.

For most requests, I return a response object. 
Some return an image Form.

	ZnEasy getPng: 'http://www.pharo-project.org/images/pharo.png'.

For my implementation I use ZnClient, a full featured HTTP client.

Part of Zinc HTTP Components.!

!ZnEasy class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/8/2011 22:16'!
client
	^ ZnClient new
		beOneShot;
		yourself! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
delete: urlObject
	^ self client
		url: urlObject;
		delete;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
delete: urlObject username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		delete;
		response
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
get: urlObject
	"ZnClient get: 'http://caretaker.wolf359.be/small.html'."
	
	^ self client
		url: urlObject;
		get;
		response
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
get: urlObject username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		get;
		response
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
getGif: urlObject
	"self getGif: 'http://homepage.mac.com/svc/ADayAtTheBeach/calculator.gif'."
	
	^ self 
		getImageOfType: ZnMimeType imageGif 
		usingParser: GIFReadWriter 
		fromUrl: urlObject 
	! !

!ZnEasy class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
getImageOfType: mimeType usingParser: parserClass fromUrl: urlObject
	| client |
	(client := self client)
		url: urlObject;
		accept: mimeType;
		enforceHttpSuccess: true;
		enforceAcceptContentType: true;
		get.
	^ (parserClass on: client entity readStream) nextImage
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/9/2012 22:17'!
getJpeg: urlObject
	"ZnEasy getJpeg: 'http://caretaker.wolf359.be/sun-fire-x2100.jpg'."
	
	^ self 
		getImageOfType: ZnMimeType imageJpeg 
		usingParser: JPEGReadWriter 
		fromUrl: urlObject 
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/9/2012 22:17'!
getPng: urlObject
	"self getPng: 'http://www.pharo-project.org/images/pharo.png'."
	"(ZnEasy getPng: 'http://chart.googleapis.com/chart?cht=tx&chl=',
		'a^2+b^2=c^2' encodeForHTTP) asMorph openInHand."
	
	^ self 
		getImageOfType: ZnMimeType imagePng 
		usingParser: PNGReadWriter 
		fromUrl: urlObject 
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
head: urlObject
	^ self client
		url: urlObject;
		head;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
head: urlObject username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		head;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
post: urlObject data: entity
	^ self client
		url: urlObject;
		entity: entity;
		post;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
post: urlObject data: entity username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		entity: entity;
		post;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
put: urlObject data: entity
	^ self client
		url: urlObject;
		entity: entity;
		put;
		response	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
put: urlObject data: entity username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		entity: entity;
		put;
		response	! !

Object subclass: #ZnEntity
	instanceVariableNames: 'contentType contentLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnEntity, an abstract HTTP Entity.

An entity has a content type (mime-type), a content length (byte length) and a contents.

Entities can encode and decode themselves to and from binary streams.

Part of Zinc HTTP Components.!

ZnEntity subclass: #ZnApplicationFormUrlEncodedEntity
	instanceVariableNames: 'fields representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnApplicationFormUrlEncodedEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnApplicationFormUrlEncodedEntity, a concrete HTTP Entity 
holding a set of fields (key value pairs) that is form URL encoded.
I am a ZnEntity.

Part of Zinc HTTP Components.!

!ZnApplicationFormUrlEncodedEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:41'!
designatedMimeType
	^ ZnMimeType applicationFormUrlEncoded ! !

!ZnApplicationFormUrlEncodedEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/16/2011 23:26'!
add: association
	self at: association key put: association value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
addAll: keyedCollection
	keyedCollection keysAndValuesDo: [ :key :value |
		self at: key put: value ].
	^ keyedCollection ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:07'!
at: key
	^ self fields at: key! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:23'!
at: key add: value
	self invalidateRepresentation.
	^ self fields at: key add: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key ifAbsent: block
	self invalidateRepresentation.
	^ self fields at: key ifAbsent: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key put: value
	self invalidateRepresentation.
	^ self fields at: key put: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:19'!
computeRepresentation
	representation := String streamContents: [ :stream | self writeRepresentationOn: stream ].
	contentLength := representation size! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 16:19'!
contents
	^ self fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:00'!
fields
	^ fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/31/2011 12:16'!
fieldsDo: block
	self fields keysAndValuesDo: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/31/2011 12:15'!
initialize
	super initialize.
	fields := ZnMultiValueDictionary new.
	self contentType: self class designatedMimeType ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:17'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:41'!
isEmpty
	^ self fields isEmpty! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty
		ifTrue: [ ^ self ].
	stream space.
	fields printElementsOn: stream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 14:44'!
readFrom: stream
	"We parse from stream, limited to content length if available."
	
	| readStream |
	representation := nil.
	self contentLength isNil
		ifTrue: [ readStream := stream ]
		ifFalse: [ (readStream := ZnLimitedReadStream on: stream limit: self contentLength) ascii ].
	fields := ZnUtils parseQueryFrom: readStream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
writeOn: stream
	"In most use cases, contentLength will have been asked first so that a representation is cached,
	there probably is no need to cache the representation generated here"
	
	representation isNil
		ifTrue: [ self writeRepresentationOn: stream ] 
		ifFalse: [ stream nextPutAll: representation ] ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'NorbertHartl 12/6/2011 17:40'!
writeRepresentationOn: stream
	self contentType charSet
		ifNotNil: [:charSet|
			ZnUtils 
				writeQueryFields: self fields 
				withTextEncoding: charSet 
				on: stream] 
		ifNil: [ 
			ZnUtils 
				writeQueryFields: self fields 
				on: stream ]! !

ZnEntity subclass: #ZnByteArrayEntity
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnByteArrayEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnByteArrayEntity, a concrete HTTP Entity based on a ByteArray.
It is used to hold binary (non textual) data.
I am a ZnEntity.

Part of Zinc HTTP Components.!

!ZnByteArrayEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:13'!
bytes: byteArray
	^ (self type: ZnMimeType applicationOctetStream length: byteArray size)
		bytes: byteArray;
		yourself ! !

!ZnByteArrayEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"I have no designated mime type, I can handle all binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes
	^ bytes! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes: anObject
	bytes := anObject! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:05'!
contents
	^ self bytes! !

!ZnByteArrayEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:53'!
isEmpty 
	^ self bytes isNil or: [ self bytes isEmpty ]! !

!ZnByteArrayEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self bytes notNil ifTrue: [ 
		stream space; print: self bytes ]! !

!ZnByteArrayEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/31/2012 13:32'!
readFrom: stream
	self contentLength isNil
		ifTrue: [ 
			self bytes: (ZnUtils readUpToEnd: stream limit: ZnConstants maximumEntitySize).
			self contentLength: self bytes size ]
		ifFalse: [ | byteArray |
			self contentLength > ZnConstants maximumEntitySize
				ifTrue: [ ZnEntityTooLarge signal ].
			byteArray := ByteArray ofSize: self contentLength.
			stream next: self contentLength into: byteArray.
			self bytes: byteArray ]! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 19:27'!
readStream
	^ self bytes readStream! !

!ZnByteArrayEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
writeOn: stream
	stream nextPutAll: self bytes! !

!ZnEntity class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 13:57'!
byteArrayEntityClass
	^ ZnByteArrayEntity ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:00'!
bytes: bytes
	^ self byteArrayEntityClass bytes: bytes! !

!ZnEntity class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/7/2012 17:35'!
concreteSubclassForType: mimeType binary: forceBinary
	"Answer the concrete ZnEntity subclass that handles mimeType"
	
	^ self allSubclasses 
		detect: [ :each | 
			each designatedMimeType = mimeType ]
		ifNone: [ 
			(mimeType isBinary or: [ forceBinary ])
				ifTrue: [ self byteArrayEntityClass ] 
				ifFalse: [ self stringEntityClass ] ]
! !

!ZnEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"Answer the mime type we are designated to handle.
	See #concreteSubclassForType: "
	
	self subclassResponsibility ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
html: html
	^ self stringEntityClass html: html! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 17:07'!
new
	^ super new initialize ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/7/2012 17:38'!
readBinaryFrom: stream usingType: mimeType andLength: length
	"Create and return a new buffered entity of mimeType and length from stream.
	The concrete subclass depends on mimeType, but textual type will remain binary"
	
	| newEntity |
	newEntity := (self concreteSubclassForType: mimeType binary: true) 
						type: mimeType length: length.
	newEntity readFrom: stream.
	^ newEntity 
! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/7/2012 17:36'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new buffered entity of mimeType and length from stream.
	The concrete subclass depends on mimeType"
	
	| newEntity |
	newEntity := (self concreteSubclassForType: mimeType binary: false) 
						type: mimeType length: length.
	newEntity readFrom: stream.
	^ newEntity 
! !

!ZnEntity class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2010 13:48'!
stringEntityClass
	^ ZnStringEntity! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
text: text
	^ self stringEntityClass text: text! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 22:16'!
type: type
	"Create a new entity with a given mime type and (currently) undetermined length"
	
	^ self new
		contentType: type;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:06'!
type: type length: length
	"Create a new entity with a given mime type and length"
	
	^ self new
		contentType: type;
		contentLength: length;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:02'!
with: object
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	You will probably want to set the mime type to something more specific"
	
	^ object isString 
		ifTrue: [ self text: object ] 
		ifFalse: [ self bytes: object ]! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 20:39'!
with: object type: type
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	The entity is set to mime type"
	
	^ (self with: object)
		contentType: type;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/11/2011 18:12'!
xml: text
	^ self stringEntityClass xml: text! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 12:50'!
close! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:17'!
contentLength 
	"Content length can be nil when it is (currently) unknown."
	
	^ contentLength! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentLength: integer
	contentLength := integer! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentType
	^ contentType ! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 19:57'!
contentType: object
	"We only allow assignment compatible with our designated mime type"
	| newType |
	newType := object asZnMimeType.
	contentType = newType 
		ifTrue: [ ^ self ]
		ifFalse: [ 
			(self class designatedMimeType isNil or: [ self class designatedMimeType = newType ])
				ifTrue: [ contentType := newType ] ]! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:04'!
contents
	self subclassResponsibility! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:25'!
hasContentLength
	^ contentLength notNil! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:26'!
hasContentType
	^ contentType notNil! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:51'!
isEmpty 
	self subclassResponsibility ! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:19'!
printContentTypeAndLengthOn: stream
	self hasContentType ifTrue: [ 
		stream print: self contentType; space ].
	self hasContentLength ifTrue: [
		stream print: self contentLength; nextPut: $B ]! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 10:56'!
printContentsOn: stream
	self printContentTypeAndLengthOn: stream! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:24'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self printContentsOn: stream.
	stream nextPut: $)! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/6/2010 22:02'!
readFrom: stream
	self subclassResponsibility ! !

!ZnEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:30'!
writeOn: stream
	self subclassResponsibility ! !

ZnEntity subclass: #ZnMultiPartFormDataEntity
	instanceVariableNames: 'parts representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMultiPartFormDataEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnMultiPartFormDataEntity, a concrete HTTP Entity 
holding multi part form data that is encoded according to specific rules.
I am a ZnEntity.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMultiPartFormDataEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:42'!
designatedMimeType
	^ ZnMimeType multiPartFormData  ! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 10:29'!
addPart: mimePart
	self invalidateRepresentation.
	self parts add: mimePart! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 20:11'!
computeRepresentation
	representation := ByteArray streamContents: [ :stream | 
		self writeRepresentationOn: (ZnBivalentWriteStream on: stream) ].
	contentLength := representation size! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 17:11'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 19:58'!
contentType: object
	"We only allow assignment compatible with our designated mime type.
	The main/sub must be equal but the parameters must be allowed to be different"
	
	| newType |
	newType := object asZnMimeType.
	self class designatedMimeType = newType
		ifTrue: [ contentType := newType ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 19:14'!
contents
	^ self parts! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/31/2011 12:13'!
generateBoundary
	^ String streamContents: [ :stream | | letters |
			stream nextPutAll: 'Boundary-Zn-'.
			letters := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.
			8 timesRepeat: [ stream nextPut: letters atRandom ] ]
	! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:17'!
getBoundary
	"Return the multipart/form-data MIME part boundary.
	This should really be there, set externally or internally.
	See #initialize and #mimeTypeWithBoundary"
	
	^ (self contentType parameterAt: 'boundary') withoutQuoting ! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/5/2010 17:17'!
initialize
	super initialize.
	self contentType: self mimeTypeWithBoundary  
	! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 19:15'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnMultiPartFormDataEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:43'!
isEmpty
	^ parts isNil or: [ parts isEmpty ]! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:19'!
mimeTypeWithBoundary
	| mimeType |
	mimeType := self class designatedMimeType copy.
	mimeType parameterAt: 'boundary' put: self generateBoundary.
	^ mimeType! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'PaulDeBruicker 4/10/2011 11:55'!
parse: bytes boundary: boundary
	| next start fullBoundary |
	fullBoundary := #(45 45) asByteArray , boundary.
	start := fullBoundary size + 1.
	next := bytes
		indexOfSubCollection: fullBoundary
		startingAt: start 
		ifAbsent: [ bytes size ].
	[ next < (bytes size - 2) ] whileTrue: [
		self addPart: (ZnMimePart readFrom: (bytes copyFrom: start + 2 to: next - 3) readStream).
		start := next + fullBoundary size.
		next := bytes
			indexOfSubCollection: fullBoundary
			startingAt: start
			ifAbsent: [ bytes size ] ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:12'!
partNamed: fieldName
	^ self partNamed: fieldName ifNone: [ self error: 'Cannot find part named ', fieldName asString ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:18'!
partNamed: fieldName ifNone: block
	parts isNil ifTrue: [ block value ].
	^ self parts detect: [ :each | each fieldName = fieldName ] ifNone: block! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 20:07'!
parts
	parts isNil ifTrue: [ parts := OrderedCollection new ].
	^ parts! !

!ZnMultiPartFormDataEntity methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 2/27/2011 20:13'!
partsDo: block
	parts isNil ifTrue: [ ^ self ].
	parts do: block  ! !

!ZnMultiPartFormDataEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty
		ifTrue: [ ^ self ].
	stream space.
	self parts printElementsOn: stream! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/31/2012 13:33'!
readFrom: stream
	"Switch to streaming implementation later on"
	
	| bytes |
	"We have to use the contentLength instance variable instead of the accessor else a representation will be computed"
	contentLength isNil
		ifTrue: [ 
			bytes := ZnUtils readUpToEnd: stream limit: ZnConstants maximumEntitySize ]
		ifFalse: [
			self contentLength > ZnConstants maximumEntitySize ifTrue: [ ZnEntityTooLarge signal ].
			bytes := ByteArray ofSize: contentLength.
			stream next: contentLength into: bytes ].
	self parse: bytes boundary: self getBoundary asByteArray! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 20:28'!
writeOn: stream
	"We always go via our representation because we need a bivalent stream"
	
	representation isNil ifTrue: [ self computeRepresentation ].
	stream nextPutAll: representation! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:24'!
writeRepresentationOn: stream
	| boundary |
	boundary := self getBoundary.
	self parts do: [ :each |
		stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: String crlf.
		each writeOn: stream.
		stream nextPutAll: String crlf ].
	stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: '--'; nextPutAll: String crlf
! !

ZnEntity subclass: #ZnStreamingEntity
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStreamingEntity commentStamp: 'SvenVanCaekenberghe 8/18/2011 14:48' prior: 0!
I am ZnStreamingEntity, a concrete HTTP Entity based on a stream.
I am a ZnEntity.

When a ZnStreamingEntity is created using #readFrom: 
its content type and length are set but no body is actually read.
Instead, stream is set to a ZnLimitedReadStream that clients can/must
use to read from.

When the client is done, either all data must be read or #consumeContent must be called.
A client should normally not close the stream.

A ZnStreamingEntity can also be instanciated with an existing read stream.
During #writeOn: this stream is copied to the output stream.

Part of Zinc HTTP Components.!

!ZnStreamingEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:23'!
designatedMimeType
	"I have no designated mime type, I can handle all content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStreamingEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 17:13'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new streaming entity of mimeType and length from stream.
	Clients can/must use the embedded ZnLimitedReadStream to read all content, 
	or call #consumeContent when they are done. They should not close the stream."
	
	^ (self type: mimeType length: length)
		readFrom: stream;
		yourself
! !

!ZnStreamingEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 12:51'!
close
	stream ifNotNil: [ 
		stream close. 
		stream := nil ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:10'!
consumeContent
	"To be improved later on using some kind of larger buffer"
	
	[ self stream atEnd ] 
		whileFalse: [ self stream next ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 12:59'!
contents
	"Since this is a non-repeatable entity, 
	#contents will #consumeContent, 
	this can thus only be done once"
	
	^ ZnUtils readUpToEnd: self stream limit: ZnConstants maximumEntitySize! !

!ZnStreamingEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 18:25'!
isEmpty 
	"We actually don't know"
	
	^ false! !

!ZnStreamingEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 19:34'!
readFrom: readStream
	self contentLength isNil
		ifTrue: [ 
			self stream: readStream ]
		ifFalse: [ | limitedStream |
			limitedStream := ZnLimitedReadStream on: readStream limit: self contentLength.
			self contentType isBinary 
				ifTrue: [ limitedStream binary ] 
				ifFalse: [ limitedStream ascii ]. 
			self stream: limitedStream ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:14'!
readStream 
	^ self stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream 
	^ stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream: readStream
	stream := readStream! !

!ZnStreamingEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/17/2010 13:43'!
writeOn: outputStream
	ZnUtils 
		streamFrom: self stream 
		to: outputStream 
		size: self contentLength.
	stream close! !

ZnEntity subclass: #ZnStringEntity
	instanceVariableNames: 'string encoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStringEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:05' prior: 0!
I am ZnStringEntity, a concrete HTTP Entity based on a String.
It is used to hold textual (non-binary) data.
I am a ZnEntity.

Optionally, an encoding is used to convert to and from bytes.
The default encoding it UTF-8.

Part of Zinc HTTP Components.!

!ZnStringEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:58'!
designatedMimeType
	"I have no designated mime type, I can handle all non-binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
html: string
	^ (self type: ZnMimeType textHtml)
		string: string;
		yourself! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
text: string
	^ (self type: ZnMimeType textPlain)
		string: string;
		yourself! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/11/2011 18:12'!
xml: string
	^ (self type: ZnMimeType applicationXml)
		string: string;
		yourself! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/2/2012 16:19'!
computeContentLength
	self string isNil ifTrue: [ ^ self ].
	self initializeEncoder.
	contentLength := encoder encodedByteCountForString: string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 10:26'!
contentLength
	contentLength isNil ifTrue: [ self computeContentLength ].
	^ contentLength! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
contents
	^ self string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:59'!
encoder
	self initializeEncoder.
	^ encoder! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
encoder: anEncoding
	(encoder notNil and: [ anEncoding ~= encoder ]) ifTrue: [ self invalidateContentLength ].
	encoder := anEncoding! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/14/2010 12:09'!
hasEncoder
	^ encoder notNil! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/25/2011 13:42'!
initializeEncoder
	| charSet newEncoder |
	self hasEncoder ifTrue: [ ^ self ].
	newEncoder := (charSet := contentType charSet) isNil
		ifTrue: [ ZnNullEncoder new ]
		ifFalse: [ ZnCharacterEncoder newForEncoding: charSet ].
	self encoder: newEncoder! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 12:20'!
invalidateContentLength
	self contentLength: nil! !

!ZnStringEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:54'!
isEmpty 
	^ self string isNil or: [ self string isEmpty ] ! !

!ZnStringEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self string notNil ifTrue: [ 
		stream space; nextPutAll: self string ]! !

!ZnStringEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/1/2011 13:03'!
readFrom: stream
	self contentLength isNil 
		ifTrue: [ self readUpToEndFrom: stream ] 
		ifFalse: [ self readLimitedFrom: stream ]! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/15/2012 19:04'!
readLimitedFrom: stream
	| readStream newString |
	self contentLength > ZnConstants maximumEntitySize 
		ifTrue: [ ZnEntityTooLarge signal ].
	readStream := ZnLimitedReadStream on: stream limit: self contentLength.
	self initializeEncoder.
	newString := String new: self contentLength streamContents: [ :stringStream |
		[ readStream atEnd ] whileFalse: [
			stringStream nextPut: (encoder nextFromStream: readStream) ] ].
	self string: newString! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
readStream
	^ self string readStream! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/31/2012 13:29'!
readUpToEndFrom: stream
	| newString limit |
	limit := ZnConstants maximumEntitySize.
	self initializeEncoder.
	newString := String streamContents: [ :stringStream |
		[ [ stream atEnd or: [ stream peek isNil ] ] whileFalse: [
			stringStream nextPut: (encoder nextFromStream: stream).
			(limit := limit - 1) < 0 ifTrue: [ ZnEntityTooLarge signal ] ] ] 
			on: ConnectionClosed 
			do: [ :exception | exception return ] ].
	self string: newString! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
string
	^ string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
string: aString
	(string notNil and: [ aString ~= string ]) ifTrue: [ self invalidateContentLength ].
	string := aString! !

!ZnStringEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/2/2010 13:09'!
writeOn: stream
	self initializeEncoder.
	self string do: [ :each | 
		encoder nextPut: each toStream: stream ]! !

Object subclass: #ZnEntityReader
	instanceVariableNames: 'headers stream streaming binary allowReadingUpToEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnEntityReader commentStamp: 'SvenVanCaekenberghe 4/7/2012 16:37' prior: 0!
I am ZnEntityReader, I help reading ZnEntities from a stream given meta data in headers.

I deal with chunking and gzip decoding.

I have several options:
	- to read streaming entities, where the client has to do the actual reading
	- to read binary entities, where textual content is not interpreted
	- to allow reading up to end, when there is no content length set

Part of Zinc HTTP Components.!

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowReadingUpToEnd
	allowReadingUpToEnd := true! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowsReadingUpToEnd
	^ allowReadingUpToEnd notNil and: [ allowReadingUpToEnd ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/7/2012 16:37'!
binary
	binary := true! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:39'!
canReadContent
	^ self hasContentLength or: [ self isChunked or: [ self allowsReadingUpToEnd ] ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:10'!
contentLength
	^ (self headers includesKey: 'Content-Length')
		ifTrue: [ self headers contentLength ]
		ifFalse: [ nil ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:11'!
contentType
	^ (self headers includesKey: 'Content-Type') 
		ifTrue: [ self headers contentType ] 
		ifFalse: [ ZnMimeType applicationOctetStream ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:35'!
hasContentLength
	^ self headers includesKey: 'Content-Length'! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
headers
	^ headers! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
headers: znHeaders
	headers := znHeaders! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/7/2012 16:37'!
isBinary
	^ binary notNil and: [ binary ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 13:10'!
isChunked
	| transferEncoding |
	transferEncoding := self headers at: 'Transfer-Encoding' ifAbsent: [ ^ false ].
	^ transferEncoding beginsWith: 'chunked'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:55'!
isGzipped
	| contentEncoding |
	contentEncoding := self headers at: 'Content-Encoding' ifAbsent: [ ^ false ].
	^ contentEncoding = 'gzip'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 16:45'!
isIdentityEncoded
	^ self isGzipped not! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:03'!
isStreaming
	^ streaming notNil and: [ streaming ]! !

!ZnEntityReader methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/1/2011 17:50'!
readEntity
	| entity |
	self canReadContent ifFalse: [ ^ nil ].
	entity := self readEntityFromStream.
	^ entity isEmpty
		ifTrue: [ nil ]
		ifFalse: [ entity ]    ! !

!ZnEntityReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/7/2012 16:33'!
readEntityFromStream
	| entity decodedEntityLength chunkedStream |
	self canReadContent ifFalse: [ ^ nil ].
	chunkedStream := self isChunked ifTrue: [ stream := ZnChunkedReadStream on: stream ] ifFalse: [ nil ].
	self isGzipped 
		ifTrue: [ 
			self hasContentLength ifTrue: [ stream := ZnLimitedReadStream on: stream limit: self contentLength ].
			decodedEntityLength := nil.
			stream := GZipReadStream on: stream ]
		ifFalse: [ decodedEntityLength := self contentLength ].
	entity := self readFrom: stream usingType: self contentType andLength: decodedEntityLength.
	self isChunked ifTrue: [ 
		self isIdentityEncoded ifTrue: [ entity contentLength: stream totalSize ].
		chunkedStream extraHeaders ifNotNil: [ :extraHeaders | self headers addAll: extraHeaders ] ].
	^ entity! !

!ZnEntityReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/7/2012 16:42'!
readFrom: readStream usingType: contentType andLength: length
	| entityClass selector |
	entityClass := self isStreaming 
		ifTrue: [ ZnStreamingEntity ]
		ifFalse: [ ZnEntity ].
	selector := self isBinary 
		ifTrue: [ #readBinaryFrom:usingType:andLength: ]
		ifFalse: [ #readFrom:usingType:andLength: ].
	^ entityClass perform: selector 
		with: readStream
		with: contentType 
		with: length! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
stream
	^ stream! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
stream: readStream
	stream := readStream ! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:31'!
streaming
	streaming := true! !

Object subclass: #ZnFixedClient
	instanceVariableNames: 'scheme host port connection username password lastUsed log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Deprecated'!

!ZnFixedClient commentStamp: 'SvenVanCaekenberghe 10/4/2011 19:02' prior: 0!
I am ZnFixedClient.
I implement the client part of the HTTP protocol. 

DEPRECATED: Please use ZnNeoClient

I talk to one fixed host:port combination, reusing the same connection for each request.
If I lose the connection, I simply reconnect. 
If something goes wrong, I do one retry.
I optionally use basic authentication.

(ZnFixedClient host: 'caretaker.wolf359.be')
	get: 'index.html';
	get: 'small.html'

Part of Zinc HTTP Components.!

ZnFixedClient subclass: #ZnExtendedFixedClient
	instanceVariableNames: 'preProcessHook postProcessHook newConnectionHook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Deprecated'!

!ZnExtendedFixedClient commentStamp: 'SvenVanCaekenberghe 10/4/2011 19:02' prior: 0!
I am ZnExtendedFixedClient, a ZnFixedClient with some hooks to extend its behavior.

DEPRECATED: Please use ZnNeoClient

Part of Zinc HTTP Components!

!ZnExtendedFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
newConnection
	| newConnection |
	newConnection := super newConnection.
	newConnectionHook notNil ifTrue: [
		newConnectionHook value: newConnection ].  
	^ newConnection ! !

!ZnExtendedFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:03'!
newConnectionHook: block
	"If not nil, block will be given each connection just after a new one gets created"
	
	newConnectionHook := block! !

!ZnExtendedFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:03'!
postProcessHook: block
	"If not nil, block will be given each response just before it is returned"
	
	postProcessHook := block! !

!ZnExtendedFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
postProcessResponse: response
	super postProcessResponse: response.  
	postProcessHook notNil 
		ifTrue: [ postProcessHook value: response ]! !

!ZnExtendedFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:03'!
preProcessHook: block
	"If not nil, block will be given each request just before it gets executed"
	
	preProcessHook := block! !

!ZnExtendedFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
preProcessRequest: request
	super preProcessRequest: request.  
	preProcessHook notNil ifTrue: [
		preProcessHook value: request ]  ! !

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:40'!
host: hostName
	^ self 
		scheme: #http 
		host: hostName! !

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:40'!
host: hostName port: port
	^ self 
		scheme: #http 
		host: hostName 
		port: port! !

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/4/2012 10:32'!
new
	self deprecated: 'Please use ZnEasy' on: '8 Nov 2011' in: 'Zinc HTTP Components'.
	^ super new! !

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:40'!
scheme: scheme host: hostName
	^ self 
		scheme: scheme 
		host: hostName 
		port: (ZnUrl defaultPortForScheme: scheme)! !

!ZnFixedClient class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:40'!
scheme: scheme host: hostName port: port
	^ self new
		scheme: scheme;
		host: hostName;
		port: port;
		yourself! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/9/2011 10:11'!
baseUrl
	"Return the base or fixed URL that we talk to"
	
	^ ZnUrl new 
		scheme: self scheme; 
		host: self host; 
		port: self port; 
		yourself ! !

!ZnFixedClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/20/2011 12:10'!
close
	connection ifNil: [ ^ self ].
	self log debug: 'Closing connection'.
	connection close. 
	connection := nil.
	lastUsed := nil! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/28/2011 15:10'!
connection
	(self isConnected and: [ self isConnectionReusable ])
		ifFalse: [ 
			self close.
			connection := self newConnection ].
	^ connection! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:05'!
delete: uri
	| request |
	request := ZnRequest delete: (self fixedUrl: uri).
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 4/20/2011 12:38'!
executeRequest: request
	"Execute request to our fixed scheme://host:port and return the response.
	Make sure that the URI inside request is as processed by #fixedUrl:"

	| initialMilliseconds response |
	initialMilliseconds := Time millisecondClockValue.
	self preProcessRequest: request.
	(response := self executeRequestOnceIgnoringErrors: request) isNil
		ifTrue: [
			response := [ self close; executeRequestOnce: request ] 
				ifCurtailed: [ self log debug: [ 'Failed' ]. self close ] ].
	self logRequest: request response: response started: initialMilliseconds.  
	^ response! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:23'!
executeRequestOnce: request
	| stream response |
	stream := self connection.
	request writeOn: stream.
	stream flush.
	response := ZnResponse readFrom: stream.
	self postProcessResponse: response.
	^ response! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/17/2011 15:36'!
executeRequestOnceIgnoringErrors: request
	^ [ self executeRequestOnce: request ] 
		on: Exception 
		do: [ :err | 
			self log debug: [ 'Once Ignoring ', err printString ].
			nil ]! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/9/2011 10:02'!
fixedUrl: uri
	"Accept either a string that we parse, or a ZnURL object,
	and make sure it points to the fixed scheme://host:port that we talk too.
	This is necessary because we try to reuse the connection."
	
	| url |
	url := uri asZnUrl.
	self host ifNil: [ self error: 'Host has not been set' ].
	url
		scheme: self scheme;
		host: self host;
		port: self port.
	^ url! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 20:41'!
get: uri
	| request |
	request := ZnRequest get: (self fixedUrl: uri).
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:06'!
host
	^ host! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:40'!
host: hostName
	host := hostName! !

!ZnFixedClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/28/2011 15:07'!
isConnected
	^ connection notNil and: [ 
		connection isConnected and: [ 
			connection isOtherEndConnected ] ]! !

!ZnFixedClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/28/2011 15:12'!
isConnectionReusable
	^ connection notNil and: [ 
		lastUsed notNil
			and: [ TimeStamp now <= (lastUsed + self maxKeepAlive) ] ]! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:05'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:16'!
logRequest: request response: response started: initialMilliseconds
	self log transaction: [
		String streamContents: [ :stream |
			request requestLine printMethodAndUriOn: stream.  
			stream space; print: response code.
			response hasEntity ifTrue: [
				stream space; print: response entity contentLength; nextPut: $B ].  
			stream space; print: (Time millisecondsSince: initialMilliseconds); nextPutAll:  'ms' ] ]! !

!ZnFixedClient methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
logToTranscript
	self log addListener: ZnTranscriptLogger new    ! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/28/2011 15:09'!
maxKeepAlive
	"Maximum age of kept alive connections that we try to reuse"
	
	^ 5 seconds! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/9/2011 10:11'!
newConnection
	self log debug: 'Opening connection'.
	^ ZnNetworkingUtils socketStreamToUrl: self baseUrl ! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/9/2011 10:06'!
port
	port isNil ifTrue: [ port := ZnUrl defaultPortForScheme: self scheme ].
	^ port! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:07'!
port: integer
	port := integer! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:05'!
post: uri data: entity
	| request |
	(request := ZnRequest post: (self fixedUrl: uri))
		entity: entity.
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:04'!
postProcessResponse: response
	lastUsed := TimeStamp now.
	response isConnectionClose
		ifTrue: [ self close ]! !

!ZnFixedClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:04'!
preProcessRequest: request
	(username notNil and: [ password notNil ]) ifTrue: [
		request setBasicAuthenticationUsername: username password: password ]! !

!ZnFixedClient methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/25/2010 20:53'!
printOn: stream
	super printOn: stream.
	self host notNil ifTrue: [
		stream nextPut: $(; nextPutAll: 'http://'; nextPutAll: self host; nextPut: $:; print: self port; nextPut: $) ]! !

!ZnFixedClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/25/2010 22:06'!
put: uri data: entity
	| request |
	(request := ZnRequest put: (self fixedUrl: uri))
		entity: entity.
	^ self executeRequest: request! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/9/2011 10:01'!
scheme
	^ scheme ifNil: [ scheme := #http ]! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/9/2011 10:01'!
scheme: symbol
	scheme := symbol! !

!ZnFixedClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 21:02'!
username: user password: pwd
	username := user.
	password := pwd! !

Object subclass: #ZnHTTPSocketFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHTTPSocketFacade commentStamp: '<historical>' prior: 0!
I am ZnHTTPSocketFacade.

On my class side I implement all necessary replacement methods of HTTPSocket.

The interface (signature and semantics) of this API is defined by HTTPSocket.

Overwriting these HTTPSocket methods with mine will effectively redirect all
HTTP client access in this Smalltalk image through Zinc HTTP Components.

Part of Zinc HTTP Components.!

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/8/2011 22:16'!
client
	^ ZnClient new
		beOneShot;
		yourself! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/7/2011 09:34'!
constructMultiPartFormDataEntity: arguments
	| entity |
	entity := ZnMultiPartFormDataEntity new.
	arguments keysAndValuesDo: [ :key :values |
		values do: [ :value |
			entity addPart: ((value isKindOf: MIMEDocument)
				ifFalse: [ 
					 ZnMimePart fieldName: key value: value ]
				ifTrue: [ | fileEntity contents |
					contents := value content isNil 
						ifTrue: [ (FileStream fileNamed: value url pathForFile) contentsOfEntireFile ] 
						ifFalse: [ value content ].
					fileEntity := ZnEntity with: contents type: value contentType.
					ZnMimePart fieldName: key fileName: value url pathForFile entity: fileEntity ]) ] ].
	^ entity
! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 21:30'!
execute: request on: stream
	"All our requests are one shot"
	
	request setConnectionClose.
	^ [ 
		request writeOn: stream.
		stream flush.
		ZnResponse readFrom: stream ] ensure: [ stream close ]! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/13/2010 21:47'!
extendHeaders: headers with: object 
	(object isNil or: [ object isEmpty ]) ifTrue: [ ^ headers ].
	object isString 
		ifTrue: [ headers addAll: (ZnHeaders readFrom: object readStream) ] 
		ifFalse: [ headers withAll: object ].
	^ headers
	
! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/30/2011 13:57'!
extendUrl: urlObject withArguments: arguments
	| url |
	url := urlObject asZnUrl.
	(arguments isNil or: [ arguments isEmpty ]) ifTrue: [ ^ url ].
	^ arguments isString
		ifTrue: [ 
			(url printString, (arguments first = $? ifTrue: [ arguments ] ifFalse: [ '?', arguments ])) asZnUrl ] 
		ifFalse: [
			url == urlObject ifTrue: [ url := url copy ].   
			arguments keysAndValuesDo: [ :key :value | 
				value do: [ :each | url queryAt: key add: each ] ].
			^ url ]! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url
	^ self 
		httpGet: url 
		accept: '*/*'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/20/2011 13:14'!
httpGet: url accept: mimeType
	^ self 
		httpGet: url 
		args: nil 
		accept: mimeType! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGet: url args: queryArguments accept: mimeType
	^ self 
		httpGet: url 
		args: queryArguments 
		accept: mimeType 
		request: nil! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 8/19/2011 11:17'!
httpGet: urlObject args: queryArguments accept: mimeType request: extraHeaders
	| client |
	(client := self client)
		url: (self extendUrl: urlObject withArguments: queryArguments);
		accept: mimeType.
	self extendHeaders: client request headers with: extraHeaders. 
	client get.
	^ self streamOrErrorStringFrom: client response uri: client request url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 8/19/2011 11:18'!
httpGet: urlObject args: queryArguments user: username passwd: password
	| client |
	(client := self client)
		url: (self extendUrl: urlObject withArguments: queryArguments);
		setBasicAuthenticationUsername: username password: password;
		get.
	^ self streamOrErrorStringFrom: client response uri: client request url! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:17'!
httpGetDocument: url
	^ self 
		httpGetDocument: url 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/20/2011 13:39'!
httpGetDocument: url accept: mimeType
	^ self 
		httpGetDocument: url 
		args: nil 
		accept: mimeType! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/14/2010 08:18'!
httpGetDocument: url args: queryArguments
	^ self 
		httpGetDocument: url 
		args: queryArguments 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 9/20/2011 13:39'!
httpGetDocument: url args: queryArguments accept: mimeType
	^ self 
		httpGetDocument: url 
		args: queryArguments 
		accept: mimeType
		request: nil! !

!ZnHTTPSocketFacade class methodsFor: 'operations get' stamp: 'SvenVanCaekenberghe 8/19/2011 11:19'!
httpGetDocument: urlObject args: queryArguments accept: mimeType request: extraHeaders
	| client |
	(client := self client)
		url: (self extendUrl: urlObject withArguments: queryArguments);
		accept: mimeType.
	self extendHeaders: client request headers with: extraHeaders. 
	client get.
	^ self mimeDocumentOrErrorStringFrom: client response uri: client request url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/15/2011 16:28'!
httpGif: url
	^ ZnEasy getGif: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/15/2011 16:28'!
httpJpeg: url
	^ ZnEasy getJpeg: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations images' stamp: 'SvenVanCaekenberghe 9/15/2011 16:28'!
httpPng: url
	^ ZnEasy getPng: url! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 8/19/2011 11:28'!
httpPost: urlObject args: arguments accept: mimeType
	| client |
	(client := self client)
		url: urlObject; 
		accept: mimeType;
		formAddAll: arguments;
		post.
	^ self streamOrErrorStringFrom: client response uri: client request url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 8/19/2011 11:34'!
httpPost: urlObject args: arguments user: username passwd: password
	| client |
	(client := self client)
		url: urlObject; 
		setBasicAuthenticationUsername: username password: password;
		formAddAll: arguments;
		post.
	^ self mimeDocumentOrErrorStringFrom: client response uri: client request url

! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:05'!
httpPostDocument: url args: arguments
	^ self 
		httpPostDocument: url 
		args: arguments 
		accept: 'application/octet-stream'! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 9/17/2010 20:05'!
httpPostDocument: url args: arguments accept: mimeType
	^ self 
		httpPostDocument: url 
		args: arguments 
		accept: mimeType 
		request: nil! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 8/19/2011 11:26'!
httpPostDocument: urlObject args: arguments accept: mimeType request: extraHeaders
	| client |
	(client := self client)
		url: urlObject; 
		accept: mimeType;
		formAddAll: arguments.
	self extendHeaders: client request headers with: extraHeaders. 
	client post.
	^ self mimeDocumentOrErrorStringFrom: client response uri: client request url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations post' stamp: 'SvenVanCaekenberghe 8/19/2011 11:24'!
httpPostMultipart: urlObject args: arguments accept: mimeType request: extraHeaders
	| client |
	(client := self client)
		url: urlObject; 
		accept: mimeType;
		entity: (self constructMultiPartFormDataEntity: arguments).
	self extendHeaders: client request headers with: extraHeaders. 
	client post.
	^ self mimeDocumentOrErrorStringFrom: client response uri: client request url
! !

!ZnHTTPSocketFacade class methodsFor: 'operations put' stamp: 'SvenVanCaekenberghe 8/19/2011 11:22'!
httpPut: string to: urlObject user: username passwd: password 
	"We're implementing the weird semantics of the original ;-)"

	| client |
	(client := self client)
		url: urlObject; 
		setBasicAuthenticationUsername: username password: password; 
		entity: (ZnEntity with: string type: ZnMimeType applicationOctetStream);
		put.
	^ String streamContents: [ :stringStream | client response writeOn: stringStream ]! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2011 21:33'!
mimeDocumentOrErrorStringFrom: response uri: uri
	^ (#(200 201) includes: response statusLine code)
		ifTrue: [
			MIMEDocument
				contentType: response contentType asString
				content: response entity contents 
				url: uri asString ]
		ifFalse: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'HTTP request for '; print: uri; nextPutAll: ' failed with '.
				response statusLine writeOn: stream ] ]
		! !

!ZnHTTPSocketFacade class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 20:42'!
streamOrErrorStringFrom: response uri: uri
	^ (#(200 201) includes: response statusLine code)
		ifTrue: [
			(RWBinaryOrTextStream with: response entity contents) 
				reset; 
				yourself ]
		ifFalse: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'HTTP request for '; print: uri; nextPutAll: ' failed with '.
				response statusLine writeOn: stream ] ]
		! !

Object subclass: #ZnHeaders
	instanceVariableNames: 'headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHeaders commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:57' prior: 0!
I am ZnHeaders.
I am a collection of HTTP headers.
I can be used for generating and parsing.

Header names are normalized and used case insensitive.
Header values can be multi-valued.

Part of Zinc HTTP Components.!

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 21:12'!
defaultRequestHeaders
	^ self new
		at: 'User-Agent' put: ZnConstants defaultUserAgent;
		at: 'Accept' put: '*/*';
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/31/2011 08:54'!
defaultResponseHeaders
	^ self new
		at: 'Server' put: ZnConstants defaultServerString;
		at: 'Date' put: ZnUtils httpDate;
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 17:07'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/27/2011 14:41'!
requestHeadersFor: url
	| headers |
	headers := self defaultRequestHeaders.
	headers request: url.
	^ headers! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 21:45'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:08'!
acceptEntityDescription: entity
	"Take over the content type and length from entity"
	
	entity 
		ifNotNil: [
			self
				at: 'Content-Type' put: entity contentType printString;
				at: 'Content-Length' put: entity contentLength printString ]
		ifNil: [
			self 
				removeKey: 'Content-Type' ifAbsent: [];
				removeKey: 'Content-Length' ifAbsent: [] ] ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:25'!
addAll: keyedCollection
	"Note that we use #at:add:"
	
	keyedCollection keysAndValuesDo: [ :key :value |
		self at: key add: value ].
	^ keyedCollection ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:25'!
at: headerName
	"Return the value stored under headerName,
	a String for single-valued headers, an Array of Strings for multi-valued headers"
	
	^ self headers at: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 17:27'!
at: headerName add: value
	"Store value under headerName, optionally turning it into a multi-valued header 
	when a value was already present"
	
	^ self headers at: (self normalizeHeaderKey: headerName) add: value 
		
! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
at: headerName ifAbsent: block
	"Like #at: but executes block when the headerName is not found"
	
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers at: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'PaulDeBruicker 5/12/2012 10:02'!
at: headerName ifPresent: block
	"Like #at: but executes block when the headerName is found"

	^ self headers at: (self normalizeHeaderKey: headerName) ifPresent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:26'!
at: headerName put: value
	"Store value under headerName, replace existing entries"
	
	^ self headers at: (self normalizeHeaderKey: headerName) put: value! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:49'!
at: headerName put: value ifPresentMerge: binaryBlock
	"Store value under headerName, when there is an existing entry,
	stored the result of evaluating binaryBlock with old and new value"
	
	| normalizedKey existingValue newValue |
	normalizedKey := self normalizeHeaderKey: headerName.
	existingValue := self headers at: normalizedKey ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [ binaryBlock value: existingValue value: value ].
	^ self headers at: normalizedKey put: newValue 
		

! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/20/2011 13:48'!
contentLength
	"We allow multiple content-length headers provided they are identical."
	
	| value |
	(value := self headers at: 'Content-Length') isString
		ifFalse: [
			value asSet size = 1
				ifTrue: [ value := value first ]
				ifFalse: [ self error: 'Multiple, different Content-Length headers are not allowed' ] ].
	^ Integer readFrom: value ifFail: [ self error: 'Illegal HTTP Content Length' ] ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:27'!
contentType
	^ ZnMimeType fromString: (self headers at: 'Content-Type')! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/22/2012 10:44'!
extendHeaderAt: key from: line
	"The value of a continuation header line is concatenated, 
	keeping the whitespace, but without the CRLF"
	
	self at: key put: (self at: key), line
	! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:12'!
hasContentLength
	^ self includesKey: 'Content-Length'! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:11'!
hasContentType
	^ self includesKey: 'Content-Type'! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/2/2012 17:24'!
headers
	headers isNil ifTrue: [ headers := ZnMultiValueDictionary new ].
	^ headers! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:14'!
headersDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self isEmpty ifTrue: [ ^ self ].
	self headers keysAndValuesDo: [ :headerKey :headerValue |
		headerValue isArray
			ifTrue: [ 
				headerValue do: [ :each | 
					block value: headerKey value: each ] ]
			ifFalse: [
				block value: headerKey value: headerValue ] ]! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:13'!
includesKey: headerName
	^ self isEmpty not and: [ self headers includesKey: (self normalizeHeaderKey: headerName) ]
	! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:29'!
isDescribingEntity
	"Do I include enough information to describe an entity (i.e. content length and type) ?"
	
	^ (self headers includesKey: 'Content-Type') 
		and: [ self headers includesKey: 'Content-Length' ]
! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/6/2010 17:09'!
isEmpty 
	^ headers isNil or: [ self headers isEmpty ]! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/26/2010 18:13'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self headersDo: block! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/10/2010 20:04'!
normalizeHeaderKey: string
	"Test string to see if it has proper header key capitalization,
	if true, return string, if not, return a properly capitalized copy"
	
	^ (ZnUtils isCapitalizedString: string)
		ifTrue: [ string ] 
		ifFalse: [ ZnUtils capitalizeString: string ] ! !

!ZnHeaders methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:20'!
printOn: stream
	super printOn: stream.
	self isEmpty ifFalse: [ self headers printElementsOn: stream ]
	! !

!ZnHeaders methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/22/2012 10:42'!
readFrom: stream
	| line reader |
	reader := ZnLineReader on: stream.
	[ (line := reader nextLine) isEmpty ] whileFalse: [ | key |
		key := self readOneHeaderFrom: line readStream.
		"Continuation header lines start with a space or tab"
		[ stream atEnd not and: [ ' 	' includes: stream peek ] ]
			whileTrue: [ 
				self extendHeaderAt: key from: reader nextLine ] ] ! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/22/2012 10:27'!
readOneHeaderFrom: stream
	| key value |
	key := stream upTo: $:.
	[ stream peek == Character space ] whileTrue: [ stream next ].
	value := stream upToEnd.
	self at: key add: value.
	^ key! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 13:36'!
removeKey: headerName	
	^ self headers removeKey: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
removeKey: headerName ifAbsent: block
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers removeKey: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/12/2011 13:08'!
request: url
	"Setup the receiver to request url"
	
	(url isNil or: [ url hasHost not ]) ifTrue: [ ^ self ].  
	self at: 'Host' put: url authority.
	(ZnNetworkingUtils proxyAuthorizationHeaderValueToUrl: url)
		ifNotNil: [ :value | self at: 'Proxy-Authorization' put: value ].    
  ! !

!ZnHeaders methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:39'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	self headersDo: [ :key :value |
		writeStream nextPutAll: key; nextPut: $:; space; nextPutAll: value; nextPutAll: String crlf ]! !

Object subclass: #ZnLineReader
	instanceVariableNames: 'stream buffer position limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnLineReader commentStamp: 'SvenVanCaekenberghe 9/27/2010 17:31' prior: 0!
I am ZnLineReader and I help reading CRLF terminated lines from a stream.
I can work on ascii and binary streams doing conversions as necessary.
I limit lines to a maximum length.
I work with a reusable buffer.

Part of Zinc HTTP Components.!

!ZnLineReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/27/2010 17:15'!
on: stream
	^ self new
		on: stream;
		yourself! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/28/2011 14:36'!
growBuffer
	| newBuffer |
	buffer size >= limit 
		ifTrue: [
			(ZnLineTooLong limit: limit) signal ].
	newBuffer := String new: buffer size * 2.
	newBuffer replaceFrom: 1 to: buffer size with: buffer.
	buffer := newBuffer ! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 17:30'!
limit: size
	limit := size! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:05'!
nextLine
	self reset.
	[ self processNext isNil ] whileFalse.
	^ buffer copyFrom: 1 to: position ! !

!ZnLineReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 17:51'!
on: readStream
	stream := readStream.
	buffer := String new: 64.
	self 
		limit: ZnConstants maximumLineLength; 
		reset! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2011 14:07'!
processNext
	| item |
	stream atEnd ifTrue: [ ^ nil ].
	(item := stream next) ifNil: [ ^ nil ].
	stream isBinary ifTrue: [ item := item asCharacter ].
	item == Character lf
		ifTrue: [
			(position > 0 and: [ (buffer at: position) == Character cr ])
				ifTrue: [
					position := position - 1.
					^ nil ] ].
	self store: item.
	^ item
			
	! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 17:29'!
reset
	position := 0! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 18:43'!
store: item
	position >= buffer size ifTrue: [ self growBuffer ].
	position := position + 1.
	buffer at: position put: item		
	! !

Object subclass: #ZnLogListener
	instanceVariableNames: 'enabled filter lock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogListener commentStamp: 'SvenVanCaekenberghe 8/18/2011 14:47' prior: 0!
I am ZnLogListener.

I am an abstract class, subclasses should implement #event:.
I add filtering and monitor for exclusive access.

Part of Zinc HTTP Components.!

ZnLogListener subclass: #ZnFileLogger
	instanceVariableNames: 'stream pathName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnFileLogger commentStamp: 'SvenVanCaekenberghe 3/29/2011 09:37' prior: 0!
I am ZnFileLogger, a ZnLogListener that appends to a file.

Part of Zinc HTTP Components.!

!ZnFileLogger class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/29/2011 12:32'!
onFileNamed: pathName
	^ self new
		initializeOn: pathName;
		yourself  ! !

!ZnFileLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/20/2011 12:21'!
close
	self stream ifNil: [ ^ self ].
	self event: (ZnLogEvent info: 'Closing ', self printString).
	stream close. 
	stream := nil! !

!ZnFileLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:04'!
event: logEvent
	logEvent printOn: self stream.
	self stream lf! !

!ZnFileLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 12:34'!
initializeOn: aPathName 
	self pathName: aPathName.
	self open  ! !

!ZnFileLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/2/2012 16:13'!
open
	| fileStream |
	self close.
	fileStream := ZnFileSystemUtils fileStreamFor: self pathName.
	fileStream setToEnd.
	self stream: fileStream.
	self event: (ZnLogEvent info: 'Opening ', self printString) ! !

!ZnFileLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:33'!
pathName 
	^ pathName ! !

!ZnFileLogger methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 12:33'!
pathName: aPathName 
	pathName := aPathName  ! !

!ZnFileLogger methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:44'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self pathName.  
	aStream nextPut: $)! !

!ZnFileLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 12:03'!
stream
	^ stream! !

!ZnFileLogger methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 12:35'!
stream: aStream
	stream := aStream! !

!ZnLogListener methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 10:11'!
acceptEvent: logEvent
	filter isNil ifTrue: [ ^ true ].
	filter isSymbol ifTrue: [ ^ logEvent category = filter ].
	^ filter value: logEvent category  ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:30'!
allow: collection
	self filter: [ :category | collection includes: category ]! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:29'!
allowAll
	self filter: nil! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:38'!
enabled 
	^ enabled ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:38'!
enabled: boolean
	enabled := boolean! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:18'!
event: logEvent
	self subclassResponsibility ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:09'!
filter
	^ filter! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:09'!
filter: symbolOrBlock
	filter := symbolOrBlock ! !

!ZnLogListener methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:46'!
handleEvent: logEvent
	(self enabled and: [ self acceptEvent: logEvent ])
		ifTrue: [ 
			lock critical: [ self event: logEvent ] ]  ! !

!ZnLogListener methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 11:45'!
initialize 
	lock := Monitor new.
	self enabled: true  ! !

ZnLogListener subclass: #ZnMemoryLogger
	instanceVariableNames: 'logEvents limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnMemoryLogger commentStamp: 'SvenVanCaekenberghe 3/29/2011 09:52' prior: 0!
I am ZnMemoryLogger, a ZnLogListener that collects ZnLogEvents in memory.

Part of Zinc HTTP Components.!

!ZnMemoryLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/29/2011 11:06'!
clear
	logEvents ifNotNil: [ logEvents removeAll ]  ! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:10'!
event: logEvent
	(self limit notNil and: [ self logEvents size >= self limit ])
		ifTrue: [ self shrink ].  
	self logEvents add: logEvent ! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:06'!
limit
	^ limit! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:06'!
limit: count
	limit := count! !

!ZnMemoryLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 11:05'!
logEvents 
	^ logEvents ifNil: [ logEvents := OrderedCollection new ]! !

!ZnMemoryLogger methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 11:15'!
shrink
	(self limit / 10) rounded timesRepeat: [ self logEvents removeFirst ]  ! !

ZnLogListener subclass: #ZnStandardOutputLogger
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnStandardOutputLogger commentStamp: 'SvenVanCaekenberghe 3/4/2012 20:02' prior: 0!
I am ZnStandardOutputLogger, a ZnLogListener that appends to the OS level standard output stream.

Part of Zinc HTTP Components.!

!ZnStandardOutputLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/4/2012 20:03'!
event: logEvent
	logEvent printOn: self stream.
	self stream lf! !

!ZnStandardOutputLogger methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 3/4/2012 20:02'!
initialize
	super initialize.
	! !

!ZnStandardOutputLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/4/2012 20:04'!
stream
	^ stream ifNil: [
		(stream := FileStream stdout)
			wantsLineEndConversion: true;
			converter; "Force lazy initialization of converter"
			yourself ]! !

ZnLogListener subclass: #ZnTranscriptLogger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnTranscriptLogger commentStamp: 'SvenVanCaekenberghe 3/29/2011 09:38' prior: 0!
I am ZnTranscriptLogger, a ZnLogListener that appends to the Transcript.

Part of Zinc HTTP Components.!

!ZnTranscriptLogger methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 10:37'!
event: logEvent
	Transcript cr; show: logEvent! !

Object subclass: #ZnLogSupport
	instanceVariableNames: 'announcer enabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogSupport commentStamp: 'SvenVanCaekenberghe 3/29/2011 11:39' prior: 0!
I am ZnLogSupport, I help HTTP servers and clients to manage ZnLogEvents.

To use the logging framework, instanciate your own log support: 

	log := ZnLogSupport new.

and use it to log messages:

	log info: 'Started server'.
	log debug: [ 'Created ', someComplexObject printString ].

To view the log, add a log listener:

	log addListener: ZnTranscriptLogger new.
	
Make sure to enable your log support as it is off by default:

	log enabled: true.

Part of Zinc HTTP Components.!

!ZnLogSupport class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/29/2011 11:40'!
newEnabled
	^ self new
		enabled: true;
		yourself  ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2011 12:28'!
addListener: logListener
	self enable.  
	self announcer on: ZnLogEvent send: #handleEvent: to: logListener ! !

!ZnLogSupport methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/28/2011 22:04'!
announcer
	^ announcer ifNil: [ announcer := Announcer new ]! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
debug: message
	self enabled 
		ifTrue: [  
			self announcer announce: (ZnLogEvent debug: message value) ]
	! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2011 12:28'!
disable
	self enabled: false! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2011 12:28'!
enable
	self enabled: true! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:29'!
enabled
	^ enabled ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:30'!
enabled: boolean
	enabled := boolean ! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
info: message
	self enabled 
		ifTrue: [
			self announcer announce: (ZnLogEvent info: message value) ]
	! !

!ZnLogSupport methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2011 12:28'!
initialize 
	self disable! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2011 12:28'!
removeListener: logListener
	self announcer unsubscribe: logListener.
	self announcer numberOfSubscriptions isZero 
		ifTrue: [ self disable ]! !

!ZnLogSupport methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 09:50'!
transaction: message
	self enabled 
		ifTrue: [  
			self announcer announce: (ZnLogEvent transaction: message value) ]
	! !

Object subclass: #ZnMessage
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMessage commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:46' prior: 0!
I am ZnMessage, an abstract class representing an HTTP Message.

Part of Zinc HTTP Components.!

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/7/2012 16:41'!
readBinaryFrom: stream
	^ self new
		readBinaryFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 12:18'!
readHeaderFrom: stream
	^ self new
		readHeaderFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readStreamingFrom: stream
	^ self new
		readStreamingFrom: stream;
		yourself! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 10:37'!
clearEntity
	"Set my entity to nil. 
	Unlike #resetEntity:, my content type and length remain unchanged."

	entity ifNotNil: [
		entity close.
		entity := nil ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:17'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:15'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:03'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:05'!
cookies
	self subclassResponsibility ! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
entity
	^ entity! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:09'!
entity: object
	"Set my entity to object. Unless I already am describing an entity using my content type and length
	set my content type and length as defined by object."

	entity := object.
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMessage methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 19:02'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		yourself! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:02'!
hasEntity
	^ self entity notNil! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 20:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMessage methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:04'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:51'!
isConnectionClose
	| value |
	value := self headers at: 'Connection' ifAbsent: [ ^ false ].
	^ value sameAs: 'close'! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:51'!
isConnectionKeepAlive
	| value |
	value := self headers at: 'Connection' ifAbsent: [ ^ false ].
	^ value sameAs: 'keep-alive'! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/7/2012 16:42'!
readBinaryFrom: stream
	| entityReader |
	self readHeaderFrom: stream.
	(entityReader := self entityReaderOn: stream)
		binary.
	self entity: entityReader readEntity! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/28/2011 10:31'!
readEntityFrom: stream
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/28/2011 10:31'!
readFrom: stream
	self readHeaderFrom: stream.
	self readEntityFrom: stream
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:32'!
readHeaderFrom: stream
	self headers: (ZnHeaders readFrom: stream)
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 12:17'!
readStreamingFrom: stream
	| entityReader |
	self readHeaderFrom: stream.
	(entityReader := self entityReaderOn: stream)
		streaming.
	self entity: entityReader readEntity
	! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:26'!
resetEntity: object
	"Set my entity to object. Always set my content type and length as defined by object,
	even if I am already describing an entity using my content type and length.
	See also: #clearEntity"

	(entity ~= object and: [ entity notNil ]) 
		ifTrue: [ entity close ].
	entity := object.
	self headers acceptEntityDescription: object! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:32'!
setConnectionClose
	self headers at: 'Connection' put: 'close'! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 13:58'!
setConnectionKeepAlive
	self headers at: 'Connection' put: 'keep-alive'! !

!ZnMessage methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:34'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self headers writeOn: bivalentWriteStream.
	bivalentWriteStream nextPutAll: String crlf.
	self hasEntity ifTrue: [ self entity writeOn: bivalentWriteStream ]! !

ZnMessage subclass: #ZnRequest
	instanceVariableNames: 'requestLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequest commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:58' prior: 0!
I am ZnRequest, representing an HTTP Request, 
consisting of a request line, headers and an optional entity (body).
I am a ZnMessage.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
delete: urlObject
	^ self method: #DELETE url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/27/2011 14:57'!
empty
	^ self new
		requestLine: ZnRequestLine empty;
		headers: ZnHeaders defaultRequestHeaders;
		yourself! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:00'!
get: urlObject
	^ self method: #GET url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
head: urlObject
	^ self method: #HEAD url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 14:01'!
method: method url: urlObject
	| url |
	url := urlObject asZnUrl.
	^ self new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (ZnHeaders requestHeadersFor: url);
		yourself! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
post: urlObject
	^ self method: #POST url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
put: urlObject
	^ self method: #PUT url: urlObject! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/17/2011 16:28'!
accept
	^ self headers at: 'Accept'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:05'!
authorization
	^ self headers at: 'Authorization'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:12'!
basicAuthentication
	| authorization username password |
	authorization := self authorization findTokens: ' '.
	(authorization size = 2 and: [ authorization first = 'Basic' ]) ifFalse: [ 
		^ NotFound signal: 'Incomplete Basic Authentication' ].
	authorization := (ZnUtils decodeBase64: authorization second) findTokens: ':'.
	authorization size = 2 ifFalse: [
		^ NotFound signal: 'Incomplete Basic Authentication' ].
	^ authorization! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:41'!
cookies
	| value cookies |
	value := self headers at: 'Cookie' ifAbsent: [ ^ #() ].
	cookies := value isString
		ifTrue: [ value findTokens: #($;) ]
		ifFalse: [ value ].
	^ cookies collect: [ :each | ZnCookie fromString: each ]! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
isHttp10
	^ self requestLine isHttp10 ! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
isHttp11
	^ self requestLine isHttp11! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:35'!
method
	^ self requestLine method ! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/27/2011 14:45'!
method: method
	self requestLine method: method! !

!ZnRequest methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:00'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self requestLine printMethodAndUriOn: stream.
	stream nextPut: $)! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:33'!
readHeaderFrom: stream
	self requestLine: (ZnRequestLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine
	^ requestLine ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine: object
	requestLine := object! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 20:00'!
setAccept: object
	self headers at: 'Accept' put: object asString! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:28'!
setAcceptEncodingGzip
	self headers at: 'Accept-Encoding' put: 'gzip'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:04'!
setAuthorization: authorization
	self headers at: 'Authorization'  put: authorization! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:05'!
setBasicAuthenticationUsername: username password: password
	(username isNil | password isNil)
		ifTrue: [
			self headers removeKey: 'Authorization' ifAbsent: []]
		ifFalse: [
			self setAuthorization: 'Basic ', (ZnUtils encodeBase64: (username, ':', password)) ]
! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/29/2012 19:49'!
setCookie: cookie
	self headers at: 'Cookie' put: cookie
! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/28/2011 10:56'!
setIfModifiedSince: reference
	self headers at: 'If-Modified-Since' put: (ZnUtils httpDate: reference)! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:36'!
uri
	^ self requestLine uri ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 21:48'!
url
	^ self uri ! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/27/2011 14:44'!
url: url
	self requestLine uri: url.
	self headers request: self url! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
wantsConnectionClose
	^ self isConnectionClose 
		or: [ self isHttp10 and: [ self isConnectionKeepAlive not ] ]! !

!ZnRequest methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:36'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self requestLine writeOn: bivalentWriteStream.
	super writeOn: bivalentWriteStream! !

ZnMessage subclass: #ZnResponse
	instanceVariableNames: 'statusLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnResponse commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:59' prior: 0!
I am ZnResponse, representing an HTTP Response 
consisting of a status line, headers and an optional entity (body).
I am a ZnMessage.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:00'!
badRequest: request
	^ self new
		statusLine: (ZnStatusLine badRequest);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Bad Request ', request method, ' ', request uri pathQueryFragmentPrintString, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:01'!
created: url
	| location |
	location := url asZnUrl pathQueryFragmentPrintString.
	^ self new
		statusLine: (ZnStatusLine created);
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: location;
		entity: (ZnEntity text: 'Created ', location, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 12:44'!
methodNotAllowed: request
	^ self new
		statusLine: (ZnStatusLine methodNotAllowed);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Method Not Allowed ', request method, ' ', request uri pathQueryFragmentPrintString, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 08:59'!
notFound: url
	^ self new
		statusLine: (ZnStatusLine notFound);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: 'Not Found ', url asZnUrl pathQueryFragmentPrintString, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 5/12/2012 09:54'!
notModified
	^ self new
		statusLine: ZnStatusLine notModified;
		headers: ZnHeaders defaultResponseHeaders;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:01'!
ok: entity
	^ self new
		statusLine: (ZnStatusLine ok);
		headers: ZnHeaders defaultResponseHeaders;
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:01'!
redirect: url
	| location |
	location := url asZnUrl pathQueryFragmentPrintString.
	^ self new
		statusLine: (ZnStatusLine redirect);
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: location;
		entity: (ZnEntity text: 'Redirect ', location, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:02'!
serverError: string
	^ self new
		statusLine: (ZnStatusLine internalServerError);
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity text: string, String crlf);
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:46'!
unauthorized
	^ self unauthorized: 'Basic realm=ZincHTTPComponents'! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:03'!
unauthorized: authString
	^ self new
		statusLine: (ZnStatusLine unauthorized);
		headers: ZnHeaders defaultResponseHeaders;
		setWWWAuthenticate: authString;
		entity: (ZnEntity text: 'Unauthorized', String crlf);
		yourself! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:44'!
addCookie: cookie
	self headers at: 'Set-Cookie' add: cookie fullString
! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 20:44'!
code
	^ self statusLine code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:53'!
cookies
	| value cookies |
	value := self headers at: 'Set-Cookie' ifAbsent: [ ^ #() ].
	cookies := value isString
		ifTrue: [ Array with: value ]
		ifFalse: [ value ].
	^ cookies collect: [ :each | ZnCookie fromString: each ]! !

!ZnResponse methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 19:02'!
entityReaderOn: stream
	"Reading undefined content is allowed for responses"
	
	^ (super entityReaderOn: stream)  
		allowReadingUpToEnd;
		yourself! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:03'!
isAuthenticationRequired
	^ self code = 401 ! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/14/2011 14:43'!
isCreated
	^ self code = 201! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2012 10:26'!
isError
	^self code > 399! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/28/2011 10:28'!
isInformational
	^ self code between: 100 and: 199! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/28/2011 10:27'!
isNoContent
	^ self code = 204! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/21/2011 10:45'!
isNotModified
	^ self code = 304! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:03'!
isRedirect
	^ #(301 302) includes: self code! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/28/2010 12:38'!
isSuccess
	^ #(200 201 202 204) includes: self code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:31'!
location
	^ self headers at: 'Location'! !

!ZnResponse methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 12:54'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self statusLine printCodeAndReasonOn: stream.
	self hasEntity ifTrue: [ 
		stream space.
		self entity printContentTypeAndLengthOn: stream ].
	stream nextPut: $)
	! !

!ZnResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/28/2011 10:36'!
readEntityFrom: stream
	(self isInformational or: [ self isNoContent or: [ self isNotModified ] ])
		ifFalse: [ super readEntityFrom: stream ]! !

!ZnResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:34'!
readHeaderFrom: stream
	self statusLine: (ZnStatusLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:34'!
setKeepAliveFor: request
	(request isHttp10 and: [ request isConnectionKeepAlive ])
		ifTrue: [ self setConnectionKeepAlive ]
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setLocation: uriString
	self headers at: 'Location' put: uriString! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setWWWAuthenticate: string
	self headers at: 'WWW-Authenticate' put: string! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 18:35'!
status
	^ self statusLine code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine
	^ statusLine! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine: object
	statusLine := object! !

!ZnResponse methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:36'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self statusLine writeOn: bivalentWriteStream.
	super writeOn: bivalentWriteStream! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:28'!
wwwAuthenticate
	^ self headers at: 'WWW-Authenticate'! !

Object subclass: #ZnMimePart
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMimePart commentStamp: 'SvenVanCaekenberghe 10/5/2010 19:20' prior: 0!
I am ZnMimePart.
I hold headers and an optional entity.
I am used in MultiPartMimeEncoding.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/16/2011 23:28'!
fieldName: fieldName entity: entity
	^ self new
		setContentDisposition: 'form-data;name="', fieldName, '"';
		entity: entity;
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/31/2011 20:13'!
fieldName: fieldName fileName: fileName entity: entity
	^ self new
		setContentDisposition: 'form-data;name="', fieldName, '";filename="', fileName, '"';
		entity: entity;
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2012 16:10'!
fieldName: fieldName fileNamed: fileName
	| mimeType size fileStream entity baseName |
	mimeType := ZnMimeType forFilenameExtension: (ZnFileSystemUtils extensionFor: fileName).
	fileStream := ZnFileSystemUtils oldFileStreamFor: fileName.
	mimeType isBinary ifTrue: [ fileStream binary ].
	size := ZnFileSystemUtils fileSizeFor: fileName.
	(entity := ZnStreamingEntity type: mimeType length: size)
		stream: fileStream.
	baseName := ZnFileSystemUtils baseNameFor: fileName.
	^ self fieldName: fieldName fileName: baseName entity: entity
	! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/29/2012 19:27'!
fieldName: fieldName value: fieldValue
	^ self
		fieldName: fieldName
		entity: (ZnEntity with: fieldValue)! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 19:57'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:12'!
contentDisposition
	^ self headers at: 'Content-Disposition' ifAbsent: [ nil ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2010 09:30'!
contentDispositionValues
	| contentDisposition values start semiColonIndex |
	(contentDisposition := self contentDisposition) isNil ifTrue: [ ^ #() ].
	values := OrderedCollection new: 3.
	start := 1.
	semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ].
	[ semiColonIndex isZero ] whileFalse: [
		| value |
		value := contentDisposition copyFrom: start to: semiColonIndex - 1.
		values add: (ZnUtils trimString: value).
		start := semiColonIndex + 1.
		semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ] ].
	values add: (ZnUtils trimString: (contentDisposition copyFrom: start to: contentDisposition size)).
	^ values! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
detectContentDispositionValue: aString
	self contentDispositionValues do: [ :each | 
		((each beginsWith: aString)
			and: [ each size > (aString size + 1)
			and: [ (each at: aString size + 1) = $= ] ]) ifTrue: [
				^ (each copyAfter: $=) withoutQuoting ] ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
entity
	^ entity! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:09'!
entity: object
	entity := object.
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/3/2010 16:10'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		allowReadingUpToEnd;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:03'!
fieldName
	^ self detectContentDispositionValue: 'name'! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:54'!
fieldValue
	^ self hasEntity 
		ifTrue: [ self entity contents ] 
		ifFalse: [ nil ] ! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/29/2011 14:17'!
fieldValueString
	^ self fieldValue
		ifNil: [ String empty ]
		ifNotNil: [ :value | value asString ]! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
fileName
	^ self detectContentDispositionValue: 'filename'! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
hasEntity
	^ self entity notNil! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 13:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMimePart methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMimePart methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
readFrom: stream
	self headers: (ZnHeaders readFrom: stream).
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:49'!
setContentDisposition: value
	^ self headers at: 'Content-Disposition' put: value ! !

!ZnMimePart methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
writeOn: stream
	self headers writeOn: stream.
	stream nextPutAll: String crlf.
	self hasEntity ifTrue: [ self entity writeOn: stream ]! !

Object subclass: #ZnMimeType
	instanceVariableNames: 'main sub parameters'
	classVariableNames: 'ExtensionsMap MimeTypes'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnMimeType commentStamp: 'SvenVanCaekenberghe 12/6/2011 20:52' prior: 0!
I am ZnMimeType.

I know whether I am binary or not and know my charset when applicable.

My class side offers access to some common constants.
Note that for text types we default to UTF-8 encoding.

I started life as a copy of WAMineType.

Part of Zinc HTTP Components.!

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
any
	^ self predefinedInstanceAt: #any
	! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationFormUrlEncoded
	^ self predefinedInstanceAt: #applicationFormUrlEncoded! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationJavascript
	^ self predefinedInstanceAt: #applicationJavascript! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationJson
	^ self predefinedInstanceAt: #applicationJson! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 19:59'!
applicationOctetStream
	^ self predefinedInstanceAt: #applicationOctetStream! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/27/2012 18:42'!
applicationPdf
	^ self predefinedInstanceAt: #applicationPdf! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
applicationXml
	^ self predefinedInstanceAt: #applicationXml! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 15:48'!
default
	^ self applicationOctetStream ! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/16/2010 20:29'!
forFilenameExtension: extension
	^ ExtensionsMap 
		at: extension 
		ifAbsent: [ self default ]! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/8/2012 14:17'!
fromString: aString 
	| main endOfSub sub parts parameters |
	main := aString copyUpTo: $/.
	endOfSub := aString indexOf: $;.
	endOfSub := endOfSub = 0 
		ifTrue: [ aString size ]
		ifFalse: [ endOfSub - 1 ].
	sub := aString copyFrom: main size + 2 to: endOfSub.
	endOfSub = aString size ifTrue: [ ^ self main: main sub: sub ].
	parts := (aString copyFrom: endOfSub + 1 to: aString size) subStrings: ';'.
	parameters := Dictionary new.
	parts do: [ :each | 
		parameters
			at: (each copyUpTo: $=) trimBoth 
			put: (each copyAfter: $=) trimBoth ].
	^ self 
		main: main
		sub: sub
		parameters: parameters! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
imageGif
	^ self predefinedInstanceAt: #imageGif! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
imageJpeg
	^ self predefinedInstanceAt: #imageJpeg! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
imagePng
	^ self predefinedInstanceAt: #imagePng! !

!ZnMimeType class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/16/2010 20:27'!
initialize
	self initializeMimeTypes.
	self initializeExtensionsMap! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/27/2012 18:42'!
initializeExtensionsMap 
	"To be extended later"
	
	ExtensionsMap := Dictionary new
		at: 'txt' put: self textPlain;
		at: 'text' put: self textPlain;
		at: 'htm' put: self textHtml;
		at: 'html' put: self textHtml;
		at: 'gif' put: self imageGif;
		at: 'jpg' put: self imageJpeg;
		at: 'jpeg' put: self imageJpeg;
		at: 'png' put: self imagePng;
		at: 'js' put: self applicationJavascript;
		at: 'xml' put: self applicationXml;
		at: 'css' put: self textCss;
		at: 'pdf' put: self applicationPdf;
		yourself! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/27/2012 18:41'!
initializeMimeTypes 
	"To be extended later"

	MimeTypes := Dictionary new
		at: #applicationOctetStream put: (self main: 'application' sub: 'octet-stream');
		at: #applicationJavascript put: (self main: 'application' sub: 'javascript');
		at: #applicationJson put: (self main: 'application' sub: 'json');
		at: #applicationXml put: (self main: 'application' sub: 'xml');
		at: #applicationPdf put: (self main: 'application' sub: 'pdf');
		at: #applicationFormUrlEncoded put: (self main: 'application' sub: 'x-www-form-urlencoded');
		at: #imageGif put: (self main: 'image' sub: 'gif');
		at: #imageJpeg put: (self main: 'image' sub: 'jpeg');
		at: #imagePng put: (self main: 'image' sub: 'png');
		at: #textPlain put: ((self main: 'text' sub: 'plain') setCharSetUTF8; yourself);
		at: #textHtml put: ((self main: 'text' sub: 'html') setCharSetUTF8; yourself);
		at: #textCss put: (self main: 'text' sub: 'css');
		at: #textJavascript put: (self main: 'text' sub: 'javascript');
		at: #multiPartFormData put: (self main: 'multipart' sub: 'form-data');
		at: #any put: (self main: '*' sub: '*');
		at: #text put: (self main: 'text' sub: '*');
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
main: aMainString sub: aSubString
	^ self new
		main: aMainString;
		sub: aSubString;
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
main: aMainString sub: aSubString parameters: aDictionary
	^ (self main: aMainString sub: aSubString)
		parameters: aDictionary;
		yourself! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
multiPartFormData
	^ self predefinedInstanceAt: #multiPartFormData! !

!ZnMimeType class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2011 20:43'!
predefinedInstanceAt: key
	"Look up and return a predefined instance known under key.
	I return a copy that can be modified.
	See #initializeMimeTypes for the full list."
	
	^ (self sharedInstanceAt: key) copy! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/6/2011 19:58'!
sharedInstanceAt: key
	"Look up and return a predefined instance known under key.
	See #initializeMimeTypes for the full list.
	The returned instance should not be modified."
	
	^ MimeTypes at: key! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
text
	^ self predefinedInstanceAt: #text
	! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
textCss
	^ self predefinedInstanceAt: #textCss! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:00'!
textHtml
	^ self predefinedInstanceAt: #textHtml! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:01'!
textJavascript
	^ self predefinedInstanceAt: #textJavascript! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 12/6/2011 20:01'!
textPlain
	^ self predefinedInstanceAt: #textPlain! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
= other
	^ (self class == other class)
		and: [ self main = other main
		and: [ self sub = other sub ] ]! !

!ZnMimeType methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ self! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
charSet
	^ self parameterAt: 'charset' ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:24'!
charSet: aString
	self parameters at: 'charset' put: aString! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:54'!
clearCharSet
	self removeParameter: 'charset'! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
hash
	^ self main hash bitXor: self sub hash! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
isBinary
	"answers whether the contents of a document of the receiving mime type are binary"
	| subTypes |
	self main = 'text' ifTrue: [ ^ false ].
	self = self class applicationJson ifTrue: [ ^ false ].
	self main = 'application' ifFalse: [ ^ true ].
	subTypes := self sub subStrings: '+'.
	^ subTypes noneSatisfy: [ :each | 
		#('x-javascript' 'xml') includes: each ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/2/2010 11:19'!
isCharSetUTF8
	^ self charSet notNil and: [ self charSet sameAs: 'utf-8' ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
isNonStandard
	"tests if the receiver is a non-standard mime type that is not registered with IANA"
	^ (self main beginsWith: 'x-')
		or: [ (self main beginsWith: 'X-')
		or: [ (self sub beginsWith: 'x-')
		or: [ self sub beginsWith: 'X-' ] ] ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:45'!
isVendorSpecific
	"tests if the receiver is a vendor specific mime type"
	
	^ self sub beginsWith: 'vnd.'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main
	^ main! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main: aString
	main := aString! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
matches: aMimeType
	"aMimeType is the pattern to match, it is a normal instance where main or sub can be wildcards"
	^ (aMimeType main = '*'
		or: [ aMimeType main = self main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = self sub ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 11:00'!
parameterAt: key
	^ self parameterAt: key ifAbsent: [ KeyNotFound signalFor: key ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:53'!
parameterAt: key ifAbsent: block
	parameters isNil ifTrue: [ ^ block value ].
	^ self parameters at: key ifAbsent: block! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 9/7/2010 14:50'!
parameterAt: aKey put: aValue
	self parameters at: aKey put: aValue! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2011 20:40'!
parameters
	parameters isNil ifTrue: [ parameters := SmallDictionary new ].
	^ parameters! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
parameters: aDictionary
	parameters := aDictionary! !

!ZnMimeType methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 12/6/2011 20:41'!
postCopy
	super postCopy.
	parameters := parameters copy! !

!ZnMimeType methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:49'!
printOn: stream
	stream
		nextPutAll: self main;
		nextPut: $/;
		nextPutAll: self sub.
	parameters isNil ifFalse: [ 
		parameters keysAndValuesDo: [ :key :value | 
			stream
				nextPut: $;;
				nextPutAll: key;
				nextPut: $=;
				nextPutAll: value ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:53'!
removeParameter: key
	parameters isNil ifTrue: [ ^ self ].
	^ self parameters removeKey: key ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:20'!
setCharSetUTF8
	self charSet: 'utf-8'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub
	^ sub! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub: aString
	sub := aString! !

Object subclass: #ZnMonticelloServerDelegate
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnMonticelloServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnMonticelloServerDelegate.
I am a proof of concept implementation of a basic Monticello server.
I function as a delegate for ZnServer.

ZnServer startDefaultOn: 1701.
ZnServer default delegate: ((ZnMonticelloServerDelegate new) 
									directory: (FileDirectory on: '/Users/sven/Tmp/monticello'); 
									yourself).

Part of Zinc HTTP Components.!

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:20'!
directory
	^ directory ! !

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:20'!
directory: fileDirectory
	directory := fileDirectory! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 14:49'!
handleGetMczEntry: request
	| name |
	name := request uri firstPathSegment.
	^ ((self isValidMczName: name) and: [ self directory fileExists: name ]) 
		ifTrue: [ | file entity |
			(file := self directory readOnlyFileNamed: name)
				binary.
			(entity := ZnStreamingEntity type: ZnMimeType applicationOctetStream length: file size)
				stream: file.
			ZnResponse ok: entity ]
		ifFalse: [ ZnResponse notFound: request uri ]! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
handleListMczEntries: request
	^ ZnResponse ok: (ZnEntity html: self repositoryListing)! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 14:49'!
handlePutMczEntry: request
	| name |
	name := request uri firstPathSegment.
	^ ((self isValidMczName: name) and: [ (self directory fileExists: name) not ]) 
		ifTrue: [
			self directory newFileNamed: name do: [ :file |
				file binary.
				request entity writeOn: file ].
			ZnResponse created: request uri ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnMonticelloServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/4/2011 13:42'!
handleRequest: request
	(request method = #GET)
		ifTrue: [
			^ request uri isSlash
				ifTrue: [ self handleListMczEntries: request ]
				ifFalse: [ self handleGetMczEntry: request ] ].
	^ (request method = #PUT) 
		ifTrue: [ self handlePutMczEntry: request ]
		ifFalse: [ ZnResponse badRequest: request ]! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 10:24'!
isValidMczName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcz')
				and: [ (string includes: $/) not ] ] ]! !

!ZnMonticelloServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 10:21'!
mczEntries
	^ (self directory fileNames select: [ :each | each endsWith: '.mcz' ])
		asSortedCollection! !

!ZnMonticelloServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/17/2010 11:00'!
repositoryListing
	^ String streamContents: [ :str |
		str nextPutAll: '<html><head><title>Monticello Repository</title></head>'.
		str nextPutAll: '<body><h1>Monticello Repository</h1>'.
		str nextPutAll: '<ul>'.
		self mczEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>' ].
		str nextPutAll: '</ul></body></html>' ]! !

Object subclass: #ZnNetworkingUtils
	instanceVariableNames: 'socketStreamClass secureSocketStreamClass'
	classVariableNames: 'Default DefaultSocketStreamTimeout'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnNetworkingUtils commentStamp: 'SvenVanCaekenberghe 5/2/2011 22:05' prior: 0!
I am ZnNetworkingUtils holding various utilities related to HTTP networking.

My class side is a general API.

My instance side is a factory.

Part of Zinc HTTP Components.!

!ZnNetworkingUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2011 22:08'!
default
	^ Default ifNil: [ Default := self new ]! !

!ZnNetworkingUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2011 22:08'!
default: instance
	Default := instance ! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 6/18/2011 22:44'!
defaultSocketStreamTimeout
	"Global default timeout in seconds for SocketStream IO"
	
	^ DefaultSocketStreamTimeout! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 6/18/2011 22:45'!
defaultSocketStreamTimeout: seconds
	"Global default timeout in seconds for SocketStream IO"
	
	^ DefaultSocketStreamTimeout := seconds! !

!ZnNetworkingUtils class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 6/19/2011 16:37'!
initialize 
	Default := self new.
	DefaultSocketStreamTimeout := 30! !

!ZnNetworkingUtils class methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
ipAddressToString: byteArray
	^ String streamContents: [ :stream |
			byteArray 
				do: [ :each | stream print: each ] 
				separatedBy: [ stream nextPut: $. ] ]! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/24/2011 08:05'!
listenBacklogSize
	"Server socket backlog size (number of pending connection waiting to be accepted)"
	
	^ 32! !

!ZnNetworkingUtils class methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:43'!
proxyAuthorizationHeaderValueToUrl: url
	"Answer the Proxy-Authorization header value for HTTP access to url, or nil if none"
	
	^ self default proxyAuthorizationHeaderValueToUrl: url  ! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 5/2/2011 22:40'!
serverSocketOn: port
	^ self default 
		serverSocketOn: port  ! !

!ZnNetworkingUtils class methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 3/12/2012 19:22'!
serverSocketOn: port interface: address
	^ self default 
		serverSocketOn: port interface: address ! !

!ZnNetworkingUtils class methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:41'!
shouldProxyUrl: url
	"Answer if we should proxy HTTP access to url."
	
	^ self default shouldProxyUrl: url  ! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/24/2011 08:05'!
socketBufferSize
	"Size in bytes for Sockets and SocketStream IO buffers"

	^ 4096! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 5/2/2011 22:40'!
socketStreamOn: socket
	^ self default 
		socketStreamOn: socket  ! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 8/18/2011 22:19'!
socketStreamTimeout
	"Access the current timeout in seconds for SocketStream IO"
	
	^ ZnConnectionTimeout value 
		ifNil: [ ZnNetworkingUtils defaultSocketStreamTimeout ]! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 5/2/2011 22:40'!
socketStreamToUrl: url
	^ self default
		socketStreamToUrl: url  ! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:54'!
httpProxyPassword
	"Return the password for proxy authorization"
	
	^ NetworkSystemSettings proxyPassword ! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:51'!
httpProxyPort
	"Return the port of the proxy server to use"
	
	^ NetworkSystemSettings httpProxyPort! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:54'!
httpProxyServer
	"Return the hostname of the proxy server to use"
	
	^ NetworkSystemSettings httpProxyServer! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'MarcoSchmidt 6/17/2011 15:25'!
httpProxyUser
	"Return the user for proxy authorization"
	
	^ NetworkSystemSettings proxyUser! !

!ZnNetworkingUtils methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/10/2011 13:43'!
initialize
	super initialize.
	self socketStreamClass: SocketStream.
	self secureSocketStreamClass: (Smalltalk globals at: #ZdcSecureSocketStream ifAbsent: [ nil ])! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:55'!
isProxyAuthorizationRequired
	"Does the proxy require authorization ?"
	
	^ self isProxySet and: [ NetworkSystemSettings useNetworkAuthentification ]! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:55'!
isProxySet
	"Should a proxy be used ?"
	
	^ NetworkSystemSettings useHTTPProxy ! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:49'!
proxyAuthorizationHeaderValueToUrl: url
	"Answer the Proxy-Authorization header value for HTTP access to url, or nil if none"
	
	^ (self isProxyAuthorizationRequired and: [ self shouldProxyUrl: url ])
		ifTrue: [ 'Basic ', (ZnUtils encodeBase64: (self httpProxyUser, ':', self httpProxyPassword)) ] 
		ifFalse: [ nil ]  ! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/13/2011 21:45'!
secureSocketStreamClass
	^ secureSocketStreamClass ifNil: [ 
		secureSocketStreamClass := Smalltalk globals at: #ZdcSecureSocketStream ifAbsent: [ nil ] ]! !

!ZnNetworkingUtils methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/10/2011 13:43'!
secureSocketStreamClass: anObject
	secureSocketStreamClass := anObject! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/2/2011 22:41'!
serverSocketOn: port
	| socket |
	socket := Socket newTCP.
	self setServerSocketOptions: socket.
	socket listenOn: port backlogSize: self class listenBacklogSize.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ', port printString ].
	^ socket! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 3/12/2012 19:22'!
serverSocketOn: port interface: address
	| socket |
	socket := Socket newTCP.
	self setServerSocketOptions: socket.
	socket 
		listenOn: port 
		backlogSize: self class listenBacklogSize 
		interface: address.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ', port printString ].
	^ socket! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/2/2011 22:41'!
setServerSocketOptions: socket
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: self class socketBufferSize;
		setOption: 'SO_RCVBUF' value: self class socketBufferSize ! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/2/2011 22:42'!
setSocketStreamParameters: stream
	stream 
		binary;
		shouldSignal: true;
		autoFlush: false;
		bufferSize: self class socketBufferSize;
		timeout: self class socketStreamTimeout.
! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:38'!
shouldProxyUrl: url
	"Even when a proxy is set, some URLs should not be proxied.
	There is no setting for this in the image.
	We exclude localhost access from being proxied."
	
	^ self isProxySet and: [ url isLocalHost not ]! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2011 13:47'!
socketStreamClass
	^ socketStreamClass ifNil: [ socketStreamClass := SocketStream ]! !

!ZnNetworkingUtils methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/10/2011 13:15'!
socketStreamClass: anObject
	socketStreamClass := anObject! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 11/10/2011 13:16'!
socketStreamOn: socket
	| stream |
	stream := self socketStreamClass on: socket.
	self setSocketStreamParameters: stream.  
	^ stream! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:41'!
socketStreamToHostNamed: hostName port: port
	| stream |
	stream := SocketStream openConnectionToHostNamed: hostName port: port.
	self setSocketStreamParameters: stream.  
	^ stream
	! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/10/2011 13:30'!
socketStreamToProxy
	| proxyUrl |
	"System settings do currently not support HTTPS proxies."
	(proxyUrl := ZnUrl new) 
		scheme: #http; 
		host: self httpProxyServer; 
		port: self httpProxyPort.
	^ self socketStreamToUrlDirectly: proxyUrl! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/11/2011 19:37'!
socketStreamToUrl: url
	url hasHost ifFalse: [ ZnMissingHost signal ].  
	^ (self shouldProxyUrl: url)
		ifTrue: [ self socketStreamToProxy ] 
		ifFalse: [ self socketStreamToUrlDirectly: url ]! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 11/10/2011 13:30'!
socketStreamToUrlDirectly: url
	| stream |
	stream := (self streamClassForScheme: url scheme) 
		openConnectionToHostNamed: url host
		port: url portOrDefault.  
	self setSocketStreamParameters: stream.  
	url isHttps ifTrue: [ stream connect ].  
	^ stream! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/10/2011 13:49'!
streamClassForScheme: scheme
	scheme = #http ifTrue: [ 
		^ self socketStreamClass ].
	scheme = #https ifTrue: [ 
		^ self secureSocketStreamClass ifNil: [
			self error: 'No secure socket stream class set or available' ] ].
	(ZnUnknownScheme scheme: scheme) signal! !

Object subclass: #ZnRequestLine
	instanceVariableNames: 'method uri version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequestLine commentStamp: '<historical>' prior: 0!
I am an HTTP Request Line.
I consist of an HTTP method, an URI and an HTTP protocol version.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/27/2011 14:58'!
empty
	^ self new
		version: ZnConstants defaultHTTPVersion;
		yourself ! !

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
method: method uri: uri
	^ self new
		method: method;
		uri: uri;
		version: ZnConstants defaultHTTPVersion;
		yourself ! !

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 13:50'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnRequestLine methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:30'!
isHttp10
	^ self version = 'HTTP/1.0'! !

!ZnRequestLine methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:31'!
isHttp11
	^ self version = 'HTTP/1.1'! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
method
	^ method! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:04'!
method: string
	method := ZnConstants knownHTTPMethods 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknownHttpMethod method: string) signal ] ! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printMethodAndUriOn: stream
	(self method notNil and: [ self uri notNil ])
		ifFalse: [ ^ self ].
	stream
		nextPutAll: self method;
		space.
	self uri printPathQueryFragmentOn: stream! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printOn: stream
	super printOn: stream.
	(self method notNil and: [ self uri notNil ])
		ifFalse: [ ^ self ].
	stream nextPut: $(.
	self printMethodAndUriOn: stream.
	stream nextPut: $)! !

!ZnRequestLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:25'!
readFrom: stream
	| line lineStream |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self method: (lineStream upTo: Character space).
	self uri: (lineStream upTo: Character space).
	self version: (lineStream upToEnd)! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
uri
	^ uri! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:25'!
uri: anURI
	uri := anURI asZnUrl! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 13:56'!
uriPathQueryFragment
	^ String streamContents: [ :stream | 
			self uri printPathQueryFragmentOn: stream ]! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
version
	^ version! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:04'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknownHttpVersion version: string) signal ] ! !

!ZnRequestLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 6/16/2011 22:40'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self method; space.
	(ZnNetworkingUtils shouldProxyUrl: self uri)
		ifTrue: [ self uri printOn: writeStream ]  
		ifFalse: [ self uri printPathQueryFragmentOn: writeStream ].
	writeStream space; nextPutAll: self version; nextPutAll: String crlf ! !

Object subclass: #ZnServer
	instanceVariableNames: ''
	classVariableNames: 'ManagedServers'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnServer commentStamp: 'SvenVanCaekenberghe 12/22/2011 12:01' prior: 0!
I am ZnServer, an abstract superclass of HTTP Servers.
I am a facade for controlling a default server implementation.

I delegate my public class protocol methods to #defaultServerClass.

ZnServer startDefaultOn: 1701.
ZnClient get: 'http://localhost:1701'.

Subclasses can register with me to have start/stop sent to them on System startUp/shutDown.
The default server instance will be registered automatically when it is started.

Part of Zinc HTTP Components.!

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:27'!
default
	"Return the default instance that we manage.
	Delegate to the default server class."
	
	^ self defaultServerClass default! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:03'!
defaultOn: portNumber
	"Return the default instance on a given port,
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop it, change the port if necessary.
	Delegate to the default server class."

	^ self defaultServerClass defaultOn: portNumber ! !

!ZnServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/4/2012 20:05'!
defaultServerClass
	"Return the default ZnServer subclass to use"
	
	^ ZnManagingMultiThreadedServer ! !

!ZnServer class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 6/19/2011 12:00'!
initialize
	ManagedServers := IdentitySet new.
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!ZnServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/19/2011 12:17'!
managedServers
	^ ManagedServers ifNil: [ ManagedServers := IdentitySet new ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:28'!
on: portNumber
	"Instantiate a new listener on a given port,
	send #start to it to start listening.
	Delegate to the default server class."

	^ self defaultServerClass on: portNumber ! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 6/19/2011 12:17'!
register: server
	"Arrange for server to be sent start/stop on system startUp/shutDown"
	
	self managedServers add: server! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 12/22/2011 12:22'!
shutDown
	"Our system shutDown hook: stop all servers we manage"
	
	self managedServers do: [ :each | each stop: false ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:04'!
startDefaultOn: portNumber
	"Start and return the default instance on a given port.
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop and start it, effectively restarting it.
	Change the port if necessary.
	Starting the default server will register it automatically.
	Delegate to the default server class."

	^ self defaultServerClass startDefaultOn: portNumber ! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:30'!
startOn: portNumber
	"Instanciate and return a new listener on a given port and start listening.
	Delegate to the default server class."

	^ self defaultServerClass startOn: portNumber ! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 6/19/2011 12:18'!
startUp
	"Our system startUp hook: start all servers we manage"

	self managedServers do: [ :each | each start ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:29'!
stopDefault
	"Stop the default instance, if any, and clear it.
	Return the stopped instance, if any.
	Delegate to the default server class."

	^ self defaultServerClass stopDefault ! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 8/18/2011 14:10'!
unregister: server
	"No longer send server start/stop on system startUp/shutDown"
	
	self managedServers remove: server ifAbsent: [ ]! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/13/2012 13:13'!
bindingAddress
	"Return the interface address we are (or will be) listening on.
	Nil means that we are (or will be) listening on all/any interfaces."
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/13/2012 13:15'!
bindingAddress: address
	"Set the interface address we will be listening on.
	Specify nil to listen on all/any interfaces, the default.
	Address must be a 4 element ByteArray, like #[127 0 0 1].
	Cannot be changed after the server is already running."
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 21:22'!
isListening
	"Return true when I have a valid server socket listening at the correct port"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:26'!
isRunning
	"Return true when I am running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2012 20:34'!
localUrl
	"Return a ZnUrl to access me."
	
	^ ZnUrl new
		scheme: #http;
		host: NetNameResolver loopBackName;
		port: self port;
		yourself! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
port
	"Return the integer port number we are (or will be) listening on"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
port: integer
	"Set the port number we will be listening on.
	Cannot be changed after the server is already running."
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
process
	"Return the process that is running my main listening loop.
	Will be nil when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/19/2011 12:00'!
register
	"Ask for start/stop to be sent to me on System startUp/shutDown"
	
	self class register: self! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
serverSocket
	"Return the server socket that I am using.
	Will be nil when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
start
	"Start me. I will start listening on my port for incoming HTTP connections.
	If I am running, I will first stop and thus effectively restart"
	
	self subclassResponsibility! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:19'!
stop
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	Does nothing when I am not running"
	
	self stop: true ! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:19'!
stop: unregister
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	If unregister is true, unregister me from the list of managed instances.
	Does nothing when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/19/2011 12:00'!
unregister
	"Ask for start/stop to no longer be sent to me on System startUp/shutDown"

	self class unregister: self! !

ZnServer subclass: #ZnSingleThreadedServer
	instanceVariableNames: 'port interface process serverSocket delegate authenticator reader log lastRequest lastResponse'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

ZnSingleThreadedServer class
	instanceVariableNames: 'Default'!

!ZnSingleThreadedServer commentStamp: 'SvenVanCaekenberghe 12/15/2010 15:38' prior: 0!
I am ZnSingleThreadedServer.
I can be customized with a delegate (#handleRequest:) and an authenticator (#authenticateRequest:do:).

ZnSingleThreadedServer startDefaultOn: 1701.
ZnSingleThreadedServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
ZnClient get: 'http://localhost:1701' username: 'foo' password: 'secret'.

I use ZnDefaultServerDelegate when no other delegate is set.
I am single threaded, I run in a single process.
I close connections after each request/response cycle.

Acknowledgement: basic design copied from Blackfoot.

Part of Zinc HTTP Components.!

ZnSingleThreadedServer class
	instanceVariableNames: 'Default'!

ZnSingleThreadedServer subclass: #ZnMultiThreadedServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnMultiThreadedServer commentStamp: 'SvenVanCaekenberghe 12/15/2010 15:39' prior: 0!
I am ZnMultiThreadedServer.
I inherit most features from ZnSingleThreadedServer.

ZnMultiThreadedServer startDefaultOn: 1701.
ZnMultiThreadedServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
ZnClient get: 'http://localhost:1701' username: 'foo' password: 'secret'.

I am multi threaded, I fork a new process for each incoming request.
I try to keep connections alive in each process.

Part of Zinc HTTP Components.!

ZnMultiThreadedServer subclass: #ZnManagingMultiThreadedServer
	instanceVariableNames: 'lock connections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnManagingMultiThreadedServer commentStamp: 'SvenVanCaekenberghe 4/7/2012 17:49' prior: 0!
ZnManagingMultiThreadedServer is a ZnMultiThreadedServer that manages the socket stream connections used by the worker threads that are spawned.

ZnManagingMultiThreadedServer startDefaultOn: 1701.

I keep track of all the connections that I spawn so that I can close them when I am stopped.

Part of Zinc HTTP Components.!

!ZnManagingMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/5/2011 14:14'!
closeConnections
	self connections isEmpty ifTrue: [ ^ self ].
	self lock critical: [
		self connections do: [ :each | 
			self log debug: [ 'Closing ', each printString ].
			[ each close ] 
				on: Exception 
				do: [ :exception | exception printString, ' during close' ] ].
		self connections removeAll ]! !

!ZnManagingMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/5/2011 14:17'!
closeSocketStream: socketStream
	super closeSocketStream: socketStream.
	self lock critical: [ 
		self connections 
			remove: socketStream 
			ifAbsent: [ self log debug: [ 'Could not remove ', socketStream printString, ' ignoring' ] ] ]! !

!ZnManagingMultiThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2011 14:14'!
connections
	^ connections ifNil: [ connections := OrderedCollection new ]! !

!ZnManagingMultiThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2011 14:14'!
lock
	^ lock ifNil: [ lock := Mutex new ]! !

!ZnManagingMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/5/2011 14:17'!
socketStreamOn: socket
	| socketStream |
	socketStream := super socketStreamOn: socket.
	self lock critical: [ 
		self connections add: socketStream ].
	^ socketStream! !

!ZnManagingMultiThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 9/5/2011 14:14'!
stop
	super stop.
	self closeConnections! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/5/2011 14:15'!
closeSocketStream: socketStream
	[ socketStream close ] 
		on: Exception 
		do: [ :exception | exception printString, ' during close' ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2011 09:46'!
exceptionSet: classNames
	^ classNames
		inject: ExceptionSet new
		into: [ :set :each |
			(Smalltalk includesKey: each)
				ifTrue: [ set add: (Smalltalk at: each); yourself ]
				ifFalse: [ set ] ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/4/2011 13:50'!
executeRequestResponseLoopOn: stream
	"Execute the HTTP request / response loop on stream in 3 steps
	#readRequest: #handleRequest: and #writeResponse:on:
	until the other end indicates it wants to stop or times out"
	
	| request response |
 	self log debug: 'Executing request/response loop'.
    [ (request := self readRequestSafely: stream) notNil
		ifTrue: [
			(response := self handleRequest: request) setKeepAliveFor: request.
	 		self writeResponseSafely: response on: stream ].
	request isNil or: [ request wantsConnectionClose ] ] whileFalse ! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2010 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid 
			ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/1/2012 09:00'!
readRequestSafely: stream
	"Read request from stream, returning nil when the connection is closed or times out"
	
	^ [ self readRequest: stream ]
		on: (self exceptionSet: #(ConnectionClosed ConnectionTimedOut PrimitiveFailed
										ZnParseError ZnTooManyDictionaryEntries ZnEntityTooLarge))
		do: [ :exception |
			self log debug: [ exception printString, ' while reading request' ]. 
			nil ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/4/2011 12:38'!
serveConnectionsOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeRequestResponseLoopOn: on it"
		 
	| stream socket |
	socket := (listeningSocket waitForAcceptFor: self acceptWaitTimeout) ifNil: [
		^ self noteAcceptWaitTimedOut ].
	stream := self socketStreamOn: socket.
	[ [ [ self executeRequestResponseLoopOn: stream ]
		ensure: [ self log debug: 'Closing stream'. self closeSocketStream: stream ] ]
			ifCurtailed: [ self log debug: 'Destroying socket'. socket destroy ] ]
				forkAt: Processor lowIOPriority
				named: self workerProcessName ! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/13/2010 16:33'!
workerProcessName
	^ String streamContents: [ :stream |
		stream nextPutAll: self class name; nextPutAll: ' HTTP worker' ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2011 09:41'!
writeResponseSafely: response on: stream
	"Write response to stream, when the connection is closed or times out we ignore this"
	
	[ self writeResponse: response on: stream ]
		on: (self exceptionSet: #(ConnectionClosed ConnectionTimedOut PrimitiveFailed))
		do: [ :exception |
			self log debug: [ exception printString, ' while writing response' ] ]! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:30'!
default
	"Return the default instance that we manage.
	Note that this is a class instance variable."
	
	^ Default! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:59'!
defaultOn: portNumber
	"Return the default instance on a given port,
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop it, change the port if necessary."

	^ Default 
		ifNil: [
			Default := self on: portNumber ] 
		ifNotNil: [ 
			Default stop; port: portNumber; yourself ]! !

!ZnSingleThreadedServer class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 4/20/2011 12:12'!
initialize
	Default := nil! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:34'!
on: aNumber
	"Instantiate and return a new listener on a given port,
	send #start to it to start listening."

	^ self new 
		port: aNumber; 
		yourself! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:04'!
startDefaultOn: portNumber
	"Start and return the default instance on a given port.
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop and start it, effectively restarting it.
	Change the port if necessary.
	Starting the default server will register it automatically."

	^ (self defaultOn: portNumber) 
		start; 
		yourself! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:37'!
startOn: portNumber
	"Instanciate and return a new listener on a given port and start listening."

	^ (self on: portNumber) 
		start;
		yourself! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:59'!
stopDefault
	"Stop and unregister the Default instance, if any, and clear it.
	Return the stopped instance, if any."
	
	| server |
	(server := Default) ifNotNil: [ 
		Default stop.  
		Default := nil ].
	^ server! !

!ZnSingleThreadedServer methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/30/2011 19:15'!
acceptWaitTimeout
	"How many seconds to we wait for a server socket listening for an accept ?"
	
	^ 300! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2010 13:58'!
authenticateRequest: request do: block
	"Validate request and execute block.
	When our authenticator is not nil, pass the responsability"

	^ self authenticator isNil
		ifTrue: [ block value ]
		ifFalse: [ self authenticator authenticateRequest: request do: block ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:50'!
authenticator
	"Return the optional authenticator,
	the object that will be sent #authenticateRequest:do:
	to authenticate or refuse the requests. 
	When authentication succeeds, the block should be executed,
	when authentication fails, a appropriate response should be returned.
	If there is no authenticator, all requests will pass"
	
	^ authenticator ! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:48'!
authenticator: object
	"Set the object that will be sent #authenticateRequest:do:
	to authenticate or refuse the requests. Can be nil.
	When authentication succeeds, the block should be executed,
	when authentication fails, a appropriate response should be returned"

	authenticator := object! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/13/2012 13:12'!
bindingAddress
	"Return the interface address we are (or will be) listening on.
	Nil means that we are (or will be) listening on all interfaces."
	
	^ interface! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/13/2012 13:15'!
bindingAddress: address
	"Set the interface address we will be listening on.
	Specify nil to listen on all/any interfaces, the default.
	Address must be a 4 element ByteArray, like #[127 0 0 1].
	Cannot be changed after the server is already running."
	
	interface := address! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2011 18:51'!
closeDelegate
	(delegate notNil and: [ delegate respondsTo: #close ])
		ifTrue: [ 
			self log debug: 'Closing delegate'.
			delegate close ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 15:32'!
delegate
	"Return the optional delegate, 
	the object that will be sent #handleRequest: to handle a request and produce a response.
	The default delegate is ZnDefaultServerDelegate"
	
	delegate isNil ifTrue: [ delegate := ZnDefaultServerDelegate new ].
	^ delegate! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:48'!
delegate: object
	"Set the delegate to object. Can be nil.
	This will be sent #handleRequest: to handle a request and produce a response"
	
	delegate := object! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 13:05'!
executeOneRequestResponseOn: stream
	"Execute one single HTTP request / response cycle on stream in 3 steps:
	#readRequest: #handleRequest: and #writeResponse:on:
	Indicate that we intend to close the connection afterwards"

	| request response |
	self log debug: 'Executing 1 request/response cycle'.
	request := self readRequest: stream.
	(response := self handleRequest: request) setConnectionClose.
	self writeResponse: response on: stream! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'PaulDeBruicker 4/19/2012 16:29'!
handleRequest: request
	"Handle request and return a response.
	If we have a delegate, pass the responsability.
	If we have no delegate, we return a page not found"

	| response initialMilliseconds |

	initialMilliseconds := Time millisecondClockValue.
	response := self
		authenticateRequest: request
		do: [ 
			self delegate isNil
				ifTrue: [ ZnResponse notFound: request uri ]
				ifFalse: [ self delegate handleRequest: request ] ].
	request method = #HEAD
		ifTrue: [ response clearEntity ].
	self logRequest: request response: response started: initialMilliseconds.
	^ response! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/12/2012 19:26'!
initializeServerSocket
	"Set up a new server socket and listen on it"
	
	self log debug: 'Initializing server socket'.
	serverSocket := interface
		ifNil: [ ZnNetworkingUtils serverSocketOn: port ]
		ifNotNil: [ ZnNetworkingUtils serverSocketOn: port interface: interface ]! !

!ZnSingleThreadedServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 21:24'!
isListening
	"Return true when I have a valid server socket listening at the correct port"
	
	^ self serverSocket notNil
		and: [ self serverSocket isValid and: [ self serverSocket localPort = self port ] ]! !

!ZnSingleThreadedServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:57'!
isRunning
	"Return true when I am running"
	
	^ self process notNil and: [ self serverSocket notNil ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:13'!
lastRequest
	"The last request that I received.
	Will only be tracked when logging is enabled"
	
	^ lastRequest ! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:13'!
lastResponse
	"The last response that I sent.
	Will only be tracked when logging is enabled"
	
	^ lastResponse ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/10/2010 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid
			 ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 13:02'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:00'!
logRequest: request
	"Log a request that was just read"
	
	self log info: [ 
		lastRequest := request.
		'Read ', request printString ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:51'!
logRequest: request response: response started: initialMilliseconds
	self log transaction: [
		String streamContents: [ :stream |
			request requestLine printMethodAndUriOn: stream.  
			stream space; print: response code.
			response hasEntity ifTrue: [
				stream space; print: response entity contentLength; nextPut: $B ].  
			stream space; print: (Time millisecondsSince: initialMilliseconds); nextPutAll:  'ms' ] ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 13:00'!
logResponse: response
	"Log a response that was just written"
	
	self log info: [ 
		lastResponse := response.
		'Wrote ', response printString ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/4/2012 20:08'!
logToStandardOutput
	"Setup logging to the OS level standard output stream"

	self log addListener: ZnStandardOutputLogger new  ! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
logToTranscript
	"Setup logging to the Transcript"

	self log addListener: ZnTranscriptLogger new  ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/30/2011 19:15'!
noteAcceptWaitTimedOut
	"The accept wait on a server socket timed out (see #acceptWaitTimeout).
	This can be used as an opportunity to do some periodic tasks."
	
	self log debug: 'Wait for accept timed out'.
	self periodicTasks! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/1/2011 14:07'!
onRequestRespond: block
	"Convenience method to quickly set up a delegate.
	On any incoming request, block will be executed with the request 
	as argument to produce a valid response."
	
	self delegate: (ZnValueDelegate with: block)  ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/30/2011 19:16'!
periodicTasks
	"Every #acceptWaitTimeout seconds this method is called.
	Note that during this time the server is not actively listening for connections."! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:51'!
port
	"Return the integer port number we are (or will be) listening on"
	
	^ port! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:53'!
port: integer
	"Set the port number we will be listening on.
	Cannot be changed after the server is already running."
	
	(self isRunning and: [ port ~= integer ]) 
		ifTrue: [ self error: 'Stop me before changing my port' ].
	port := integer! !

!ZnSingleThreadedServer methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/13/2012 13:12'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream nextPutAll: (self isRunning ifTrue: [ 'running' ] ifFalse: [ 'stopped' ]).
	self port ifNotNil: [ stream space; print: self port ].
	self bindingAddress ifNotNil: [ stream space; print: self bindingAddress ].
	stream nextPut: $)! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:54'!
process
	"Return the process that is running my main listening loop.
	Will be nil when I am not running"
	
	^ process! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/7/2012 17:53'!
readRequest: stream
	"Read a request from stream.
	We add a virtual header containing the remote IP address of the client"

	| request |
	request := self reader value: stream.
	request headers 
		at: ZnConstants remoteAddressHeader
		put: (ZnNetworkingUtils ipAddressToString: stream socket remoteAddress).
	self logRequest: request.
	^ request! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/7/2012 17:52'!
reader
	"Return a block that when given a stream reads an entity from it."
	
	^ reader ifNil: [ 
		reader := [ :stream | ZnRequest readFrom: stream ] ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/7/2012 17:53'!
reader: block
	"Customize how entities are read from a stream, see #reader"
	
	reader := block! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/20/2011 12:11'!
releaseServerSocket
	"Release our server socket"
	
	serverSocket ifNil: [ ^ self ].
	self log debug: 'Releasing server socket'.
	(Delay forMilliseconds: 10) wait.
	serverSocket destroy.
	serverSocket := nil! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/10/2011 13:18'!
serveConnectionOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeOneRequestResponseOn: on it"
		 
	| stream socket |
	socket := (listeningSocket waitForAcceptFor: self acceptWaitTimeout) ifNil: [
		^ self noteAcceptWaitTimedOut ].
	stream := self socketStreamOn: socket.
	[ [ [ self executeOneRequestResponseOn: stream ]
		ensure: [ self log debug: 'Closing stream'. stream close ] ]
			ifCurtailed: [ self log debug: 'Destroying socket'. socket destroy ] ]
				forkAt: Processor highIOPriority
				named: 'Zinc HTTP Server Worker'! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/13/2010 16:42'!
serverProcessName
	^ String streamContents: [ :stream |
		stream nextPutAll: self class name; nextPutAll: ' HTTP port '; print: self port ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:54'!
serverSocket
	"Return the server socket that I am using.
	Will be nil when I am not running"
	
	^ serverSocket! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/10/2011 13:18'!
socketStreamOn: socket
	^ ZnNetworkingUtils socketStreamOn: socket! !

!ZnSingleThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:00'!
start
	"Start me. I will start listening on my port for incoming HTTP connections.
	If I am running, I will first stop and thus effectively restart"
	
	self stop.
	self log info: [ 'Starting ', self serverProcessName ].
	self class default = self ifTrue: [ self register ].
	process := [ [ self listenLoop ] repeat ]
		forkAt: Processor highIOPriority 
		named: self serverProcessName ! !

!ZnSingleThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:20'!
stop: unregister
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	If unregister is true, unregister me from the list of managed instances.
	Does nothing when I am not running"
	
	self isRunning ifFalse: [ ^ self ].
	process terminate. 
	process := nil.
	unregister ifTrue: [ self unregister ].
	self closeDelegate.
	self log info: [ 'Stopped ', self serverProcessName ]! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 10/4/2011 12:35'!
stopLogging
	"Stop doing any logging by disabling our log support"

	self log disable! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/14/2010 14:37'!
writeResponse: response on: stream
	"Write response to stream and flush the stream"
	
	response writeOn: stream.
	stream flush.
	self logResponse: response ! !

Object subclass: #ZnStatusLine
	instanceVariableNames: 'version code reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStatusLine commentStamp: '<historical>' prior: 0!
I am an HTTP Status Line.
I consist of an HTTP protocol version, an integer HTTP status code and an HTTP reason string.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:00'!
badRequest
	^ self code: 400! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:48'!
code: code
	^ self new
		code: code;
		version: ZnConstants defaultHTTPVersion;
		yourself! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:05'!
created
	^ self code: 201! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:10'!
forbidden
	^ self code: 403! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:09'!
found
	^ self code: 302! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:02'!
internalServerError
	^ self code: 500! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:09'!
methodNotAllowed
	^ self code: 405! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:09'!
movedPermanently
	^ self code: 301! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 08:59'!
notFound
	^ self code: 404! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:08'!
notImplemented
	^ self code: 501! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:08'!
notModified
	^ self code: 304! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 08:58'!
ok
	^ self code: 200! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:47'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:07'!
redirect
	^ self code: 302! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:07'!
seeOther
	^ self code: 303! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:02'!
unauthorized
	^ self code: 401! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:49'!
code
	^ code! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:04'!
code: integer
	(ZnConstants httpStatusCodes includesKey: integer) 
		ifFalse: [ 
			(ZnUnknownHttpStatusCode code: integer) signal ].
	code := integer! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 10:57'!
printCodeAndReasonOn: stream
	self code notNil
		ifTrue: [
			stream print: self code; space; nextPutAll: self reason ]! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printOn: stream
	super printOn: stream.
	self code notNil
		ifFalse: [ ^ self ].
	stream nextPut: $(.
	self printCodeAndReasonOn: stream.
	stream nextPut: $)! !

!ZnStatusLine methodsFor: 'initialize-release' stamp: 'DamienPollet 8/4/2011 14:04'!
readFrom: stream
	| line lineStream httpCode |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self version: (lineStream upTo: Character space).
	httpCode := lineStream upTo: Character space.
	httpCode := Integer readFrom: httpCode ifFail: [ (ZnUnknownHttpStatusCode code: code) signal ].
	self code: httpCode.
	self reason: (lineStream upToEnd)
! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:27'!
reason
	reason isNil 
		ifTrue: [ 
			reason := ZnConstants httpStatusCodes at: self code ifAbsent: [ 'Unknown Status Code' ] ].
	^ reason! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 16:29'!
reason: string
	reason := string! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:46'!
version
	^ version! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:05'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknownHttpVersion version: version) signal ] ! !

!ZnStatusLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:40'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self version; space.
	self code printOn: writeStream.
	writeStream space; nextPutAll: self reason; nextPutAll: String crlf ! !

Object subclass: #ZnUrl
	instanceVariableNames: 'scheme host port segments query fragment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUrl commentStamp: 'SvenVanCaekenberghe 1/24/2012 11:48' prior: 0!
I am ZnUrl, an implementation of an interpreted HTTP(S) URL/URI. 
Contrary to other implementations, this one is specific for HTTP(S).

I consist of the following parts:
  - scheme - either #http, #https or nil
  - host - hostname string or nil
  - port - port integer or nil
  - segments - collection of path segments, ends with #/ for directories
  - query - query dictionary or nil
  - fragment - fragment string or nil

The syntax of my external representation informally looks like this

  scheme://host:port/segments?query#fragment

I am most often created by parsing my external representation using either my #fromString: class method or by sending the #asZnUrl convenience method to a String. Using #asZnUrl helps in accepting both Strings and ZnUrls arguments.

  ZnUrl fromString: 'http://www.google.com/search?q=Smalltalk'.

I can also be constucted programmatically.

  ZnUrl new 
    scheme: #https; 
    host: 'encrypted.google.com'; 
    addPathSegment: 'search'; 
    queryAt: 'q' put: 'Smalltalk'; 
    yourself.
  
My components can be manipulated destructively. Here is an example:

  ('http://www.google.com/?one=1&two=2' asZnUrl)
    queryAt: 'three' put: '3';
    queryRemoveKey: 'one';
    yourself.

Some characters of parts of a URL are illegal because they would interfere with the syntax and further processing and thus have to be encoded. The methods in accessing protocols do not do any encoding, those in parsing and printing do. Here is an example:

  ('http://www.google.com' asZnUrl)
    addPathSegment: 'some encoding here';
    queryAt: 'and some encoding' put: 'here, too';
    yourself

My parser is somewhat forgiving and accepts some unencoded URLs as well, like most browsers would.

  'http://www.example.com:8888/a path?q=a, b, c' asZnUrl.

I can parse in the context of a default scheme, like a browser would do.

  ZnUrl fromString: 'www.example.com' defaultScheme: #http

Given a scheme, I know its default port, try #portOrDefault.

A path defaults to what is commonly referred to as slash, test with #isSlash. Paths are most often (but don't have to be) interpreted as filesystem paths. To support this, I have #isFile and #isDirectory tests and #file and #directory accessors.

I have some support to handle one URL in the context of another one, this is also known as a relative URL in the context of an absolute URL. Refer to #isAbsolute, #isRelative and #inContextOf:

  '/folder/file.txt' asZnUrl inContextOf: ('http://fileserver.example.net:4400' asZnUrl).

Sometimes, the combination of my host and port are referred to as authority, see #authority.

URL/URI/URN (Uniform/Universal Resource Locator/Identifier/Name) are closely related and can be and are used as synonyms is many contexts. Refer to http://en.wikipedia.org/wiki/Url for more information.

There is a convenience method #retrieveContents to download the resource a ZnUrl points to,

  'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents.
  
Part of Zinc HTTP Components.!

!ZnUrl class methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:05'!
defaultPortForScheme: scheme
	scheme = #http ifTrue: [ ^ 80 ].
	scheme = #https ifTrue: [ ^ 443 ].
	(ZnUnknownScheme scheme: scheme) signal! !

!ZnUrl class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
fromString: string
	^ (self new)
		parseFrom: string; 
		yourself ! !

!ZnUrl class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/11/2011 14:47'!
fromString: string defaultScheme: defaultScheme
	^ (self new)
		parseFrom: string defaultScheme: defaultScheme; 
		yourself ! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:00'!
= anObject
	^ self class == anObject class 
		and: [ self equals: anObject ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:22'!
addPathSegment: segment
	segments isNil ifTrue: [ 
		segments := OrderedCollection new ].
	segments add: segment! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 8/17/2011 11:32'!
addPathSegments: pathSegments
	pathSegments do: [ :each | self addPathSegment: each ]! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 11:24'!
asZnUrl
	^ self! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/11/2011 12:05'!
asZnUrlWithDefaults
	^ self copy 
		setDefaults; 
		yourself! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 21:01'!
authority
	^ self hasNonDefaultPort 
		ifTrue: [ 
			String streamContents: [ :stream | 
				stream nextPutAll: self host; nextPut: $:; print: self port ] ]
		ifFalse: [ 
			self host ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:59'!
authorityPrintString
	^ String streamContents: [ :stream | 
			self printAuthorityOn: stream ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 11:56'!
closePath
	self isSlash
		ifFalse: [ self addPathSegment: #/ ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:21'!
directory
	^ self isDirectory
		ifTrue: [ self path ]
		ifFalse: [ self joinSegments: segments allButLast ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/3/2011 16:17'!
encode: string on: stream
	stream nextPutAll: string encodeForHTTP 
! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:59'!
equals: url
	self scheme = url scheme ifFalse: [ ^ false ].
	self host = url host ifFalse: [ ^ false ].
	self portOrDefault = url portOrDefault ifFalse: [ ^ false ].
	((self isSlash and: [ url isSlash ]) or: [ self segments = url segments ]) ifFalse: [ ^ false ].
	self query = url query ifFalse: [ ^ false ].
	self fragment = url fragment ifFalse: [ ^ false ].
	^ true! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:15'!
file
	^ self isDirectory
		ifTrue: [ false ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:46'!
firstPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments first ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment
	^ fragment! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment: string
	fragment := string! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasFragment
	^ fragment notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasHost
	^ host notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 21:01'!
hasNonDefaultPort
	^ self hasPort 
		and: [ self port ~= (self class defaultPortForScheme: self schemeOrDefault) ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:03'!
hasPath
	^ segments notNil and: [ segments isEmpty not ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasPort
	^ port notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:47'!
hasQuery
	^ query notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasScheme
	^ scheme notNil! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:43'!
hash
	^ self hasPath
		ifTrue: [ segments inject: host hash into: [ :sum :each | sum bitXor: each hash ] ]
		ifFalse: [ host hash ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
host
	^ host! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:41'!
host: hostName
	host := hostName asLowercase ! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 10/4/2011 15:38'!
inContextOf: absoluteUrl
	"Return a copy of the receiver where scheme, host and port
	are taken from absoluteUrl unless they are already in the receiver"
	
	| copy |
	copy := self copy.
	copy hasScheme ifFalse: [ copy scheme: absoluteUrl scheme ].
	copy hasHost ifFalse: [ copy host: absoluteUrl host ].
	(copy hasPort not and: [ copy scheme = absoluteUrl scheme ]) 
		ifTrue: [ copy port: absoluteUrl port ].      
	^ copy! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:55'!
isAbsolute
	^ self hasScheme! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:57'!
isDirectory
	^ self hasPath 
		ifTrue: [ segments last = #/ ]
		ifFalse: [ true ]
	! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 20:06'!
isEmpty
	^ (self hasScheme | self hasHost | self hasPath | self hasQuery | self hasFragment) not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:56'!
isFile
	^ self isDirectory not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttp
	^ scheme == #http! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttps
	^ scheme == #https! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/13/2011 10:41'!
isLocalHost
	^ self hasHost and: [ #('localhost' '127.0.0.1') includes: self host ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:55'!
isRelative
	^ self isAbsolute not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:54'!
isSlash
	^ self hasPath not or: [ segments size = 1 and: [ segments first = #/ ] ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:11'!
joinSegments: collection
	^ String streamContents: [ :stream |
		collection 
			do: [ :each | stream nextPutAll: each ]
			separatedBy: [ stream nextPut: $/ ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
lastPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/11/2011 15:08'!
parseAuthority: stream
	| hostString |
	(hostString := stream upTo: $:) isEmpty 
		ifFalse: [  
			self host: (ZnUtils decodePercent: hostString) ].
	stream atEnd 
		ifFalse: [ 
			self port: (Integer readFrom: stream ifFail: [ ZnPortNotANumber signal ]) ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/11/2011 14:47'!
parseFrom: string
	self parseFrom: string defaultScheme: nil! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/11/2011 15:00'!
parseFrom: string defaultScheme: defaultScheme
	| start end index |
	start := 1.
	end := string size.
	(index := string indexOf: $#) > 0
		ifTrue: [ 
			self fragment: (ZnUtils decodePercent: (string copyFrom: index + 1 to: end)).
			end := index - 1 ].
	(index := string indexOf: $?) > 0
		ifTrue: [
			self query: (ZnUtils parseQueryFrom: (ReadStream on: string from: index + 1 to: end)).
			end := index - 1 ].
	(index := string indexOfSubCollection: '://') > 0
		ifTrue: [
			self scheme: (string copyFrom: 1 to: index - 1).
			start := index + 3 ]
		ifFalse: [ 
			defaultScheme ifNotNil: [ self scheme: defaultScheme ] ].
	self hasScheme 
		ifTrue: [  
			(index := string indexOf: $/ startingAt: start) > 0
				ifTrue: [
					self parseAuthority: (ReadStream on: string from: start to: index - 1).
					start := index ]
				ifFalse: [ 
					^ self parseAuthority: (ReadStream on: string from: start to: end) ] ].
	self parsePath: (ReadStream on: string from: start to: end)! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:53'!
parsePath: stream
	stream peekFor: $/.
	[ stream atEnd ] whileFalse: [ | segment |
		segment := String streamContents: [ :stringStream |
			[ stream atEnd not and: [ stream peek ~= $/ ] ] whileTrue: [
				stringStream nextPut: stream next ] ].
		(segment isEmpty or: [ segment = '.' ])
			ifFalse: [
				segment = '..' 
					ifTrue: [ self removeLastPathSegment ]
					ifFalse: [ self addPathSegment: (ZnUtils decodePercent: segment) ] ].
		((stream peekFor: $/) and: [ stream atEnd ])
			ifTrue: [ self closePath ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/3/2011 22:21'!
path
	^ self joinSegments: segments! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:16'!
pathPrintString
	^ String streamContents: [ :stream | 
			self printPathOn: stream ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:17'!
pathQueryFragmentPrintString
	^ String streamContents: [ :stream | 
			self printPathQueryFragmentOn: stream ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
pathSegments
	^ segments! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
port
	^ port! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:14'!
port: integer
	port := integer! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/11/2011 14:37'!
portOrDefault
	^ self hasPort
		ifTrue: [ self port ]
		ifFalse: [ self class defaultPortForScheme: self schemeOrDefault ]! !

!ZnUrl methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 4/26/2011 12:34'!
postCopy
	super postCopy.
	segments := segments copy.
	query := query copy! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:58'!
printAuthorityOn: stream
	self hasHost ifTrue: [ 
		stream nextPut: $/; nextPut: $/. 
		self encode: self host on: stream ].
	self hasPort ifTrue: [ 
		stream nextPut: $:; print: self port ]
! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 15:10'!
printOn: stream
	self hasScheme ifTrue: [ 
		stream nextPutAll: self scheme; nextPut: $: ].
	self printAuthorityOn: stream.
	self printPathQueryFragmentOn: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 4/17/2011 10:27'!
printPathOn: stream
	segments isNil ifTrue: [ ^ stream nextPut: $/ ].
 	segments do: [ :each | 
		stream nextPut: $/.
		each == #/
			ifFalse: [ 
				self encode: each on: stream ] ] ! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printPathQueryFragmentOn: stream
	self printPathOn: stream.
	self hasQuery
		ifTrue: [ self printQueryOn: stream ].
	self hasFragment
		ifFalse: [ ^ self ].
	stream nextPut: $#.
	self encode: self fragment on: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:22'!
printQueryOn: stream
	stream nextPut: $?. 
	ZnUtils writeQueryFields: self query on: 	stream! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 16:26'!
query
	^ query! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 20:15'!
query: dictionary
	query := dictionary! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/12/2011 11:26'!
queryAddAll: keyedCollection
	"Note that we use #at:add:"
	
	keyedCollection keysAndValuesDo: [ :key :value |
		self queryAt: key add: value ].
	^ keyedCollection ! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/29/2012 19:29'!
queryAt: key
	^ self hasQuery
		ifTrue: [ self query at: key ]
		ifFalse: [ KeyNotFound signalFor: key printString ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/12/2011 11:21'!
queryAt: key add: value
	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key add: value! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
queryAt: key ifAbsent: block
	^ self hasQuery
		ifTrue: [ self query at: key ifAbsent: block ]
		ifFalse: [ block value ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/12/2011 11:21'!
queryAt: key put: value
	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key put: value! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 1/31/2011 12:18'!
queryDo: block
	self hasQuery ifTrue: [ 
		self query keysAndValuesDo: block ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 12/13/2011 14:07'!
queryKeys
	^ self hasQuery
		ifTrue: [ self query keys ]
		ifFalse: [ #() ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 8/18/2011 14:11'!
queryRemoveKey: key
	self hasQuery
		ifTrue: [ self query removeKey: key ifAbsent: [ ] ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 11:55'!
removeLastPathSegment
	segments isEmpty 
		ifFalse: [ segments removeLast ]! !

!ZnUrl methodsFor: 'downloading' stamp: 'SvenVanCaekenberghe 1/24/2012 11:46'!
retrieveContents
	"Download and return the resource that I refer to.
	This will typically return a String or a ByteArray (see ZnClient>>#get:).
	If something goes wrong, an exception will be signalled."
	
	" 'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents. "
	
	^ ZnClient new
		beOneShot;
		enforceHttpSuccess: true;
		get: self! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
scheme
	^ scheme! !

!ZnUrl methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:05'!
scheme: symbol
	symbol isNil 
		ifTrue: [ scheme := nil ]
		ifFalse: [ 
			(#(http https) includes: symbol) 
				ifTrue: [ scheme := symbol asSymbol ]
				ifFalse: [ (ZnUnknownScheme scheme: symbol) signal ] ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/11/2011 11:05'!
schemeOrDefault
	^ self hasScheme 
		ifTrue: [ self scheme ]
		ifFalse: [ #http ]    ! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 10:56'!
segments
	^ segments! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 13:22'!
segments: collection
	segments := collection! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/11/2011 12:04'!
setDefaults
	self hasScheme 
		ifFalse: [ self scheme: self schemeOrDefault ].
	self hasPort 
		ifFalse: [ self port: self portOrDefault ]    ! !

Object subclass: #ZnUserAgent
	instanceVariableNames: 'session settings request response log errorHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Deprecated'!

!ZnUserAgent commentStamp: 'SvenVanCaekenberghe 10/4/2011 19:02' prior: 0!
I implement a client for making HTTP requests. I can maintain session state across multiple requests, automatically follow redirects and notify observers of my request and response activity. The contents of my request headers can be modified easily.

DEPRECATED: Please use ZnNeoClient

Much of my behavior can be controlled with an instance of ZnUserAgentSettings.

Some examples of my use:

client := ZnUserAgent new.
response := client get: 'pharo-project.org'.
args := (Dictionary new) at: 'foo' put: 'bar'; yourself.
response := client post: 'mytestdomain.com/form' data: args.!

ZnUserAgent subclass: #ZnHttpClient
	instanceVariableNames: 'url params headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Deprecated'!

!ZnHttpClient commentStamp: 'SvenVanCaekenberghe 10/4/2011 19:02' prior: 0!
I implement an HTTP client with a high level API.

DEPRECATED: Please use ZnNeoClient

Some examples of my use:

response := ZnHttpClient new
	url: 'somedomain.com';
	auth: 'joe' password: 'secret';
	parameterAt: 'name' put: 'joe';
	headerAt: 'X-Custom Header' put: 'woot';
	get.
	 !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
auth: username password: password
	| cred |
	cred := ZnBasicCredential new
				username: username;
				password: password;
				domain: self url authority;
				yourself.
	self session addCredential: cred! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
contentLength
	^ self response contentLength! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
contentType
	^ self response contentType! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
cookieAt: aName
	^ (super cookieAt: aName) value! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 13:29'!
delete
	self 
		sendMethod: [ self method: #DELETE for: self url headers: self headers data: self params ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:36'!
digestAuth: username password: password
	| cred |
	cred := ZnDigestCredential new
				username: username;
				password: password;
				domain: self url authority;
				yourself.
	self session addCredential: cred! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/20/2011 20:36'!
get
	self url queryAddAll: self params.
	self 
		sendMethod: [ self method: #GET for: self url headers: self headers data: nil ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/20/2011 20:36'!
head
	self url queryAddAll: self params.
	self 
		sendMethod: [ self method: #HEAD for: self url headers: self headers data: nil ];
		resetParameters.
	^ self response! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:27'!
headerAt: aName add: aValue
	self headers at: aName add: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:17'!
headerAt: aName put: aValue
	self headers at: aName put: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:27'!
headers
	^ headers ifNil: [ headers := ZnMultiValueDictionary new ]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
headers: anObject
	headers := anObject! !

!ZnHttpClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
isAuthenticationRequired
	^ self response isAuthenticationRequired ! !

!ZnHttpClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:13'!
linkCredentials
	(self session credentialsForUrl: self url) do: [ :cred |
		cred linkToAuthRequest: self response wwwAuthenticate ]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
parameterAt: aName
	^ self params at: aName ifAbsent: [ nil ]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:28'!
parameterAt: aName add: aValue
	self params at: aName add: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:15'!
parameterAt: aName put: aValue
	self params at: aName put: aValue! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:27'!
params
	^ params ifNil: [ params := ZnMultiValueDictionary new ]! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
params: anObject
	params := anObject! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 13:30'!
post
	self 
		sendMethod: [ self method: #POST for: self url headers: self headers data: self params ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 1/12/2011 13:30'!
put
	self 
		sendMethod: [ self method: #PUT for: self url headers: self headers data: self params ];
		resetParameters.
	^ self response contents! !

!ZnHttpClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/12/2011 10:16'!
resetParameters
	params ifNotNil: [ params removeAll ]! !

!ZnHttpClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:14'!
sendMethod: aBlock
	aBlock on: Error do: self errorHandler.
	self isAuthenticationRequired ifTrue: [
		self linkCredentials.
		aBlock on: Error do: self errorHandler ].
	self resetParameters ! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
status
	^ self response code! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:10'!
url
	^ url! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 14:19'!
url: anObject
	url := anObject asZnUrl! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
userAgent
	^ self settings agent! !

!ZnHttpClient methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 20:12'!
userAgent: aString
	self settings agent: aString! !

!ZnUserAgent class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/4/2012 10:32'!
new
	self deprecated: 'Please use ZnEasy' on: '8 Nov 2011' in: 'Zinc HTTP Components'.
	^ super new! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:00'!
cookieAt: aName
	"Answers the named cookie for the current request or nil if non exists."
	
	^ self session cookieJar cookieAt: aName forUrl: self request url! !

!ZnUserAgent methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/29/2011 14:47'!
defaultErrorHandler
	^ [ :error | 
			self log debug: [ error description ]. 
			error defaultAction ]! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
delete: urlObject
	^ self delete: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
delete: urlObject headers: headerCollection
	^ self method: #DELETE for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
errorHandler
	^ errorHandler ifNil: [ errorHandler := self defaultErrorHandler ]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 10/1/2010 13:45'!
errorHandler: aBlock
	errorHandler := aBlock! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
get: urlObject
	^ self get: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
get: urlObject headers: headerCollection
	^ self method: #GET for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
head: urlObject
	^ self head: urlObject headers: nil! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
head: urlObject headers: headerCollection
	^ self method: #HEAD for: urlObject headers: headerCollection data: nil ! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:41'!
log
	^ log ifNil: [ log := ZnLogSupport new ]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 14:41'!
logProgressToTranscript
	self logToTranscript! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/29/2011 15:40'!
logToTranscript
	self log addListener: ZnTranscriptLogger new    ! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
method: aMethod for: urlObject headers: headerCollection data: dataEntity
	| limit |
	limit := self settings followRedirect ifTrue: [ self settings redirectLimit ] ifFalse: [ 0 ].
	^ self method: aMethod for: urlObject headers: headerCollection data: dataEntity limit: limit
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/20/2011 21:10'!
method: method for: url headers: headerCollection data: dataEntity limit: limit
	| stream redirects continue |
	redirects := 0.
	continue := true.
	self prepareRequest: url method: method headers: headerCollection data: dataEntity.
	[ [ continue ] whileTrue: [
			stream := self openConnection.
			request setConnectionClose.
			[ self sendRequest: stream ] ensure: [ stream close ].
			(self response isRedirect and: [ redirects < limit ])
				ifTrue: [ self prepareRedirect: headerCollection. redirects := redirects + 1 ]
				ifFalse: [ continue := false ] ] ]
		on: Error
		do: self errorHandler.
	^ self response! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/20/2011 19:36'!
openConnection
	| socketStream |
	socketStream := ZnNetworkingUtils socketStreamToUrl: request url.
	self settings timeout 
		ifNotNil: [ :timeout | socketStream timeout: timeout ].
	^ socketStream! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
post: urlObject data: dataEntity
	^ self post: urlObject headers: nil data: dataEntity! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
post: urlObject headers: headerCollection data: dataEntity
	^ self method: #POST for: urlObject headers: headerCollection data: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 8/18/2011 14:12'!
prepareCredentials: headers for: urlObject method: method
	(self session credentialsForUrl: urlObject) do: [ :cred |
		cred isComplete ifTrue: [
			[ cred setAuthorizationHeader: headers for: urlObject method: method ]
				on: Error
				do: self errorHandler ] ]
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/12/2011 10:12'!
prepareData: dataEntity
	"Prepare the ZnEntity for the request."
	
	dataEntity ifNil: [ ^ dataEntity ].
	(dataEntity isKindOf: ZnEntity) ifTrue: [ ^ dataEntity ].
	^ ZnApplicationFormUrlEncodedEntity withAll: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 8/30/2011 22:00'!
prepareHeaders: headerCollection for: urlObject method: method
	| headers |
	headers := (ZnHeaders requestHeadersFor: urlObject)
					at: 'User-Agent' put: self settings agent;
					yourself.
	headerCollection isNil 
		ifFalse: [ headers addAll: headerCollection ].
	(self session cookieJar cookiesForUrl: urlObject) do: [ :cookie |
		headers at: 'Cookie' add: cookie nameValueString ].
	self prepareCredentials: headers for: urlObject method: method.
	^ headers
	! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 8/31/2011 11:36'!
prepareRedirect: headerCollection
	"We implement the common behavior that POST/PUT redirects turn into GET request.
	Note also that the body data is dropped"
	
	| method |
	(#(POST PUT) includes: request method)
		ifTrue: [ method := #GET ]
		ifFalse: [ method := self request method ].
	self prepareRequest: self redirectUrl method: method headers: headerCollection data: nil! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/4/2011 14:21'!
prepareRequest: urlObject method: method headers: headerCollection data: dataEntity
	| url |
	url := urlObject asZnUrl.
	request := ZnRequest new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (self prepareHeaders: headerCollection for: url method: method);
		entity: (self prepareData: dataEntity);
		yourself! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 8/30/2011 22:00'!
processResponse: stream
	| cookies |
	response := (request method = #HEAD) 
		ifTrue: [ ZnResponse readHeaderFrom: stream ]
		ifFalse: [ ZnResponse readFrom: stream ].
	self log debug: [ response printString ].
	cookies := response headers at: 'Set-Cookie' ifAbsent: [ ^ response ].
	cookies := (cookies isCollection & cookies isString) ifTrue: [ Array with: cookies ] ifFalse: [ cookies ].
	cookies do: [ :cookie | 
		self session cookieJar add: (ZnCookie fromString: cookie for: self request url) ].
	^ response ! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:23'!
put: urlObject data: dataEntity
	^ self put: urlObject headers: nil data: dataEntity! !

!ZnUserAgent methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:22'!
put: urlObject headers: headerCollection data: dataEntity
	^ self method: #PUT for: urlObject headers: headerCollection data: dataEntity! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 1/20/2011 21:12'!
redirectUrl
	"We support relative locations although they are not the standard (http://en.wikipedia.org/wiki/HTTP_location)"
	
	| redirectUrl |
	redirectUrl := self response location asZnUrl.
	redirectUrl isAbsolute
		ifFalse: [ | baseUrl |
			baseUrl := self request url.
			redirectUrl 
				scheme: baseUrl scheme;
				host: baseUrl host;
				port: baseUrl port ].
	^ redirectUrl
! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
request
	^ request! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:09'!
request: znRequest
	request := znRequest! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/24/2010 17:24'!
response
	^ response! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:09'!
response: znResponse
	response := znResponse! !

!ZnUserAgent methodsFor: 'private-protocol' stamp: 'SvenVanCaekenberghe 3/29/2011 14:46'!
sendRequest: stream
	self log debug: [ request printString ].
	request writeOn: stream.
	stream flush.
	^ self processResponse: stream! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 11:17'!
session
	^ session ifNil: [ session := self settings loadSession ]! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:10'!
session: userAgentSession
	session := userAgentSession! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:11'!
settings
	settings ifNil: [ settings := ZnUserAgentSettings defaultSettingsClass new ].
	^ settings! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:10'!
settings: userAgentSettings
	settings := userAgentSettings! !

!ZnUserAgent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:47'!
url
	^ self request url! !

Object subclass: #ZnUserAgentSession
	instanceVariableNames: 'cookiejar credentials currentRealm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUserAgentSession commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I represent a session store for an HTTP client instance of ZnUserAgent. 

I hold the cookie jar for the session and any authentication credentials used by the client.

Part of Zinc HTTP Components.!

!ZnUserAgentSession class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
withCookieJar: aCookieJar
	^ self new cookiejar: aCookieJar; yourself! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:31'!
addCredential: aCredential
	self credentials at: aCredential domain, '@', aCredential realm put: aCredential ! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:00'!
cookieJar
	^ cookiejar ifNil: [ cookiejar := ZnCookieJar new ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
cookiejar: anObject
	cookiejar := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
credentialAt: aRealm forUrl: urlObject
	^ self credentials at: urlObject authority, '@', aRealm ifAbsent: [ nil ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:14'!
credentials
	^ credentials ifNil: [ credentials := Dictionary new ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
credentials: anObject
	credentials := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
credentialsForUrl: urlObject
	^ self credentials values select: [ :cred | urlObject authority includesSubString: (cred domain) ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm
	^ currentRealm! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm: anObject
	currentRealm := anObject! !

!ZnUserAgentSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 2/29/2012 17:40'!
resetCookies
	cookiejar ifNotNil: [ cookiejar removeAll ]! !

Object subclass: #ZnUserAgentSettings
	instanceVariableNames: 'settings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Deprecated'!

!ZnUserAgentSettings commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I manage the settings for a ZnUserAgent instance. 

I can be subclassed to extend capabilities to include loading settings from a configuration store or maintaining persistent storage for a cookie jar.

Part of Zinc HTTP Components.!

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
commentString
	^ '(', self platform platformName, '; ', self encryptionStrengthComment, '; ', self platformDetails, '; ', self localeComment, ')'! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2010 00:00'!
defaultCookieJar
	^ ZnCookieJar ! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
defaultSessionClass
	^ ZnUserAgentSession! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 21:00'!
defaultSettingsClass
	^ self ! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
defaultUserAgentString
	^ ZnConstants frameworkNameAndVersion, ' ', self commentString, ' ',  self platform version asString! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
encryptionStrengthComment
	"For user agent comment. Standard values are U, I, or N. Currently returning N until standardize https transport is available."
	
	^ 'N'! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
localeComment
	^ LocaleID current isoString! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/6/2010 09:32'!
platform
	^ SmalltalkImage current os! !

!ZnUserAgentSettings class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:25'!
platformDetails
	^ self platform platformName, ' ', self platform platformSubtype, ' ', self platform osVersion! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:15'!
agent
	^ settings at: #agent ifAbsent: [ nil ]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:38'!
agent: aString
	settings at: #agent put: aString ! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:15'!
followRedirect
	^ settings at: #followRedirect ifAbsent: [ nil ]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:39'!
followRedirect: aBoolean
	settings at: #followRedirect put: aBoolean! !

!ZnUserAgentSettings methodsFor: 'initialize-release' stamp: 'MattKennedy 9/26/2010 16:12'!
initialize
	super initialize.
	self settings at: #agent put: self class defaultUserAgentString.
	self followRedirect: true.
	self redirectLimit: 7.
	self timeout: 180! !

!ZnUserAgentSettings methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
loadSession
	^ self class defaultSessionClass withCookieJar: self class defaultCookieJar new! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:14'!
redirectLimit
	^ settings at: #redirectLimit ifAbsent: [ nil ]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:39'!
redirectLimit: anInteger
	settings at: #redirectLimit put: anInteger! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:14'!
settings
	^ settings isNil ifTrue: [ settings := Dictionary new ] ifFalse: [ settings ]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'MattKennedy 9/25/2010 21:30'!
settings: anObject
	settings := anObject! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:15'!
timeout
	^ settings at: #timeout ifAbsent: [ nil ]! !

!ZnUserAgentSettings methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:39'!
timeout: anInteger
	settings at: #timeout put: anInteger! !

Object subclass: #ZnUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUtils commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnUtils holding various utilities related to HTTP handling.

Part of Zinc HTTP Components.!

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/10/2010 20:14'!
capitalizeString: string
	| capitalExpected |
	capitalExpected := true.
	^ String new: string size streamContents: [ :stream |
		string do: [ :each |
			each isAlphaNumeric
				ifTrue: [
					capitalExpected 
						ifTrue: [ 
							stream nextPut: each asUppercase.
							capitalExpected := false. ]
						ifFalse: [ 
							stream nextPut: each asLowercase ] ] 
		 		ifFalse: [ 
					stream nextPut: each.
					capitalExpected := true ] ] ]! !

!ZnUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 13:10'!
currentProcessID
	"Return a integer of at most 6 digits that can be used as a description for the current active process.
	This is a reduced hash that is not necessarily unique (but uniqueness is very probably)"
	
	^ Processor activeProcess hash \\ 999999! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 3/21/2011 20:11'!
decodeBase64: string
	^ string base64Decoded ! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/28/2011 15:51'!
decodePercent: string
	^ [ string unescapePercents ]
		on: Error 
		do: [ (ZnPercentEncodingWrong string: string) signal ]  ! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 4/26/2011 18:58'!
encodeBase64: string
	^ (Base64MimeConverter respondsTo: #mimeEncode:multiLine:)
		ifTrue: [ 
			(Base64MimeConverter mimeEncode: string readStream multiLine: false) contents ]	
		ifFalse: [ 
			(Base64MimeConverter mimeEncode: string readStream) contents 
				reject: [ :each | each = Character cr ] ]! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/16/2010 15:52'!
httpDate
	^ self httpDate: DateAndTime now! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 6/28/2011 10:55'!
httpDate: object
	"Answers a string formatting object as time stamp for HTTP1.1 (RFC 1123). 
	Eg. Sun, 06 Nov 1994 08:49:37 GMT"

	| utc |
	utc := object asTimeStamp asUTC.
	^ String streamContents: [ :stream |
		stream nextPutAll: utc dayOfWeekAbbreviation; nextPut: $,; space.
		stream nextPutAll: (utc dayOfMonth printStringLength: 2 padded: true).
		stream space; nextPutAll: utc monthAbbreviation; space.
		utc year printOn: stream.
		stream space.
		utc printHMSOn: stream.
		stream nextPutAll: ' GMT' ]! !

!ZnUtils class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:15'!
isCapitalizedString: string
	| capitalExpected |
	capitalExpected := true.
	string do: [ :each |
		each isAlphaNumeric
			ifTrue: [
				((capitalExpected and: [ each isUppercase ])
					or: [ capitalExpected not and: [ each isLowercase ] ])
					ifTrue: [ capitalExpected := false ]
					ifFalse: [ ^ false ] ] 
		 	ifFalse: [ capitalExpected := true ] ].
	^ true



! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 8/18/2011 14:15'!
nextPutAll: collection on: stream chunked: size
	| written |
	written := 0.
	[ written < collection size ] whileTrue: [ | chunkSize |
		chunkSize := size min: (collection size - written).
		chunkSize printOn: stream base: 16.
		stream nextPutAll: String crlf.
		chunkSize timesRepeat: [ stream nextPut: (collection at: (written := written + 1)) ].
		stream nextPutAll: String crlf ].
	stream nextPutAll: String crlf! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SeanDeNigris 5/10/2012 12:19'!
parseHttpDate: string
	"self parseHttpDate: 'Tue, 13 Sep 2011 08:04:49 GMT'."
	"self parseHttpDate: 'Tue, 13-Sep-2011 08:04:51 GMT'."
	"self parseHttpDate: 'Tue Jan 01 00:00:01 2036 GMT'."
	
	| tokens day month year hour minute second months map yearToken |
	tokens := (string findTokens: #( $ $- $: $, )) allButFirst.
	tokens last = 'GMT' ifTrue: [ tokens := tokens allButLast ].
	months := #(jan feb mar apr may jun jul aug sep oct nov dec).
	(Integer readFrom: tokens first ifFail: [ 0 ]) isZero
		ifTrue: [ map := #(2 1 6 3 4 5) ]
		ifFalse: [ map := #(1 2 3 4 5 6) ].
	day := Integer readFrom: (tokens at: map first) ifFail: [ 0 ].
	month := months indexOf: (((tokens at: map second) asLowercase) copyFrom: 1 to: 3) ifAbsent: [ 0 ].
	yearToken := tokens at: map third.
	year := yearToken isAllDigits
		ifTrue: [
			year := yearToken asNumber.
			yearToken size = 2
				ifTrue: [
					year < 50
						ifTrue: [ year + 2000 ]
						ifFalse: [ year + 1900 ] ]
				ifFalse: [ year ] ]
		ifFalse: [ 0 ].
	hour := Integer readFrom: (tokens at: map fourth) ifFail: [ 0 ].
	minute := Integer readFrom: (tokens at: map fifth) ifFail: [ 0 ].
	second := Integer readFrom: (tokens at: map sixth) ifFail: [ 0 ].
	^ TimeStamp year: year month: month day: day hour: hour minute: minute second: second offset: Duration zero! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/28/2011 15:52'!
parseQueryFrom: stream
	| string index fields |
	fields := ZnMultiValueDictionary new.
	[ stream atEnd ] whileFalse: [
		string := stream upTo: $&.
		index := string indexOf: $=.
		index > 0
			ifFalse: [ fields at: (self decodePercent: string) put: nil ]
			ifTrue: [
				fields
					at: (self decodePercent: (string copyFrom: 1 to: index - 1))
					add: (self decodePercent: (string copyFrom: index + 1 to: string size)) ] ].
	^ fields! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 1/31/2012 12:55'!
readUpToEnd: inputStream limit: limit
	| species bufferSize buffer totalRead |
	bufferSize := limit ifNil: [ 4096 ] ifNotNil: [ 4096 min: limit ].
	species := inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ].
	buffer := species new: bufferSize.
	totalRead := 0.
	^ species streamContents: [ :out |
		[ inputStream atEnd ] whileFalse: [ | readCount |
			readCount := inputStream readInto: buffer startingAt: 1 count: bufferSize.
			(limit notNil and: [ (totalRead := totalRead + readCount) > limit ])
				ifTrue: [ ZnEntityTooLarge signal ].
			out next: readCount putAll: buffer startingAt: 1 ] ]! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 4/4/2012 13:42'!
streamFrom: inputStream to: outputStream size: totalSize
	| bufferSize buffer leftToRead |
	bufferSize := 16384 min: totalSize.
	buffer := (inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: bufferSize.
	leftToRead := totalSize.
	[ leftToRead > 0 ]
		whileTrue: [ | readCount |
			readCount := bufferSize min: leftToRead.
			inputStream next: readCount into: buffer.
			leftToRead := leftToRead - readCount.
			outputStream next: readCount putAll: buffer startingAt: 1.
			leftToRead > 0 ifTrue: [ outputStream flush ] ]! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 10/5/2010 21:24'!
trimString: string
	"Until String>>#trimBoth is available everywhere"
	
	| left right |
	left := 1.
	right := string size.
	[ left <= right and: [ (string at: left) isSeparator ] ]
		whileTrue: [ left := left + 1 ].
	[ left <= right and: [ (string at: right) isSeparator ] ]
		whileTrue: [ right := right - 1 ].
	^ string copyFrom: left to: right! !

!ZnUtils class methodsFor: 'conversions' stamp: 'NorbertHartl 12/6/2011 17:34'!
writeQueryFields: dictionary on: stream
	self writeQueryFields: dictionary withTextEncoding: 'utf-8' on: stream ! !

!ZnUtils class methodsFor: 'conversions' stamp: 'NorbertHartl 12/6/2011 17:34'!
writeQueryFields: dictionary withTextEncoding: encoding on: stream
	| first |
	first := true.
	dictionary keysAndValuesDo: [ :key :value |
		first ifFalse: [ stream nextPut: $& ].
		first := false.
		stream nextPutAll: (key encodeForHTTPWithTextEncoding: encoding).
		value isNil ifFalse: [
			stream 
				nextPut: $=;
				nextPutAll: (value encodeForHTTPWithTextEncoding: encoding) ] ]! !

Object subclass: #ZnValueDelegate
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnValueDelegate commentStamp: 'SvenVanCaekenberghe 12/7/2010 16:14' prior: 0!
I am ZnValueDelegate.
I forward #handleRequest: messages to the object that I wrap using #value:

ZnServer default delegate: 
	(ZnValueDelegate with: [ :request | 
		ZnResponse ok: (ZnEntity with: 'You asked for ', request uri printString) ] ).

Part of Zinc HTTP Components.!

!ZnValueDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
with: object
	^ (super basicNew) object: object; yourself! !

!ZnValueDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 16:16'!
handleRequest: request
	^ object value: request! !

!ZnValueDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
object: anObject
	object := anObject! !
ZnByteEncoder initialize!
ZnConstants initialize!
ZnMimeType initialize!
ZnNetworkingUtils initialize!
ZnServer initialize!
ZnSingleThreadedServer initialize!
